#--------------------- Progress Gauge --------------------------#
proc gauge_create {win {color ""}} {
    frame $win -class Gauge
    
    set len [option get $win length Length]
    canvas $win.display -borderwidth 0 -background white \
        -highlightthickness 0 -width $len -height 20
    pack $win.display -expand yes
    
    if {$color == ""} {
        set color [option get $win color Color]
    }
    $win.display create rectangle 0 0 0 20 \
        -outline "" -fill $color -tags bar
    $win.display create text [expr 0.5*$len] 10 \
        -anchor c -text "0%" -tags value
    return $win
}

proc gauge_value {win val} {
    if {$val < 0 || $val > 100} {
        error "bad value \"$val\": should be 0-100"
    }
    
    set msg [format "%3.0f%%" $val]
    $win.display itemconfigure value -text $msg
    
    set w [expr 0.01*$val*[winfo width $win.display]]
    set h [winfo height $win.display]
    $win.display coords bar 0 0 $w $h
    
    update
}

proc gauge_gui_add {title} {
    global gaugevar
    
    if {![info exists gaugevar(count)]} {
        gauge_init
    }
    
    set win ".progauge"

    if {$gaugevar(count) == 0} {


	toplevel $win
        wm title $win "Progress Gauge"
        set gaugevar(cancel) 0
        
        button $win.cancel -text "Cancel" \
            -command {
                        global gaugevar
                        set gaugevar(cancel) 1
                    }
        pack $win.cancel -side bottom -expand no -fill none -padx 10 -pady 10
    
	set x [expr ([winfo screenwidth .] - [winfo reqwidth $win])*4/5]
	set y [expr ([winfo screenheight .] - [winfo reqheight $win])/2]
	wm geometry $win "+$x+$y"
    }
    
    if {$gaugevar(count) < 5} {

        incr gaugevar(count)
        set num $gaugevar(count)

        label $win.title$num -text $title
        pack $win.title$num -side top -expand yes -fill x -padx 4 -anchor w

        gauge_create $win.gauge$num
        pack $win.gauge$num -side top -expand yes -fill both -padx 10 -pady 10

        frame $win.sep$num -height 2 -borderwidth 1 -relief sunken
        pack $win.sep$num -side top -fill x -pady 4

    } else {
        set num 0
    }
    
    update
   
    return $num
}

proc gauge_gui_delete {num} {
    global gaugevar

    if {$num != $gaugevar(count)} {
        error "attempting to delete gauge $val: should delete gauge $gaugevar(count) first"
    }
        
    set win ".progauge"

    destroy $win.title$num
    destroy $win.gauge$num
    destroy $win.sep$num

    if {$gaugevar(count) > 1} {
        incr gaugevar(count) -1
    } else {
        set gaugevar(count) 0
        destroy $win
    }
    
    update
}

proc gauge_gui_update {num value} {
    global gaugevar

    if {$num < 1 || $num > $gaugevar(count)} {
        error "bad gauge number \"$num\": should be 0-$gaugevar(count)"
    }
    
    if {$value < 0} {
        set value 0
    } elseif {$value > 100} {
        set value 100
    }
        
    set win ".progauge"
    
    gauge_value $win.gauge$num $value
    
    return $gaugevar(cancel)
}


proc gauge_init {} {
    global gaugevar
    
    set gaugevar(count) 0
    set gaugevar(cancel) 0

    option add *Gauge.borderWidth 2 widgetDefault
    option add *Gauge.relief sunken widgetDefault
    option add *Gauge.length 200 widgetDefault
    option add *Gauge.color gray widgetDefault

}
