#--------------------- Dialog Box --------------------------#

proc dialog_create { class {win "auto"} } {
  if {$win == "auto"} {
  	set count 0
	set win ".dialog[incr count]"
	
	while {[winfo exists $win]} {
		set win ".dialog[incr count]"
	}
  }

  toplevel $win -class $class -background LightGray
  
  frame $win.info -background LightGray
  pack $win.info -expand yes -fill both -padx 2 -pady 2
  
  frame $win.sep -height 2 -borderwidth 1 -relief sunken -background LightGray
  pack $win.sep -fill x -pady 4
  
  frame $win.controls -background LightGray
  pack $win.controls -fill x -padx 4 -pady 4
  
  wm title $win $class
  wm group $win .
  
  after idle [format {
    update idletasks
    wm minsize %s [winfo reqwidth %s] [winfo reqheight %s]
  } $win $win $win]
  
  return $win
}

proc dialog_info { win } {
	return "$win.info"
}

proc dialog_controls { win } {
	return "$win.controls"
}

proc dialog_wait { win varName } {
    dialog_safeguard $win
    # set x [expr [winfo rootx .] + 250]
    # set y [expr [winfo rooty .] + 450]

    set x [expr ([winfo screenwidth .] - [winfo reqwidth $win])/2]
    set y [expr ([winfo screenheight .] - [winfo reqheight $win])/2]
    
    wm geometry $win "+$x+$y"

    wm deiconify $win

    set graberr 0
    
    if [catch {grab set $win} result] {
	set graberr 1
	bgerror $result
    }
    
    vwait $varName
    
    if {$graberr == 0} {
	grab release $win
    }
    
    wm withdraw $win
}

bind modalDialog <ButtonPress> {
    wm deiconify %W
    raise %W
}

proc dialog_safeguard { win } {
   if {[lsearch [bindtags $win] modalDialog] < 0} {
     bindtags $win [linsert [bindtags $win] 0 modalDialog]
   }
}

#------------------------ Message Box -------------------------#

proc messagebox { title message } {
   global labels lang
   global messageid
   
   if {![info exists messageid]} {
        set messageid 0
   }
   
   set msgbox .mesg$messageid
   
   dialog_create $title $msgbox
   
   label $msgbox.info.mesg -justify left -text "$message" -wraplength 3i -bg LightGray -fg black
   pack $msgbox.info.mesg -side top -expand yes -padx 6 -pady 6
   
   button $msgbox.controls.dismiss -textvariable labels(button,dismiss) \
     -command [subst { destroy $msgbox }]
   pack $msgbox.controls.dismiss
   
   incr messageid     
}     



