# Procedure to handle analyst interaction in
# ring boundary identification.

proc trees_interaction_init {} {
    global interact
    
    set interact 0

   # Prompt user to click on browse image
    set answer [tk_messageBox -type okcancel -default cancel -title "Analyst Intervention" -message "Ring appears to be broken. Click on the browse image to link to that point."]

    switch -- $answer {
	ok {
	    # Create a mouse binding on btn-1 to accept user input
	    bind .trees_main.zoompanel.zoomframe.canvas <1> { trees_interaction_handle_click %W %x %y }
	    
 	    vwait interact
	}
	cancel {Sadie_TREES_Ext_SetUserInput 0 0 0 0 0}
    }
    
}

# Procedure to handle mouse-btn-1 click in the image canvas
# during analyst interaction.
proc trees_interaction_handle_click {w windowx windowy} {
    global zoom findrings interact

    
    # Convert window to canvas co-ordinates
    set canv_x [$w canvasx $windowx]
    set canv_y [$w canvasy $windowy]

    if { $canv_x < 0 || $canv_y < 0 || $canv_x >= $zoom(framewidth) || $canv_y >= $zoom(frameheight) } {
	tk_messageBox -type ok -default ok -title "Out of Range Error" -message "The point you selected lies outside the image boundary. Click within the zoom window."
    } else {

	# Convert browse image coordinates to mosaic coordinates
	# This is just a scaling
#	set canv_x [expr $canv_x*$zoom(default)/$findrings(ring,resolution)]
#	set canv_y [expr $canv_y*$zoom(default)/$findrings(ring,resolution)]
	
	# This will set the coordinates in the variable space
	# of the Tcl-C interface layer
	Sadie_TREES_Ext_SetUserInput 1 $canv_x $canv_y $zoom(X1) $zoom(Y1)
	
	bind .trees_main.zoompanel.zoomframe.canvas <1> {}
	
	set interact 1
    }
}


# This procedure is called when the user initiates rectangle region for zoom-in
proc interaction_zoom { canv_x canv_y } {
    global zoom session mosaic findrings


    # Compute zoom region at ring resolutoin
    set zoom(X1) [expr round($canv_x - $zoom(framewidth)/2)]
    set zoom(X2) [expr $zoom(X1) + $zoom(framewidth)]

    set zoom(Y1) [expr round($canv_y - $zoom(frameheight)/2)]
    set zoom(Y2) [expr $zoom(Y1) + $zoom(frameheight)]

    # Handle case when mouse is close to canvas boundary
    if { $zoom(X1) < 0 } {
	set zoom(X1) 0
	set zoom(X2) [expr $zoom(X1) + $zoom(framewidth)]
    }
    if { $zoom(Y1) < 0 } {
	set zoom(Y1) 0
	set zoom(Y2) [expr $zoom(Y1) + $zoom(frameheight)]
    }
    if { $zoom(X2) >= [expr $zoom(fullresolwidth)/$findrings(ring,resolution)] } {
	set zoom(X2) [expr $zoom(fullresolwidth)/$findrings(ring,resolution) - 1]
	set zoom(X1) [expr $zoom(X2) - $zoom(framewidth)] 
    }
    if { $zoom(Y2) >= [expr $zoom(fullresolheight)/$findrings(ring,resolution)] } {
	set zoom(Y2) [expr $zoom(fullresolheight)/$findrings(ring,resolution) - 1]
	set zoom(Y1) [expr $zoom(Y2) - $zoom(frameheight)] 
    }

    # Load, extract and display image of zoom region in window
    if {![info exists mosaic(loaded,index)]} {
	set mosaic(loaded,index) [Sadie_TREES_MosaicIndexOpen \
		$session(directory)/images/mosaic/$mosaic(file,index)]
    }
    
    set zoom(partial,mosaic) [Sadie_TREES_MosaicImport \
	    $mosaic(loaded,index) \
	    $session(directory)/images/mosaic/$mosaic(file,mosaic_columns) \
	    $zoom(Y1) $zoom(Y2) 1 $zoom(X1) $zoom(X2) 1]

    # Delete existing rings and image in zoom panel
    catch {.trees_main.zoompanel.zoomframe.canvas delete "ring"}
    catch {.trees_main.zoompanel.zoomframe.canvas delete $zoom(partial,mosaic)}

    # Create image of new zoom region
    image create photo $zoom(partial,mosaic) -format SADIE -data $zoom(partial,mosaic)

    .trees_main.zoompanel.zoomframe.canvas create image 0 0 \
	    -image $zoom(partial,mosaic) \
	    -tag $zoom(partial,mosaic) \
	    -anchor nw

    update idletasks

    # Map co-ordinates of zoom region at ring resolution back to canvas co-ordinates
    # Need this for drawing zoom region rectangle on canvas
    set zoom(topleft_x) [expr $zoom(X1) * $findrings(ring,resolution)/$zoom(default)]
    set zoom(topleft_y) [expr $zoom(Y1) * $findrings(ring,resolution)/$zoom(default)]
    set zoom(bottomright_x) [expr $zoom(X2) * $findrings(ring,resolution)/$zoom(default)]
    set zoom(bottomright_y) [expr $zoom(Y2) * $findrings(ring,resolution)/$zoom(default)]

    # This is required to display the mosaic image in the zoompanel
    # during analyst interaction in the ring identification process.
    Sadie_TREES_Ext_SetUserInput 0 0 0 $zoom(X1) $zoom(Y1)

}
