# trees_main.tcl - Main tcl functions for TREES application.

# Procedure Index:
# proc trees_main_init {} 
# proc Library_UpdateIndex { libdir } 
# proc trees_initialize {} 
# proc trees_create_gui {} 
# proc trees_create_new_session {} 
# proc trees_verify_new_session {} 
# proc trees_modify_session {} 
# proc trees_modify_samplename {}
# proc trees_create_status_vars {} 
# proc trees_set_colors {} 
# proc trees_close_session {} 
# proc trees_check_save {} 
# proc trees_save_choice {} 
# proc trees_session_save {} 
# proc trees_session_saveas {} 
# proc trees_export_variable {varname} 
# proc trees_export_data {} 
# proc get_sorted_array {arrname} 
# proc trees_session_write {filename} 
# proc trees_autosave {} 
# proc trees_load_datafile {} 
# proc handle_close { widgetname } 
# proc trees_create_buttonpanel {win} 
# proc trees_update_status {} 
# proc trees_create_imagepanel {win} 
# proc trees_create_sessioninfopanel {win} 
# proc trees_update_sessioninfopanel { win } 
# proc trees_create_zoompanel {win} 
# proc create_sessionlog {} 

#--------------------------------------------------------------------



# Main entrypoint for TREES.  This procedure is called by
# the C file trees.c.
proc trees_main_init {} {
   
    wm withdraw .
    create_sessionlog
    trees_initialize
    trees_create_gui
    trees_update_status

    # activate balloon help
    # balloonhelp
    
    
    #--------------------- Initialize Sadie Libraries ----------------------#
    Sadie_General_InitializeProg
    #-----------------------------------------------------------------------#
    
}



# Procedure to automatically update the Tcl index file each time
# the program is run.  This should probably be disabled for static
# release versions of TREES.
proc Library_UpdateIndex { libdir } {
    set index [file join $libdir tclIndex]
    if ![file exists $index] {
	set doit 1
    } else {
	set age [file mtime $index]
	set doit 0
	# Changes to directory may mean files were deleted
	if {[file mtime $libdir] > $age} {
	    set doit 1
	} else {
	    # Check each file for modification
	    foreach file [glob [file join $libdir *.tcl]] {
		if {[file mtime $file] > $age} {
		    set doit 1
		    break
		}
	    }
	}
    }
    if { $doit } {
	auto_mkindex $libdir *.tcl
    }
}



# Initialize TREES
proc trees_initialize {} {
    global labels lang trees_dir auto_path filemarker filemarker_family
    
    # Note: trees_dir is now set in trees.c
    # set trees_dir "/home/conner/trees_work/TREES_v0.0a1/library"
    lappend auto_path $trees_dir
    Library_UpdateIndex $trees_dir
    
    set lang "english"
    language_update labels $lang
    
    set filemarker_family "trees_session*data~file:v0.0"
    set filemarker "%${filemarker_family}%"
    
    trees_create_status_vars
    trees_set_colors
}


# Build the main GUI for TREES
proc trees_create_gui {} {
    toplevel .trees_main -visual truecolor -background LightGray
    wm title .trees_main "TREES v2.0.1"
    wm geometry .trees_main +10+10
    wm protocol .trees_main WM_DELETE_WINDOW { handle_close .trees }
    global guicount SadieVar jion_flag jion_gap_thr flag
    set jion_gap_thr 5
    set guicount 0
    set jion_flag 0
    set flag 1
    set SadieVar(imagelist) {}
    set screenwidth [winfo screenwidth .]
    set screenheight [winfo screenheight .]
    set startwidth [expr $screenwidth * 0.9]
    set startheight [expr $screenheight * 0.9]
    
    wm geometry .trees_main [expr int($startwidth)]x[expr int($startheight)]
    wm maxsize .trees_main [expr int($startwidth)] [expr int($startheight)]

    frame .trees_main.menubar
    trees_create_menubar .trees_main.menubar.m
    
    trees_create_buttonpanel .trees_main.buttonpanel
    
    trees_create_imagepanel .trees_main.imagepanel
    
    trees_create_sessioninfopanel .trees_main.sessioninfopanel

    trees_create_zoompanel .trees_main.zoompanel

    trees_create_profilepanel .trees_main.profilepanel

    trees_create_skelplotpanel .trees_main.skelplotpanel
    
    grid .trees_main.menubar -row 0 -column 0 -columnspan 3 -sticky new
    grid .trees_main.buttonpanel -row 1 -column 0 -rowspan 3 -sticky nw

    grid .trees_main.imagepanel -row 1 -column 1 -sticky news
    grid .trees_main.zoompanel -row 1 -column 2 -sticky news
    grid .trees_main.profilepanel -row 2 -column 1 -sticky news
    grid .trees_main.sessioninfopanel -row 2 -column 2 -sticky news
    grid .trees_main.skelplotpanel -row 3 -column 1 -columnspan 2 -sticky news
    grid rowconfigure .trees_main 0 -weight 0

    grid rowconfigure .trees_main 1 -weight 1
    grid rowconfigure .trees_main 2 -weight 6
    grid rowconfigure .trees_main 3 -weight 0
    grid columnconfigure .trees_main 0 -weight 0
    grid columnconfigure .trees_main 1 -weight 1 -minsize 150
    grid columnconfigure .trees_main 2 -weight 1 -minsize 300
}









# Procedure to open a dialog in order for a user to create
# a new working directory for a new session.
proc trees_create_new_session {} {
    global session
    
    image create bitmap dirimg -background yellow \
	-data {
    # define dir_width 17
    # define dir_height 16
    static unsigned char dir_bits[] = { 
	0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0xe0, 0x01, 0x00, 0x10, 0x02, 0x00, 
	0x08, 0x04, 0x00, 0xfc, 0x7f, 0x00, 0x04, 0x40, 0x00, 0x04, 0x40, 0x00, 
	0x04, 0x40, 0x00, 0x04, 0x40, 0x00, 0x04, 0x40, 0x00, 0x04, 0x40, 0x00, 
	0x04, 0x40, 0x00, 0xfc, 0x7f, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00};
    } \
	-maskdata { 
    # define dirm_width 17
    # define dirm_height 16
    static unsigned char dirm_bits[] = {
	0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0xe0, 0x01, 0x00, 0xf0, 0x03, 0x00,
	0xf8, 0x07, 0x00, 0xfc, 0x7f, 0x00, 0xfc, 0x7f, 0x00, 0xfc, 0x7f, 0x00,
	0xfc, 0x7f, 0x00, 0xfc, 0x7f, 0x00, 0xfc, 0x7f, 0x00, 0xfc, 0x7f, 0x00,
	0xfc, 0x7f, 0x00, 0xfc, 0x7f, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00};
    }
    
    dialog_create "Create New Session" .newsession
    
    set info [dialog_info .newsession]
    
    label ${info}.samplbl_l -text "Sample Label: " -bg LightGray -fg black
    entry ${info}.samplbl_e -textvariable session(sample_label) -bg white -fg #0000C8
    
    label ${info}.analystname_l -text "Analyst Name: " -bg LightGray -fg black
    entry ${info}.analystname_e -textvariable session(analyst_name) -bg white -fg #0000C8
    
    label ${info}.notes_l -text "Notes: " -bg LightGray -fg black
    text ${info}.notes_t -height 4 -width 25 -bg white -fg #0000C8
    
    label ${info}.workdir_l -text "Work Directory: " -bg LightGray -fg black
    entry ${info}.workdir_e -textvariable session(directory) -bg white -fg #0000C8
    button ${info}.workdir_b -image dirimg -bg LightGray -fg black\
	    -command {
	global session
	set session(directory) [tk_getDirectory]
    }
    
    
    
    
    grid ${info}.samplbl_l -row 0 -column 0 -sticky ne
    grid ${info}.samplbl_e -row 0 -column 1 -columnspan 2 -sticky news
    
    grid ${info}.analystname_l -row 1 -column 0 -sticky ne
    grid ${info}.analystname_e -row 1 -column 1 -columnspan 2 -sticky news
    
    grid ${info}.notes_l -row 3 -column 0 -sticky ne
    grid ${info}.notes_t -row 3 -column 1 -columnspan 2 -sticky news
    
    grid ${info}.workdir_l -row 4 -column 0 -sticky ne
    grid ${info}.workdir_e -row 4 -column 1 -sticky news
    grid ${info}.workdir_b -row 4 -column 2 -sticky news
    
    
    set cntls [dialog_controls .newsession]
    
    button ${cntls}.okay -text "Okay" -bg LightGray -fg black\
	    -command {
	global session
	
	set result [trees_verify_new_session]
        if {$result} {
            Sadie_TREES_open_session_doc $session(directory)
	    file mkdir $session(directory)/images
	    file mkdir $session(directory)/images/frames
	    file mkdir $session(directory)/images/mosaic
	    
	    set session(status) ACTIVE
	    set info [dialog_info .newsession]
	    
	    set session(notes) [${info}.notes_t get 1.0 end]
	    set session(needsave) yes

	    set session(start_date) [clock format [clock seconds] -format "%I:%M %p  %b %d, %Y"]
	    set session(last_save_date) [clock format [clock seconds] -format "%I:%M %p  %b %d, %Y"]
	    
	    trees_update_status
	    destroy .newsession
	}
    }
    
    button ${cntls}.cancel -text "Cancel" -bg LightGray -fg black\
	    -command {
	global session
	
	set session(status) NONE
	trees_update_status
	destroy .newsession
    }
    
    grid ${cntls}.okay x ${cntls}.cancel -sticky news
    grid columnconfigure $cntls 1 -minsize 25
}


# Make sure that the requested new session is valid
proc trees_verify_new_session {} {
    global session
    
    if {![info exists session(directory)]} {
        messagebox "Error" "Error: No session directory has been created!"
        return 0;
    } elseif {![file isdirectory $session(directory)]} {
        messagebox "Error" "Error: The work directory is not valid!"
        return 0;
    } elseif {[glob -nocomplain $session(directory)/*] != ""} {
        messagebox "Error" "Error: The work directory has files in it.  Please choose an empty work directory when creating a new session."
        return 0;
    }
    
    return 1;
}




# Create a dialog box for modifying session information
# such as analyst name, sample label, etc.
proc trees_modify_session {} {
    global session
    
    if {$session(status) != "NONE"} {
        dialog_create "Modify Session" .modsession
	
        set info [dialog_info .modsession]
	
        label ${info}.samplbl_l -text "Sample Label: " -bg LightGray -fg black
        entry ${info}.samplbl_e -textvariable session(sample_label) -bg white -fg #0000C8
	
        label ${info}.analystname_l -text "Analyst Name: " -bg LightGray -fg black
        entry ${info}.analystname_e -textvariable session(analyst_name) -bg white -fg #0000C8
	
        label ${info}.notes_l -text "Notes: " -bg LightGray -fg black
        text ${info}.notes_t -height 4 -width 25 -wrap word -bg white -fg #0000C8
        ${info}.notes_t insert 1.0 $session(notes)
	
	
        grid ${info}.samplbl_l -row 0 -column 0 -sticky ne
        grid ${info}.samplbl_e -row 0 -column 1 -columnspan 2 -sticky news

        grid ${info}.analystname_l -row 1 -column 0 -sticky ne
        grid ${info}.analystname_e -row 1 -column 1 -columnspan 2 -sticky news

        grid ${info}.notes_l -row 3 -column 0 -sticky ne
        grid ${info}.notes_t -row 3 -column 1 -columnspan 2 -sticky news
	
	
	
        set cntls [dialog_controls .modsession]
	
        button ${cntls}.okay -text "OK" -bg LightGray -fg black\
		-command {
	    set info [dialog_info .modsession]
	    set session(notes) [${info}.notes_t get 1.0 end]
	    trees_update_sessioninfopanel .trees_main.sessioninfopanel
	    destroy .modsession
	}
        button ${cntls}.cancel -text "Cancel" -bg LightGray -fg black\
		-command {
	    destroy .modsession
	}
	
        grid ${cntls}.okay ${cntls}.cancel -sticky news
    }
}


# Create a dialog box for modifying the name of the sample
proc trees_modify_samplename {} {
   global skelplot SkelPlot

   dialog_create "Modify Sample Name" .modsample

   set info [dialog_info .modsample]

   label ${info}.samplbl_l -text "Enter New Sample Name: " -bg LightGray -fg black
   entry ${info}.samplbl_e -textvariable skelplot(seriesname) -bg white -fg #0000C8

   grid ${info}.samplbl_l -row 0 -column 0 -sticky ne
   grid ${info}.samplbl_e -row 0 -column 1 -columnspan 2 -sticky news

   set cntls [dialog_controls .modsample]

   button ${cntls}.okay -text "OK" -bg LightGray -fg black\
   -command {
	         destroy .modsample
	        }

   grid ${cntls}.okay -sticky news

}

# Initialize the global variables that will be used to track
# TREES program state information.  There is one status variable
# for each module button at the left edge of the TREES gui.
proc trees_create_status_vars {} {
    global noprompting getstretched gui session capture registration mosaic findrings measurements skelplot
    global plotdata
    
    set noprompting 0
    set getstretched "true"
    
    set gui(mosaic) 0
    set gui(rings) 0
    set gui(profileplot) 0
    set gui(skelplot) 0

    set session(status) NONE
    
    set capture(status) NONE
    
    set registration(status) NONE

    set mosaic(status) NONE
    
    set findrings(status) NONE
    
    set measurements(status) NONE
    
    set skelplot(status) NONE
    
    set plotdata(status) NONE
}

# Set the trees gui colors
proc trees_set_colors {} {
   global boundarycolor
   
   # Ring boundary Colors on .trees_main.imagepanel.canvas and
   # .trees_main.zoompanel.zoomframe.canvas
   set boundarycolor(normal) green
   set boundarycolor(added) yellow
   set boundarycolor(removed) cyan
   set boundarycolor(highlight) red
   #8B0000 = DarkRed
   set boundarycolor(timeseries) #8B0000
   #0000C8 = Medium Blue 'Blue3'
   set boundarycolor(plotbones) #0000C8
}

# Close the current session
proc trees_close_session {} {
   global gui session capture registration mosaic findrings \
          measurements skelplot zoom roi SkelPlot
    
    if {$session(status) != "NONE"} {
	set result [tk_dialog .dlg "Close Session?" \
		"Do you wish to close the current session?" \
		question \
		0 "No" "Yes" ]
	if { $result } {
	    
	    set result [trees_check_save]
	
	    if {$result == "ok"} {
		
		if {[info exists mosaic(loaded,fullresol,mosaic)]} {
		    Sadie_TREES_ByteClose $mosaic(loaded,fullresol,mosaic)
		    unset mosaic(loaded,fullresol,mosaic)
		}
		if {[info exists mosaic(loaded,fullresol,index)]} {
		    Sadie_TREES_MosaicIndexClose $mosaic(loaded,fullresol,index)
		    unset mosaic(loaded,fullresol,index)
		}
		if {[info exists mosaic(loaded,mosaic)]} {
		    Sadie_TREES_ByteClose $mosaic(loaded,mosaic)
		    unset mosaic(loaded,mosaic)
		}
		if {[info exists mosaic(loaded,smallmosaic)]} {
		    Sadie_General_Close $mosaic(loaded,smallmosaic)
		    unset mosaic(loaded,smallmosaic)
		}
		if {[info exists mosaic(loaded,index)]} {
		    Sadie_TREES_MosaicIndexClose $mosaic(loaded,index)
		    unset mosaic(loaded,index)
		}
		if {[info exists measurements(loaded,ringwidths)]} {
		    Sadie_TREES_ByteClose $measurements(loaded,ringwidths)
		    unset measurements(loaded,ringwidths)
		}
		if {[info exists measurements(loaded,smallringwidths)]} {
		    Sadie_General_Close $measurements(loaded,smallringwidths)
		    unset measurements(loaded,smallringwidths)
		}
		if {[info exists stagemotion(capture,session)]} {
		    unset stagemotion
		}
		if {[info exists measurements(profiles)]} {
		    unset measurements(profiles)
		}
		if {[winfo exists .stagelinkgui]} {
		    destroy .stagelinkgui
		}
		
		if {$gui(mosaic) != 0} {
		    .trees_main.imagepanel.canvas delete all
		    .trees_main.imagepanel.canvas configure \
			    -height 300 \
			    -width 500 \
			    -scrollregion "0 0 300 500"
		    
		    #image delete "mosaicphoto"
		}
		bind .trees_main.imagepanel.canvas <Motion> {}
		.trees_main.imagepanel.canvas config -cursor left_ptr
		.trees_main.zoompanel.zoomframe.canvas delete all
	    	
		# Following lines deleted JE 7-26-99
		# The previous delete all command should do the job
		# The following has left the application in a
		# precarious state.
		# if {[info exists zoom(partial,mosaic)]} {
		# image delete $zoom(partial,mosaic)
		# }
		    
		.trees_main.zoompanel.scalevaluelabel config -text ""
		
                # Destruction of profile plot
                if {$gui(profileplot) != 0} {
                    trees_destroy_profileplot .trees_main.profilepanel
                }
		# Insert destruction of skeleton plot here
		if {$gui(skelplot) != 0} {
		    trees_destroy_skelplot .trees_main.skelplotpanel
		    destroy .trees_main.chgcutoff
		    destroy .trees_main.attributes
		    destroy .trees_main.select
		    destroy .trees_main.setendyear
		}
		Sadie_TREES_close_session_doc $session(directory)
		unset session
		unset capture
		unset registration
		unset mosaic
		unset findrings
		unset measurements
		unset skelplot

		
		trees_create_status_vars
		trees_update_status
	    }            
	    return "ok"
	} else {
	    return "err"
	}
    } else {
	return "ok"
    }
}


# If a save is necessary, then save it
proc trees_check_save {} {
    global session
    
    if {$session(status) != "NONE" && $session(needsave) == "yes"} {
        set choice [trees_save_choice]
        if {$choice == "yes"} then {         
           set result [trees_session_save]
           if {$result == "ok"} {
              return ok
           } else {
              return err
           }        
        } elseif {$choice == "no"} then {
           return ok
        } else {
           return err
        }
    }
    return ok
}


# Create a dialog for prompting the user about saving before an exit
proc trees_save_choice {} {
    global session labels choice
   
     dialog_create $labels(exit,verify) .chksave
     set info [dialog_info .chksave]
	 label ${info}.mesg -justify left -text "$labels(exit,save)" -wraplength 3i -bg LightGray -fg black
     pack ${info}.mesg -side top -expand yes -padx 6 -pady 6

     button .chksave.yes -text $labels(button,yes) -command {set choice yes} -bg LightGray -fg black
     button .chksave.no -text $labels(button,no) -command {set choice no} -bg LightGray -fg black
     button .chksave.cancel -text $labels(button,cancel) -command {set choice cancel} -bg LightGray -fg black

     pack .chksave.yes .chksave.no .chksave.cancel -side left -expand yes -pady 8
     focus .chksave.cancel

     dialog_wait .chksave choice

     destroy .chksave
     return $choice
}



# Export values of one tree-ring variable in ITRDB ring-width format
proc trees_export_variable {varname} {
    global session skelplot SkelPlot plotdata measurements
    
    # Verify that measurement information exists
    if {![array exists skelplot] \
            || ![array exists measurements] \
            || ![array exists SkelPlot] \
            || $skelplot(status) != "COMPLETE"} {
        return err
    }
    set filetypes {
        {{Text: ITRDB ring-width format} {.rwl}        }
        {{All Files}                     *             }
    }
    # Open the data export file.
    set fname "$skelplot(seriesname)_$varname.rwl";
    set exchanid 0
    set attempt 1
    while {$attempt} {
        set attempt 0
        set exportpath [tk_getSaveFile \
                            -defaultextension ".rwl" \
                            -initialfile $fname \
                            -filetypes $filetypes]
        if {$exportpath == ""} {
            return err
        }
        if { [catch {open $exportpath "w"} exchanid] } {
            set attempt [tk_dialog .dlg "Error" \
                             "Error exporting the $varname measurements: $exchanid. Try again?" \
                             question \
                             0 "No" "Yes"]
            if {!$attempt} {
                return err
            }
        }
    }
    createPlotFileData $SkelPlot(sample,series,serieslist)
    Sadie_TREES_list_to_vec $session(directory) $varname $measurements($varname)
    # Write the data themselves.
    if { [catch {
        Sadie_TREES_ExportRwl $exchanid $skelplot(startyear) $session(directory) $skelplot(seriesname) $varname
    }] } {
        tk_dialog .dlg "Warning" \
            "Could not export the $varname measurements to $exportpath" \
            warning 0 "Cancel"
        close $exchanid
        return err
    }
    close $exchanid
    return ok
}



# Export tree-ring data in formats that other software can interpret
proc trees_export_data {} {
    global session skelplot SkelPlot plotdata measurements
    
    # Verify that measurement information exists
    if {![array exists skelplot] \
            || ![array exists SkelPlot] \
            || $skelplot(status) != "COMPLETE"} {
        return err
    }
    set filetypes {
        {{Text: comma-separated values} {.csv}        }
        {{All Files}                    *             }
    }
    # Open the data export file.
    set fname "$skelplot(seriesname).csv";
    set exchanid 0
    set attempt 1
    while {$attempt} {
        set attempt 0
        set exportpath [tk_getSaveFile \
                            -defaultextension ".csv" \
                            -initialfile $fname \
                            -filetypes $filetypes]
        if {$exportpath == ""} {
            return err
        }
        if { [catch {open $exportpath "w"} exchanid] } {
            set attempt [tk_dialog .dlg "Error" \
                             "Error exporting the measurements: $exchanid. Try again?" \
                             question \
                             0 "No" "Yes"]
            if {!$attempt} {
                return err
            }
        }
    }
    createPlotFileData $SkelPlot(sample,series,serieslist)
    Sadie_TREES_list_to_vec $session(directory) $varname $measurements($varname)
    # Write the data themselves.
    if { [catch {
        Sadie_TREES_ExportCsv $exchanid $skelplot(startyear) $session(directory)
    }] } {
        tk_dialog .dlg "Warning" \
            "Could not export the measurements to $exportpath" \
            warning 0 "Cancel"
        close $exchanid
        return err
    }
    close $exchanid
    return ok
}



# Save all of the important TREES Tcl data to a datafile
proc trees_session_save {} {
    global filemarker session capture registration mosaic findrings measurements skelplot
    
    # Verify that session information exists
    if {[array exists session] != 1} {
        set cansave no
    } elseif {$session(directory) == "NONE"} {
        set cansave no
    } else {
        set cansave yes
    }
    
    if {![info exists session(datafile)]} {
        set types {
            {{TREES Data Files} {.tdf}        }
            {{All Files}        *             }
        }
        set datafile [tk_getSaveFile -defaultextension ".tdf" -initialdir $session(directory) -initialfile "datafile.tdf" -filetypes $types]
        if {$datafile == ""} {
            set cansave no
        } elseif {[file dirname $datafile] != $session(directory)} {
           set result [tk_dialog .dlg "Error" \
             "The datafile should be stored in the data directory.  Do you wish to save the file as $session(directory)/[file tail $datafile]?" \
             question \
             0 "No" "Yes" ]
           if { $result } {
              set session(datafile) [file tail $datafile]
           } else {
              set cansave no
           }
        } else {
            set session(datafile) [file tail $datafile]
        }
    }
    
    if {$cansave == "yes"} {
        set session(last_save_date) [clock format [clock seconds] -format "%I:%M %p  %b %d, %Y"]
        set filename "$session(directory)/$session(datafile)"
    
        if {[file exists $filename]} {
            set i 0
            set done no
            
            while {$done == "no"} {
                if {[file exists "${filename}~$i"]} {
                    incr i
                } else {
                    file copy "${filename}" "${filename}~$i"
                    file delete "${filename}"
                    set done yes
                }
            }
        }
        

        if {[trees_session_write $filename] != "err"} then {
            set session(needsave) no
        } else {
            return err
        }
    } else {
        return err
    }
    
    return ok
}

proc trees_session_saveas {} {
    global session

    if {[array exists session] && [info exists session(datafile)]} {
        set saveas_oldfilname $session(datafile)
        unset session(datafile)
    }
    set err [trees_session_save]
    if {$err != "ok" && [info exists saveas_oldfilname]} {
        set session(datafile) $saveas_oldfilname
    }
    return $err
}

# Sort an array into alphabetical order by index so that it is easier
# to find information in a data file
proc get_sorted_array {arrname} {
     upvar #0 $arrname arr
 
     set index_names [array names arr]
     set sorted_names [lsort -dictionary $index_names]
     
     set results ""
 
     foreach name $sorted_names {
       lappend results $name
       lappend results $arr($name)
     }
 
     return $results
}

# Actually write the Tcl arrays to a TREES datafile
proc trees_session_write {filename} {
    global filemarker session capture registration mosaic findrings measurements skelplot
    global SkelPlot

    if {$filename != ""} then {
        # open file for save
        set fileid [open $filename "w" 0666]

        # save name
        puts $fileid $filemarker

        puts $fileid "\# DO NOT MODIFY THIS FILE!"
        puts $fileid "\# It was generated automatically by TREES v2.0"
        puts $fileid "\#"

        puts $fileid "\#Session info"
        set session(notes) [split $session(notes) \n]
        puts $fileid [get_sorted_array session]
        set session(notes) [join $session(notes) \n]

        puts $fileid "\#Capture info"
        puts $fileid [get_sorted_array capture]

        puts $fileid "\#Registration info"
        puts $fileid [get_sorted_array registration]

        puts $fileid "\#Mosaic info"
        puts $fileid [get_sorted_array mosaic]

        puts $fileid "\#Ring identification info"
        puts $fileid [get_sorted_array findrings]

        puts $fileid "\#Measurement info"
        puts $fileid [get_sorted_array measurements]

        puts $fileid "\#Skeleton plot info"
        puts $fileid [get_sorted_array skelplot]
        
        puts $fileid "\#Export Skeleton plot info"
        if {$skelplot(status) == "COMPLETE"} {
           createPlotFileData $SkelPlot(sample,series,serieslist)
        } 
        puts $fileid [get_sorted_array plotdata]

        # close file
        close $fileid
    } else {
        return err
    }

	return ok
}


# Automatically generate a unique timestamped filename and write the current session to it.
# The timestamps in the file name follow ISO 8601 basic format.
proc trees_autosave {} {
    global filemarker labels session capture registration mosaic findrings measurements skelplot

    set timestamp [clock format [clock seconds] -format "%Y%m%dT%H%M%S"]
    set prefix "autosave"
    set suffix ".tdf"
    set filename "${prefix}_${timestamp}${suffix}"
    set filepath [file join $session(directory) $filename]
    set n 0
    while ([file exists $filepath]) {
        incr n
        set filename "${prefix}_${timestamp}_${n}${suffix}"
        set filepath [file join $session(directory) $filename]
    }
    if {[trees_session_write $filepath] == "err"} then {
        return err
    } else {
        return ok
    }
}





# Load the datafile from disk back into global Tcl arrays
proc trees_load_datafile {} {
    global filemarker filemarker_family labels session capture registration mosaic findrings measurements skelplot

    # get filename to open
    set types {
        {{TREES Data Files} {.tdf}        }
        {{All Files}        *             }
    }
    set filename [tk_getOpenFile -filetypes $types]

    # open file for reading
    if {[file readable $filename]} then {
        set fileid [open $filename "r"]

        # read in data
        gets $fileid valid

        # check to see if file is valid
        if {[string match *${filemarker_family}* $valid]} then {        

	    gets $fileid fileline
            while {[string match \#* $fileline]} {
                gets $fileid fileline
            }
            puts $fileline
            array set session $fileline
            set session(notes) [join $session(notes) \n]
          
	    gets $fileid fileline
            while {[string match \#* $fileline]} {
                gets $fileid fileline
            }
            array set capture $fileline

	    gets $fileid fileline
            while {[string match \#* $fileline]} {
                gets $fileid fileline
            }
            array set registration $fileline

	    gets $fileid fileline
            while {[string match \#* $fileline]} {
                gets $fileid fileline
            }
            array set mosaic $fileline

	    gets $fileid fileline
            while {[string match \#* $fileline]} {
                gets $fileid fileline
            }
            array set findrings $fileline

	    gets $fileid fileline
            while {[string match \#* $fileline]} {
                gets $fileid fileline
            }
            array set measurements $fileline

	    gets $fileid fileline
            while {[string match \#* $fileline]} {
                gets $fileid fileline
            }
            array set skelplot $fileline

            if {[info exists mosaic(loaded,fullresol,mosaic)]} {
                unset mosaic(loaded,fullresol,mosaic)
            }
            if {[info exists mosaic(loaded,fullresol,index)]} {
                unset mosaic(loaded,fullresol,index)
            }
            if {[info exists mosaic(loaded,mosaic)]} {
                unset mosaic(loaded,mosaic)
            }
            if {[info exists mosaic(loaded,smallmosaic)]} {
                unset mosaic(loaded,smallmosaic)
            }
            if {[info exists mosaic(loaded,index)]} {
                unset mosaic(loaded,index)
            }
	    if {[info exists findrings(loaded,chainmap)]} {
		unset findrings(loaded,chainmap)
	    }
	    if {[info exists findrings(loaded,smallchainmap)]} {
		unset findrings(loaded,smallchainmap)
	    }
            if {[info exists measurements(loaded,ringwidths)]} {
                unset measurements(loaded,ringwidths)
            }
            if {[info exists measurements(loaded,smallringwidths)]} {
                unset measurements(loaded,smallringwidths)
            }
            if {[info exists measurements(loaded)]} {
                unset measurements(loaded)
            }

            Sadie_TREES_open_session_doc $session(directory)
            trees_update_status
	    
	    set session(needsave) no
            set session(status) ACTIVE

        } else {
            messagebox "Invalid File" "The file you selected is not a valid TREES v2.0 file."
        }
    }
}




#-------------------------- Handle Close ---------------------------#


proc handle_close { widgetname } {
    global labels lang 
  
    switch -exact -- $widgetname {
        ".trees" -
        "." {
	    if {[trees_close_session] == "ok"} then {
		set result [tk_dialog .dlg $labels(titles,exit) \
			$labels(message,exit) \
			question \
			0 $labels(button,exitno) $labels(button,exityes) ]
		if { $result } {
		    destroy .
		    exit
		}
	    }
	}
        default {destroy $widgetname}
    }
}   






# Create the TREES button panel that exists on the left
# edge of the main gui
proc trees_create_buttonpanel {win} {
    global trees_dir

    frame ${win} -relief raised -borderwidth 2 -background LightGray

    image create bitmap capturebutton -file ${trees_dir}/camera.xbm
    image create bitmap mosaicbutton -file ${trees_dir}/mosaic.xbm
    image create bitmap ringsbutton -file ${trees_dir}/rings.xbm
    image create bitmap measurebutton -file ${trees_dir}/measure.xbm
    image create bitmap skelplotbutton -file ${trees_dir}/skelplot.xbm

    frame ${win}.capture -relief flat -borderwidth 3 -background LightGray
    frame ${win}.capture.bframe -relief flat -bg red -borderwidth 2 -background LightGray
    button ${win}.capture.bframe.button -image capturebutton -state disabled -background LightGray\
      -command {
	  				trees_capture_run
					puts "Success..."
				}
    pack ${win}.capture.bframe.button
    label ${win}.capture.label -textvariable labels(capture,label) -bg LightGray -fg black
    pack ${win}.capture.bframe ${win}.capture.label -side top
    
    frame ${win}.mosaic -relief flat -borderwidth 3 -background LightGray
    frame ${win}.mosaic.bframe -relief flat -bg red -borderwidth 2 -background LightGray
    button ${win}.mosaic.bframe.button -image mosaicbutton -state disabled -background LightGray\
      -command {
                    trees_register_and_mosaic
                    puts "Success...!!!"
                }
    pack ${win}.mosaic.bframe.button
    label ${win}.mosaic.label -textvariable labels(mosaic,label) -bg LightGray -fg black
    pack ${win}.mosaic.bframe ${win}.mosaic.label -side top
    
    frame ${win}.rings -relief flat -borderwidth 3 -background LightGray
    frame ${win}.rings.bframe -relief flat -bg red -borderwidth 2
    button ${win}.rings.bframe.button -image ringsbutton -state disabled -background LightGray\
      -command {
                    trees_find_rings
                    puts "Success...!!!"
                }
    pack ${win}.rings.bframe.button
    label ${win}.rings.label -textvariable labels(rings,label) -bg LightGray -fg black
    pack ${win}.rings.bframe ${win}.rings.label -side top

    frame ${win}.measure -relief flat -borderwidth 3 -background LightGray
    frame ${win}.measure.bframe -relief flat -bg red -borderwidth 2
    button ${win}.measure.bframe.button -image measurebutton -state disabled -background LightGray\
      -command {
                    trees_measure_ringwidths
                    puts "Success...!!!"
                }
    pack ${win}.measure.bframe.button
    label ${win}.measure.label -textvariable labels(measure,label) -bg LightGray -fg black
    pack ${win}.measure.bframe ${win}.measure.label -side top

    frame ${win}.skelplot -relief flat -borderwidth 3 -background LightGray
    frame ${win}.skelplot.bframe -relief flat -bg red -borderwidth 2
    button ${win}.skelplot.bframe.button -image skelplotbutton -state disabled -background LightGray\
      -command {
                    trees_skelplot_create
                }
    pack ${win}.skelplot.bframe.button
    label ${win}.skelplot.label -textvariable labels(skelplot,label) -bg LightGray -fg black
    pack ${win}.skelplot.bframe ${win}.skelplot.label -side top
      
    grid ${win}.capture -sticky news
    grid ${win}.mosaic -sticky news
    grid ${win}.rings -sticky news
    grid ${win}.measure -sticky news
    grid ${win}.skelplot -sticky news



    frame ${win}.legend -relief ridge -borderwidth 3 -background LightGray
    label ${win}.legend.title -textvariable labels(legend,title) -bg LightGray -fg black
    frame ${win}.legend.active -background LightGray
    canvas ${win}.legend.active.c -height 10 -width 10 -bg green
    label ${win}.legend.active.label -textvariable labels(legend,active) -bg LightGray -fg black
    pack ${win}.legend.active.c ${win}.legend.active.label \
        -side left
    frame ${win}.legend.complete -background LightGray
    canvas ${win}.legend.complete.c -height 10 -width 10 -bg blue
    label ${win}.legend.complete.label -textvariable labels(legend,complete) -bg LightGray -fg black
    pack ${win}.legend.complete.c ${win}.legend.complete.label \
        -side left
    frame ${win}.legend.notused -background LightGray
    canvas ${win}.legend.notused.c -height 10 -width 10 -bg red
    label ${win}.legend.notused.label -textvariable labels(legend,notused) -bg LightGray -fg black
    pack ${win}.legend.notused.c ${win}.legend.notused.label \
        -side left
    pack ${win}.legend.title ${win}.legend.active \
        ${win}.legend.complete ${win}.legend.notused \
        -side top -anchor w
    
    grid ${win}.legend -sticky news
}



# Update menubars, button color codes, etc. in the main gui based
# on the current state of the program
proc trees_update_status {} {
   global gui session capture registration mosaic findrings measurements skelplot labels zoom boundarycolor SkelPlot Image global_endyear

    set session(needsave) "yes"
    switch -exact -- $session(status) {
	NONE {
      .trees_main.buttonpanel.capture.bframe.button configure -state disabled
      .trees_main.buttonpanel.mosaic.bframe.button configure -state disabled
      .trees_main.buttonpanel.rings.bframe.button configure -state disabled
      .trees_main.buttonpanel.measure.bframe.button configure -state disabled
      .trees_main.buttonpanel.skelplot.bframe.button configure -state disabled
      .trees_main.menubar.m.file.menu entryconfigure $labels(menu,file,save) -state disabled
      .trees_main.menubar.m.file.menu entryconfigure $labels(menu,file,saveas) -state disabled
      .trees_main.menubar.m.file.menu entryconfigure $labels(menu,file,csv) -state disabled
      .trees_main.menubar.m.file.menu entryconfigure $labels(menu,file,widthrwl) -state disabled
      .trees_main.menubar.m.file.menu entryconfigure $labels(menu,file,laterwl) -state disabled
      .trees_main.menubar.m.file.menu entryconfigure $labels(menu,file,earlyrwl) -state disabled
      .trees_main.menubar.m.file.menu entryconfigure $labels(menu,file,close) -state disabled
      .trees_main.menubar.m.edit.menu entryconfigure $labels(menu,edit,modifysession) -state disabled
      .trees_main.menubar.m.edit.menu entryconfigure $labels(menu,edit,modseriesname) -state disabled
	  .trees_main.zoompanel.zoombutton configure -state disabled
	  .trees_main.zoompanel.resetbutton configure -state disabled
      set session(sample_label) ""
      set session(analyst_name) ""
      set session(notes) ""
      set session(directory) ""
      set session(start_date) ""
      set session(last_save_date) ""
      trees_update_sessioninfopanel .trees_main.sessioninfopanel   
      .trees_main.profilepanel config -bg white
      .trees_main.skelplotpanel config -bg white
      .trees_main.menubar.m.skeletonplot.menu entryconfigure $labels(menu,skeletonplot,options) -state disabled
      .trees_main.menubar.m.skeletonplot.menu entryconfigure $labels(menu,skeletonplot,attributes) -state disabled
      .trees_main.menubar.m.skeletonplot.menu entryconfigure $labels(menu,skeletonplot,select) -state disabled
      .trees_main.menubar.m.skeletonplot.menu entryconfigure $labels(menu,skeletonplot,addring) -state disabled
      .trees_main.menubar.m.skeletonplot.menu entryconfigure $labels(menu,skeletonplot,removering) -state disabled
      .trees_main.menubar.m.skeletonplot.menu entryconfigure $labels(menu,skeletonplot,endyear) -state disabled
      .trees_main.menubar.m.skeletonplot.menu entryconfigure $labels(menu,skeletonplot,startyear) -state disabled
      .trees_main.menubar.m.skeletonplot.menu entryconfigure $labels(menu,skeletonplot,redraw) -state disabled
      .trees_main.menubar.m.skeletonplot.menu entryconfigure $labels(menu,skeletonplot,invert) -state disabled
      .trees_main.menubar.m.stage.menu entryconfigure $labels(menu,stage,link) -state disabled
       
      .trees_main.menubar.m.segmentation.menu entryconfigure $labels(menu,segmentation,preference) -state disable
      .trees_main.menubar.m.segmentation.menu entryconfigure $labels(menu,segmentation,Selectsegment) -state disabled  
      .trees_main.menubar.m.segmentation.menu entryconfigure $labels(menu,segmentation,Insertsegment) -state disabled
      .trees_main.menubar.m.segmentation.menu entryconfigure $labels(menu,segmentation,Deletesegment) -state disabled
	
      }
	default {
      .trees_main.buttonpanel.capture.bframe.button configure -state normal
      .trees_main.buttonpanel.mosaic.bframe.button configure -state normal
      .trees_main.buttonpanel.rings.bframe.button configure -state normal
      .trees_main.buttonpanel.measure.bframe.button configure -state normal
      .trees_main.buttonpanel.skelplot.bframe.button configure -state normal
      .trees_main.menubar.m.file.menu entryconfigure $labels(menu,file,save) -state normal
      .trees_main.menubar.m.file.menu entryconfigure $labels(menu,file,saveas) -state normal
      .trees_main.menubar.m.file.menu entryconfigure $labels(menu,file,csv) -state normal
      .trees_main.menubar.m.file.menu entryconfigure $labels(menu,file,widthrwl) -state normal
      .trees_main.menubar.m.file.menu entryconfigure $labels(menu,file,laterwl) -state normal
      .trees_main.menubar.m.file.menu entryconfigure $labels(menu,file,earlyrwl) -state normal
      .trees_main.menubar.m.file.menu entryconfigure $labels(menu,file,close) -state normal
      .trees_main.menubar.m.edit.menu entryconfigure $labels(menu,edit,modifysession) -state normal
      .trees_main.menubar.m.edit.menu entryconfigure $labels(menu,edit,modseriesname) -state normal
      trees_update_sessioninfopanel .trees_main.sessioninfopanel 
              }
   }

   switch -exact -- $capture(status) {
      COMPLETE {.trees_main.buttonpanel.capture.bframe config -bg blue
                trees_update_sessioninfopanel .trees_main.sessioninfopanel 
               }
      ACTIVE { .trees_main.buttonpanel.capture.bframe config -bg green
               trees_update_sessioninfopanel .trees_main.sessioninfopanel 
             }
      default {.trees_main.buttonpanel.capture.bframe config -bg red
               trees_update_sessioninfopanel .trees_main.sessioninfopanel  
              }
	  }
   
   switch -exact -- $mosaic(status) {
       COMPLETE {
	  .trees_main.buttonpanel.mosaic.bframe config -bg blue
	  .trees_main.zoompanel.zoombutton configure -state normal
	  .trees_main.zoompanel.resetbutton configure -state disabled

         if {$gui(mosaic) == 0} {
	     
	     if {![info exists mosaic(loaded,smallmosaic)]} {
		 set mosaic(loaded,smallmosaic) [Sadie_FileIO_Open $session(directory)/images/mosaic/$mosaic(file,smallmosaic) 0 "smallmosaic"]
	     }
	     
	     image create photo "mosaicphoto" -format SADIE -data $mosaic(loaded,smallmosaic)
	     Sadie_General_Close $mosaic(loaded,smallmosaic)
	     unset mosaic(loaded,smallmosaic)
	     
	     set gui(mosaic) "mosaicphoto"
	     
	     
	     set h [image height $gui(mosaic)]
	     set w [image width $gui(mosaic)]
	     
	     .trees_main.imagepanel.canvas configure \
		     -height $h \
		     -width $w \
		     -scrollregion "0 0 $w $h"
	     
	     .trees_main.imagepanel.canvas create image 0 0 \
		     -image $gui(mosaic) \
		     -tag $gui(mosaic) \
		     -anchor nw
         }
	 
	 set zoom(default) 6
	 
	 set zoom(canvasWidth) [.trees_main.imagepanel.canvas cget -width]
	 set zoom(canvasHeight) [.trees_main.imagepanel.canvas cget -height]
	 
	 set zoom(fullresolwidth) [expr $zoom(canvasWidth) * $zoom(default)]
	 set zoom(fullresolheight) [expr $zoom(canvasHeight) * $zoom(default)]

	 .trees_main.menubar.m.stage.menu entryconfigure $labels(menu,stage,link) -state normal
	 
      }
      ACTIVE {.trees_main.buttonpanel.mosaic.bframe config -bg green}
      default {.trees_main.buttonpanel.mosaic.bframe config -bg red}

   }

   switch -exact -- $findrings(status) {
      COMPLETE {
         .trees_main.buttonpanel.rings.bframe config -bg blue
         if {$gui(rings) == 0} {

            if {![info exists findrings(loaded,smallchainmap)]} {
		set findrings(file,smallchainmap) smallchainmap.CCF
                set findrings(loaded,smallchainmap) [Sadie_TREES_Disk2Chain $session(directory)/images/mosaic/$findrings(file,smallchainmap)]
            }
                
	    Sadie_TREES_DisplayRings $findrings(loaded,smallchainmap) .trees_main.imagepanel.canvas

           # puts "smallchainmap: $findrings(loaded,smallchainmap)"

            set gui(rings) 1
         }

      }
      ACTIVE {.trees_main.buttonpanel.rings.bframe config -bg green}
      default {.trees_main.buttonpanel.rings.bframe config -bg red}
   }

   switch -exact -- $measurements(status) {
      COMPLETE {
            
         .trees_main.buttonpanel.measure.bframe config -bg blue
         if {$gui(profileplot) == 0} {
            draw_profileplot .trees_main.profilepanel $measurements(profiles)
            set gui(profileplot) 1
         }
      }
      ACTIVE {.trees_main.buttonpanel.measure.bframe config -bg green}
      default {.trees_main.buttonpanel.measure.bframe config -bg red}
   }

   switch -exact -- $skelplot(status) {
      COMPLETE {
         .trees_main.buttonpanel.skelplot.bframe config -bg blue
         .trees_main.skelplotpanel config -bg blue
         if {$gui(skelplot) == 0} {
            .trees_main.menubar.m.skeletonplot.menu entryconfigure $labels(menu,skeletonplot,options) -state normal
            .trees_main.menubar.m.skeletonplot.menu entryconfigure $labels(menu,skeletonplot,attributes) -state normal
            .trees_main.menubar.m.skeletonplot.menu entryconfigure $labels(menu,skeletonplot,select) -state normal
            .trees_main.menubar.m.skeletonplot.menu entryconfigure $labels(menu,skeletonplot,addring) -state normal
            .trees_main.menubar.m.skeletonplot.menu entryconfigure $labels(menu,skeletonplot,removering) -state normal
            .trees_main.menubar.m.skeletonplot.menu entryconfigure $labels(menu,skeletonplot,endyear) -state normal
            .trees_main.menubar.m.skeletonplot.menu entryconfigure $labels(menu,skeletonplot,startyear) -state normal
            .trees_main.menubar.m.skeletonplot.menu entryconfigure $labels(menu,skeletonplot,redraw) -state normal
            .trees_main.menubar.m.skeletonplot.menu entryconfigure $labels(menu,skeletonplot,invert) -state normal
            
            .trees_main.menubar.m.segmentation.menu entryconfigure $labels(menu,segmentation,preference) -state normal
            .trees_main.menubar.m.segmentation.menu entryconfigure $labels(menu,segmentation,Selectsegment) -state normal  
            .trees_main.menubar.m.segmentation.menu entryconfigure $labels(menu,segmentation,Insertsegment) -state disable
            .trees_main.menubar.m.segmentation.menu entryconfigure $labels(menu,segmentation,Deletesegment) -state disable
	
            # puts "ringwidths: $measurements(ringwidth)"
            
            set seriesname "series"
            set SkelPlot(sample,$seriesname,color) $boundarycolor(timeseries)
            set SkelPlot(sample,$seriesname,invert) 0
            if {![info exists skelplot(endyear)]} {
               set skelplot(endyear) 0
            }

            # Initialize skelplot variables
            skelplot_init SkelPlot

            # Initialize Image variables
            image_init Image

            # Load the measured ringwidths
            load_ringwidth_series SkelPlot $measurements(ringwidth) $seriesname
            
            set global_endyear [expr [llength $measurements(ringwidth)]-1]
            # Create mouse button-1 click binding for the browse image
            # to find coordinates and tags at click location
            bindMouseToImage SkelPlot Image .trees_main.imagepanel $seriesname canvas xsbar ysbar $SkelPlot(sample,$seriesname,masterlist)
   
            # Create initial skeleton plot canvas
            draw_skelplot SkelPlot .trees_main.skelplotpanel topmarker timeseries $seriesname bottommarker xscr .trees_main.imagepanel.canvas
            #test
            #puts "SkeltonPlot(sample): $SkelPlot(sample,$seriesname,serieslist)"
            #set a series-serieslist
            #highlight_image_ring .trees_main.imagepanel.canvas 3 $SkelPlot(sample,$seriesname,serieslist)
            #highlight_image_ring .trees_main.imagepanel.canvas 4 $SkelPlot(sample,$seriesname,serieslist)
            #highlight_image_ring .trees_main.imagepanel.canvas 5 $SkelPlot(sample,$seriesname,serieslist)
            # Finish up
	        set gui(skelplot) 1
            set skelplot(status) COMPLETE
         }            
      }
      ACTIVE {.trees_main.buttonpanel.skelplot.bframe config -bg green
            .trees_main.skelplotpanel config -bg blue
            .trees_main.menubar.m.skeletonplot.menu entryconfigure $labels(menu,skeletonplot,options) -state normal
            .trees_main.menubar.m.skeletonplot.menu entryconfigure $labels(menu,skeletonplot,attributes) -state normal
            .trees_main.menubar.m.skeletonplot.menu entryconfigure $labels(menu,skeletonplot,select) -state normal
            .trees_main.menubar.m.skeletonplot.menu entryconfigure $labels(menu,skeletonplot,addring) -state normal
            .trees_main.menubar.m.skeletonplot.menu entryconfigure $labels(menu,skeletonplot,removering) -state normal
            .trees_main.menubar.m.skeletonplot.menu entryconfigure $labels(menu,skeletonplot,endyear) -state normal
            .trees_main.menubar.m.skeletonplot.menu entryconfigure $labels(menu,skeletonplot,startyear) -state normal
            .trees_main.menubar.m.skeletonplot.menu entryconfigure $labels(menu,skeletonplot,redraw) -state normal
            .trees_main.menubar.m.skeletonplot.menu entryconfigure $labels(menu,skeletonplot,invert) -state normal
            
            .trees_main.menubar.m.segmentation.menu entryconfigure $labels(menu,segmentation,Selectsegment) -state normal  
            .trees_main.menubar.m.segmentation.menu entryconfigure $labels(menu,segmentation,Insertsegment) -state normal
            .trees_main.menubar.m.segmentation.menu entryconfigure $labels(menu,segmentation,Deletesegment) -state normal
	 
        
      }
      default {.trees_main.buttonpanel.skelplot.bframe config -bg red}
   }

   update idletasks
}



# Create the main image panel at the top of the main gui that is
# used to display browse mosaic, ring boundaries, etc.
proc trees_create_imagepanel {win} {
    frame ${win} -bg LightGray

    canvas ${win}.canvas -height 300 -width 500 \
        -background LightGray \
        -borderwidth 0 -highlightthickness 0 \
        -cursor crosshair \
        -xscrollcommand [subst -nocommands {${win}.xsbar set}] \
        -yscrollcommand [subst -nocommands {${win}.ysbar set}]
    scrollbar ${win}.xsbar -orient horizontal -background LightGray \
        -command [subst -nocommands {${win}.canvas xview}]
    scrollbar ${win}.ysbar -orient vertical -background LightGray \
        -command [subst -nocommands {${win}.canvas yview}]
#    scrollbar ${win}.xsbar -orient horizontal \
#	    -command [subst -nocommands {trees_imageHscroll ${win}}]
#    scrollbar ${win}.ysbar -orient vertical \
#	    -command [subst -nocommands {trees_imageVscroll ${win}}]
        
    grid ${win}.canvas -row 0 -column 0 -sticky news
    grid ${win}.ysbar -row 0 -column 1 -sticky ns
    grid ${win}.xsbar -row 1 -column 0 -sticky ew
    grid rowconfigure ${win} 0 -weight 1
    grid columnconfigure ${win} 0 -weight 1
    
}


# Create the session information panel that exists at the top
# right corner of the gui
proc trees_create_sessioninfopanel {win} {
   global session labels

   frame ${win} -relief sunken -borderwidth 3 -background LightGray 
    
   text ${win}.text \
     -yscrollcommand "${win}.ysbar set" \
     -state disabled \
     -wrap word \
     -width 30 \
     -height 20 \
     -background white \
     -foreground #0000C8   

   scrollbar ${win}.ysbar -orient vertical \
       -background LightGray  \
       -command "${win}.text yview" 

   trees_update_sessioninfopanel ${win}        

   grid ${win}.text ${win}.ysbar -sticky news
   grid rowconfigure ${win} 0 -weight 1
   grid columnconfigure ${win} 0 -weight 1 
  
}

# Puts information in the session information box at the top right
# hand corner of the main gui.
proc trees_update_sessioninfopanel { win } {

   global session labels
   
   ${win}.text configure -state normal
   ${win}.text delete 1.0 end
   
   if {[info exists session(sample_label)]} {
      set sample_label $session(sample_label)
   } else {
      set sample_label ""
   }
   if {[info exists session(analyst_name)]} {
      set analyst_name $session(analyst_name)
   } else {
      set analyst_name ""
   }
   if {[info exists session(start_date)]} {
      set start_date $session(start_date)
   } else {
      set start_date ""
   }
   if {[info exists session(last_save_date)]} {
      set last_save_date $session(last_save_date)
   } else {
      set last_save_date ""
   }
   if {[info exists session(notes)]} {
      set notes ""
      foreach item $session(notes) {
         append notes $item
         append notes " "
      }
   } else {
      set notes ""
   }     
         
   ${win}.text insert end "[string toupper $labels(sessioninfo,title)]\n\n"
   ${win}.text insert end "$labels(sessioninfo,label) $sample_label\n"
   ${win}.text insert end "$labels(sessioninfo,analyst) $analyst_name\n"
   ${win}.text insert end "$labels(sessioninfo,start)   $start_date\n"
   ${win}.text insert end "$labels(sessioninfo,change)  $last_save_date\n"
   ${win}.text insert end "$labels(sessioninfo,notes)\n$notes\n"   
   ${win}.text configure -state disabled

}


# Create the zoom panel that shows a full resolution view of a small window
# of image data and edge maps
proc trees_create_zoompanel {win} {
    global zoom zoomFrozen
    
#<<<<<<< trees_main.tcl
#    set zoom(framewidth) 201
#    set zoom(frameheight) 180
#=======
    set zoom(framewidth) 230
    set zoom(frameheight) 200
#>>>>>>> 1.6
    set zoom(frozen) 0
    
    frame ${win} -relief ridge -borderwidth 2 -background LightGray 
    frame ${win}.zoomframe -relief sunken -borderwidth 2 -background LightGray 
    canvas ${win}.zoomframe.canvas -height $zoom(frameheight) -width $zoom(framewidth) -bg LightGray
    label ${win}.zoomlabel -text "ZOOM" -bg LightGray -fg black
    button ${win}.zoombutton -text "On" -state disabled -bg LightGray -fg black\
	    -command {
	              trees_zoomin
                }

    button ${win}.resetbutton -text "Off" -state disabled -bg LightGray -fg black\
	    -command {
	              trees_zoomoff
                }

    label ${win}.scalelabel -text "Scale" -bg LightGray -fg black
    label ${win}.scalevaluelabel -width 5 -text "" -relief sunken -bg white -fg #0000C8
    grid ${win}.zoomframe.canvas -sticky news
    grid ${win}.zoomframe -row 0 -column 0 -rowspan 5 -sticky nw -padx 5 -pady 5
    grid ${win}.zoomlabel -row 0 -column 1 -sticky new -pady 4
    grid ${win}.zoombutton -row 1 -column 1 -sticky new -pady 2 
    grid ${win}.resetbutton -row 2 -column 1 -sticky new -pady 3
    grid ${win}.scalelabel -row 3 -column 1 -sticky sew 
    grid ${win}.scalevaluelabel -row 4 -column 1  -sticky news -pady 5
}





#------------------- Create a Session Log --------------------------------#
proc create_sessionlog {} {
    toplevel .sessionlog -bg LightGray
    wm title .sessionlog "Session Log"
    wm withdraw .sessionlog

    wm geometry .sessionlog +50+400
    wm protocol .sessionlog WM_DELETE_WINDOW { 
      tk_dialog .dlg \
        "Warning" "Session Log must remain open!" warning 0 OK
     }

    text .sessionlog.text \
      -bg white -fg #0000C8 \
      -yscrollcommand ".sessionlog.scrly set" \
      -xscrollcommand ".sessionlog.scrlx set" \
      -width 75 \
      -height 10 \
      -state disabled

    scrollbar .sessionlog.scrly -command ".sessionlog.text yview" -bg LightGray
    scrollbar .sessionlog.scrlx -command ".sessionlog.text xview" -orient horizontal -bg LightGray

    grid .sessionlog.text .sessionlog.scrly -sticky news
    grid .sessionlog.scrlx -sticky ew
    grid rowconfigure .sessionlog 0 -weight 1
    grid columnconfigure .sessionlog 0 -weight 1


    global newmessage
    trace variable newmessage w trace_newmessage

    proc trace_newmessage { varname elementname operation} {
      upvar $varname var1

      .sessionlog.text configure \
        -state normal

      .sessionlog.text insert end \
        $var1

      .sessionlog.text configure \
        -state disabled

      .sessionlog.text yview moveto 1
      .sessionlog.text xview moveto 1

    }
}
#-----------------------------------------------------------------------#








