proc trees_find_rings {} {
    global session mosaic findrings noprompting labels zoom
    global gaugevar ResolutionVar ModeVar

    set gaugenum 0

    if {$mosaic(status) != "COMPLETE"} {
        messagebox "FindRings Error" "Cannot find rings because image mosaic is not complete."
        set findrings(status) NONE
    } elseif {$findrings(status) == "COMPLETE"} {
        messagebox "FindRings Info" "Tree ring boundaries have already been identified."
    } else {

	set top [dialog_create "Mode of Operation" .specmode]

	set info [dialog_info .specmode]

	label $info.modelbl -text "Select one of the following modes of operation:" -bg LightGray

	frame $info.frm -relief ridge -borderwidth 2 -bg LightGray
	label $info.frm.automodelbl -text "requires no intervention" -bg LightGray
	label $info.frm.intermodelbl -text "requires user interaction" -bg LightGray
	radiobutton $info.frm.automodebtn -text "Automatic" -variable ModeVar \
		-value 0
	radiobutton $info.frm.intermodebtn -text "Interactive" -variable ModeVar \
		-value 1
	set ModeVar 0

	grid $info.frm.automodebtn -row 0 -column 0 -sticky nw -padx 5 -pady 5
	grid $info.frm.intermodebtn -row 1 -column 0 -sticky nw -padx 5 -pady 5
	grid $info.frm.automodelbl -row 0 -column 1 -sticky nw -padx 5 -pady 5
	grid $info.frm.intermodelbl -row 1 -column 1 -sticky nw -padx 5 -pady 5

	grid $info.modelbl -row 0 
	grid $info.frm -row 1 -sticky w -padx 5

        set cntls [dialog_controls .specmode]

        button ${cntls}.okay -text "OK" -bg lightGray \
		-command {
	    if { $ModeVar == 1 } {
		set findrings(ring,mode) INTERACTIVE
	    } else {
		set findrings(ring,mode) AUTOMATIC
	    }
	    set confirmStatus 1
	}
	grid $cntls.okay -row 0 -column 0
	
	dialog_wait $top confirmStatus
	destroy .specmode

	
	set roi(selected) 0
	set findrings(ring,resolution) 1

	set answer [tk_dialog .dlg "Select Region" \
	    "Do you wish to select a region of interest" \
	    question \
	    1 $labels(button,no) $labels(button,yes) ]
	
	if { $answer == 1 } {
	    set flag 1
            trees_roi $flag
	    vwait roi(selected)
	}

	set top [dialog_create "Resolution" .specresolution]

	set info [dialog_info .specresolution]

	label $info.resollbl -text "Specify Resolution to identify rings:" -bg LightGray

	frame $info.frm -relief ridge -borderwidth 2 -bg LightGray
	label $info.frm.fullresollbl -text "Accurate but Slow" -bg LightGray
	label $info.frm.halfresollbl -text "Less accurate but Fast" -bg LightGray
	radiobutton $info.frm.fullresolbtn -text "1:1" -variable ResolutionVar \
		-value 0
	radiobutton $info.frm.halfresolbtn -text "1:2" -variable ResolutionVar \
		-value 1
	set ResolutionVar 0

	grid $info.frm.fullresolbtn -row 0 -column 0 -sticky nw -padx 5 -pady 5
	grid $info.frm.halfresolbtn -row 1 -column 0 -sticky nw -padx 5 -pady 5
	grid $info.frm.fullresollbl -row 0 -column 1 -sticky nw -padx 5 -pady 5
	grid $info.frm.halfresollbl -row 1 -column 1 -sticky nw -padx 5 -pady 5

	grid $info.resollbl -row 0 
	grid $info.frm -row 1 -sticky w -padx 5

        set cntls [dialog_controls .specresolution]

        button ${cntls}.okay -text "OK" -bg lightGray \
		-command {
	    if { $ResolutionVar == 1 } {
		set findrings(ring,resolution) 2
	    }
	    set confirmStatus 1
	}
	grid $cntls.okay -row 0 -column 0
	
	dialog_wait $top confirmStatus
	destroy .specresolution


	update idletasks

	# Create the progress gauge
	set gaugenum [gauge_gui_add "Find Tree Rings"]

        set findrings(status) ACTIVE
        trees_update_status

	
	if { $findrings(ring,resolution) == 2 } {

	    set mosaic(file,mosaic) mosaicimg_LR.SAD
	    set mosaic(file,index) mosaicimg_LR.INDEX
	    set mosaic(file,mosaic_columns) mosaic_columnsimg_LR.SAD

	    if {![info exists mosaic(loaded,fullresol,mosaic)]} {
		set mosaic(loaded,fullresol,mosaic) [Sadie_TREES_ByteOpen\
			$session(directory)/images/mosaic/$mosaic(file,fullresol,mosaic)]
	    }
	    if {![info exists mosaic(loaded,fullresol,index)]} {
		set mosaic(loaded,fullresol,index) [Sadie_TREES_MosaicIndexOpen\
			$session(directory)/images/mosaic/$mosaic(file,fullresol,index)]
	    }
	    
	    # Subsample mosaic image
	    set mosaic(loaded,mosaic) [Sadie_TREES_Mosaic2MosaicResampl\
		    $mosaic(loaded,fullresol,mosaic)\
		    $mosaic(loaded,fullresol,index)\
		    $findrings(ring,resolution)\
		    $findrings(ring,resolution)\
		    $findrings(ring,resolution)\
		    $findrings(ring,resolution)]

	    # Close the full resolution mosaic image
	    Sadie_TREES_ByteClose $mosaic(loaded,fullresol,mosaic)
	    unset mosaic(loaded,fullresol,mosaic)

	    # Subsample mosaic index
	    set mosaic(loaded,index) [Sadie_TREES_IndexResampl\
		    $mosaic(loaded,fullresol,index)\
		    $findrings(ring,resolution)]

	    # Close the full resolution mosaic index
	    Sadie_TREES_MosaicIndexClose $mosaic(loaded,fullresol,index)
	    unset mosaic(loaded,fullresol,index)

	    # Save the low resolution mosaic image
	    Sadie_TREES_ByteSave\
		    $mosaic(loaded,mosaic)\
		    $session(directory)/images/mosaic/$mosaic(file,mosaic)

	    # Export the low resolution mosaic image
	    Sadie_TREES_ByteMosaicExport\
		    $mosaic(loaded,mosaic)\
		    $session(directory)/images/mosaic/$mosaic(file,mosaic_columns)

	    # Save the low resolution mosaic index
	    Sadie_TREES_MosaicIndexSave\
		    $mosaic(loaded,index)\
		    $session(directory)/images/mosaic/$mosaic(file,index)
	    
	    # Update the progress gauge
	    if {$gaugenum != 0} {gauge_gui_update $gaugenum 15}
	    if {$gaugevar(cancel) != 0} {
		if {$gaugenum != 0} {gauge_gui_delete $gaugenum}
		set findrings(status) "NONE"
		trees_update_status
		Sadie_TREES_ByteClose $mosaic(loaded,mosaic)
		Sadie_TREES_MosaicIndexClose $mosaic(loaded,index)
		unset mosaic(loaded,mosaic)
		unset mosaic(loaded,index)
		return;
	    }
	    
	} else {
	    set mosaic(file,mosaic) mosaicimg.SAD
	    set mosaic(file,index) mosaicimg.INDEX
	    set mosaic(file,mosaic_columns) mosaic_columnsimg.SAD
	}
	
	# Open mosaic image
        if {![info exists mosaic(loaded,mosaic)]} {
            set mosaic(loaded,mosaic) [Sadie_TREES_ByteOpen\
		    $session(directory)/images/mosaic/$mosaic(file,mosaic)]
        }

	# Open mosaic index
        if {![info exists mosaic(loaded,index)]} {
            set mosaic(loaded,index) [Sadie_TREES_MosaicIndexOpen\
		    $session(directory)/images/mosaic/$mosaic(file,index)]
        }

	# Update progress gauge
        if {$gaugenum != 0} {gauge_gui_update $gaugenum 25}
        if {$gaugevar(cancel) != 0} {
            if {$gaugenum != 0} {gauge_gui_delete $gaugenum}
            set findrings(status) "NONE"
            trees_update_status
            Sadie_TREES_ByteClose $mosaic(loaded,mosaic)
            Sadie_TREES_MosaicIndexClose $mosaic(loaded,index)
	    unset mosaic(loaded,mosaic)
	    unset mosaic(loaded,index)
            return;
        }

	# Find rings in the mosaic image 
	set zoombuffer [expr $zoom(default)/$findrings(ring,resolution)]
        set results [Sadie_TREES_Ext_TrackRings $mosaic(loaded,mosaic) $mosaic(loaded,index) $findrings(ring,mode)]
	set findrings(loaded,chainmap) [lindex $results 0]

	# Update progress gauge
        if {$gaugenum != 0} {gauge_gui_update $gaugenum 85}
        if {$gaugevar(cancel) != 0} {
            if {$gaugenum != 0} {gauge_gui_delete $gaugenum}
            set findrings(status) "NONE"
            trees_update_status
            Sadie_TREES_ByteClose $mosaic(loaded,mosaic)
            Sadie_TREES_MosaicIndexClose $mosaic(loaded,index)
	    Sadie_TREES_ChainClose $findrings(loaded,chainmap)
            unset mosaic(loaded,mosaic)
            unset mosaic(loaded,index)
	    unset findrings(loaded,chainmap)
            return;
        }

	set findrings(file,chainmap) chainmap.CCF
	set findrings(file,smallchainmap) smallchainmap.CCF

	# Save the full resolution chainmap
        Sadie_TREES_Chain2Disk\
		$findrings(loaded,chainmap)\
		$session(directory)/images/mosaic/$findrings(file,chainmap)

	# Subsample the chainmap
	set findrings(loaded,smallchainmap) [Sadie_TREES_ScaleChain\
		$findrings(loaded,chainmap)\
		$zoombuffer]

	# Save the low resolution chainmap
        Sadie_TREES_Chain2Disk\
		$findrings(loaded,smallchainmap)\
		$session(directory)/images/mosaic/$findrings(file,smallchainmap)

	# Update the progress gauge
        if {$gaugenum != 0} {gauge_gui_update $gaugenum 95}
        if {$gaugevar(cancel) != 0} {
            if {$gaugenum != 0} {gauge_gui_delete $gaugenum}
            set findrings(status) "NONE"
            trees_update_status
            Sadie_TREES_ByteClose $mosaic(loaded,mosaic)
            Sadie_TREES_MosaicIndexClose $mosaic(loaded,index)
	    Sadie_TREES_ChainClose $findrings(loaded,chainmap)
	    Sadie_TREES_ChainClose $findrings(loaded,smallchainmap)
            unset mosaic(loaded,mosaic)
            unset mosaic(loaded,index)
	    unset findrings(loaded,chainmap)
	    unset findrings(loaded,smallchainmap)
            return;
        }

	# Update status
        set findrings(status) "COMPLETE"
        trees_update_status

    }

    trees_update_status
    trees_autosave

    # Delate progress gauge
    if {$gaugenum != 0} {
        gauge_gui_delete $gaugenum
    }

    # Continuing analysis dialog
    if {$findrings(status) == "COMPLETE"} {
        if {$noprompting != 1} {
            set answer [tk_dialog .dlg "Continuing Analysis Dialog" \
                     "The next step is tree ring measurement.  Do you wish to continue?" \
                     question \
                     1 $labels(button,no) $labels(button,yes) ]
        } else {
            set answer 1
        }
        
        if {$answer == 1} {
            trees_measure_ringwidths
        }
    }
}



