# trees_skelplot_data.tcl - Skeleton Plot numeric functions.

# Procedure Index:

# proc RandomInit { seed } 
# proc Random { } 
# proc create_random_series { var size name } 
# proc load_ringwidth_series { var widthlist name } 
# proc compute_indices {var masterRingList workRingList name} 
# proc compute_bones {var ringlist} 
# proc find_maxmin_index { ringlist } 
# proc print_series_info { ringlist } 
# proc getWidth { ringlist ringnum } 
# proc getIndexval { ringlist ringnum } 
# proc getRBoundary { ringlist ringnum } 
# proc getLBoundary { ringlist ringnum } 
# proc getTypelist { ringlist ringnum } 
# proc getYear { ringlist ringnum } 
# proc getIndex { ringlist yearnum }
# proc getBonelength { ringlist ringnum } 
# proc getRingNotes { ringnum } 
# proc setRingNotes { ringnum notes }
# proc insertRingNote { ringnum notes }
# proc deleteRingNote { ringnum } 
# proc createPlotFileData { }

#-----------------------------------------------------------------#

# Random number generator procedures
# uses global variable randomSeed as seed

# reset global variable randomSeed to 'seed'
proc RandomInit { seed } {
   global randomSeed
   set randomSeed $seed
}

# generate pseudo-random number
proc Random { } {
   global randomSeed
   set randomSeed [expr ($randomSeed * 9301 + 49297) % 233280]
   return [expr $randomSeed / double(233280)]
}

# Create a random number series of length 'size' to represent a
# tree-ring sequence for testing purposes.
# Creates the 'masterlist' from which the serieslist is
#  computed by 'compute_indices'
proc create_random_series { var size name } {
   upvar #0 $var SkelPlot

   RandomInit 10000

#    set listName $name-serieslist
   set masterListName $name-masterlist
#    set SkelPlot(sample,$name,serieslist) $listName
   set SkelPlot(sample,$name,masterlist) $masterListName
   set SkelPlot(sample,$name,size) $size
   set SkelPlot(sample,$name,addBoundaryNum) $size
    
   set workListName $name-serieslist
   set SkelPlot(sample,$name,serieslist) $workListName    

#    upvar #0 $listName thelist
    upvar #0 $masterListName thelist
    set thelist {}

    set startyear 0

    for {set i 0} {$i < $size} {incr i} {
	    # Create an array to store info about this ring
#	    set bonename "bone-[expr $i + $SkelPlot(count,rings)]"
       set bonename "Boundary-[expr $i + $SkelPlot(count,rings)]"
	    upvar #0 $bonename thebone

	    set thebone(width) [expr 2 * [Random]]
	    set thebone(year) [expr $i + $startyear]

       
	    lappend thelist $bonename 
    }

    set SkelPlot(count,rings) [expr $SkelPlot(count,rings) + $size]

#    compute_indices $listName
#    compute_bones $var $listName


    compute_indices $var $masterListName $workListName $name
    compute_bones $var $workListName

}

proc load_ringwidth_series { var widthlist name }  {

   global findrings session zoom
   upvar #0 $var SkelPlot

   set size [llength $widthlist]


   set masterListName $name-masterlist

   set SkelPlot(sample,$name,masterlist) $masterListName
   set SkelPlot(sample,$name,size) $size
    
   set workListName $name-serieslist
   set SkelPlot(sample,$name,serieslist) $workListName    


   upvar #0 $masterListName thelist
   set thelist { }


   set tempBoundaryId -1

   for {set i 0} {$i <= $size} {incr i} {
	    # Create an array to store info about this ring accessible by index
        # and/or BoundaryId
        # This routine depends on the measurements(ringwidth) array and the
        # chainmaps being in synch with respect to

        set BoundaryId [Sadie_TREES_QueryBoundaryId $findrings(loaded,smallchainmap) $i] 
        if {$BoundaryId > $tempBoundaryId} {
           set tempBoundaryId $BoundaryId
        } 
        set bonename "Boundary-$BoundaryId"
	    upvar #0 $bonename thebone
        if {$i<$size} {
	       set thebone(width) [lindex $widthlist $i]
               #puts "The Initial Master List(width): $thebone(width)"
        }
        set type [Sadie_TREES_QueryRingType $findrings(loaded,smallchainmap) $BoundaryId]
        switch -exact $type {
           0 {set thebone(typelist) { normal }}
           1 {set thebone(typelist) { added }}
           2 {set thebone(typelist) { removed }}
        }
       
	    lappend thelist $bonename 
    }
    # puts "Largest Boundary ID is $tempBoundaryId"
    incr tempBoundaryId
     #puts "Next Added Boundary ID will be $tempBoundaryId"
    set SkelPlot(sample,$name,addBoundaryNum) $tempBoundaryId
   
    #puts "The Initial Master List: $thelist"
    compute_indices $var $masterListName $workListName $name
    compute_bones $var $workListName
    #puts "the worklist: $workListName"
}


#
# compute_indices --
#
#       Compute normalized index values from a list of measured
#       tree ring widths.
#
# Arguments:
#       ringlist -- The name of the skeleton plot
#
# Results:
#       This procedure expects the tree ring widths associated with
#       the skeleton plot to be registered in the input list.  The input
#       list must contain a list of array names, with the widths at
#       index "width".  Each index value will be recorded in the appropriate
#       list array at index "indexval".
#

# This procedure takes a master list of raw ring boundaries with widths
# at index width and transforms them into a working list for skeleton
# plotting.


proc compute_indices {var masterRingList workRingList name} {
   global WidthArray skelplot
   
   upvar #0 $var SkelPlot    
   upvar #0 $masterRingList theMasterList
   upvar #0 $workRingList theWorkList

   set theWorkList {}

   # Temporarily allow this procedure to be used in stand-alone Tcl
   # when compiled_with_c set to 0
   # Disabled JE 7-6-99
   set compiled_with_c 1
    
   if {$compiled_with_c} {

	  set size [expr [llength $theMasterList] - 1]
      puts "The master list has $size + 1 elements."

      # This loop creates a working list from the master list.
      # A completely sequential working list is created.
      # The name of the list is stored in $SkelPlot(sample,$name,serieslist)
      # The name of the elements of the list are ring-0,ring-1,ring-2,...
      # The relevant indices are (width), (year), and (typelist)
      # The width of a removed ring is added to the previous ring.
      # The width of an added ring is subtracted from the previous ring. 

      set j 0
	  for {set i 0} {$i < $size} {incr i} {

         set masterElementName [lindex $theMasterList $i] 
         set nextMasterElement [lindex $theMasterList [expr $i + 1]]
	     upvar #0 $masterElementName theMasterElement
         upvar #0 $nextMasterElement theNextMasterElement
         
         if {$i==0} {set SkelPlot(firstBoundary) $masterElementName}

         if {[lsearch -exact $theMasterElement(typelist) removed] >= 0} {

            # The boundary is a removed boundary, do not create a ring,
            # add its width to the previous measured ring.
            # Assertion: the first boundary may not be removed.

            set thePrevMeasRing(width) [expr $thePrevMeasRing(width) + $theMasterElement(width)]

         } elseif {[lsearch -exact $theMasterElement(typelist) added] >= 0} {

            # The boundary is an added boundary, create a ring using its 
            # theMasterElement(width). Then subtract its width from the previous
            # measured ring.
            # Assertion: the first boundary will not be an added ring

            set ringname "Ring-[expr $j]"
            upvar #0 $ringname theRing
            set theRing(width) $theMasterElement(width)
            set theRing(indexval) 0
            set theRing(typelist) [list added]
            set theRing(LBoundary) $masterElementName            
            set thePrevRing(RBoundary) $masterElementName
            set thePrevMeasRing(width) [expr $thePrevMeasRing(width) - $theMasterElement(width)]

            # Set the previous ring to this ring
            set prevRing $ringname
            upvar #0 $prevRing thePrevRing
            
            # If the next ring is removed then set the previous measured ring to 
            # this ring
            if {[lsearch -exact $theNextMasterElement(typelist) removed] >= 0} {
               set prevMeasRing $ringname
               upvar #0 $prevMeasRing thePrevMeasRing
           
            }
            
            lappend theWorkList $ringname
            incr j

         } else {
            # The boundary is a normal boundary, create a ring.
            
            set ringname "Ring-[expr $j]"
            upvar #0 $ringname theRing
            set theRing(width) $theMasterElement(width)
            set theRing(indexval) $theRing(width)            
            set theRing(typelist) [list normal ]
            set theRing(LBoundary) $masterElementName
            
            if {$j > 0} {
               set thePrevRing(RBoundary) $masterElementName
            }

            # Set the previous ring and the previous measured ring to this ring
            set prevRing $ringname
            upvar #0 $prevRing thePrevRing
            set prevMeasRing $ringname
            upvar #0 $prevMeasRing thePrevMeasRing
           
            lappend theWorkList $ringname
            incr j         
                  
         }

	  }
      # Get the Right Boundary for the last ring
      set masterElementName [lindex $theMasterList $i] 
      set thePrevRing(RBoundary) $masterElementName
      set SkelPlot(lastBoundary) $masterElementName
    
      # De-trend the working list of widths to create indexvalues
	  set size [llength $theWorkList]
      set WidthArray(count) $size
      puts "The working list has $size elements."
      set SkelPlot(count,rings) $size
      
      # Figure out what the endyear and startyear (theYear) should be
      if { $skelplot(endyear) == 0 } {
         set skelplot(endyear) [expr $size - 1]
         set skelplot(startyear) 0
         set SkelPlot(endyear) $skelplot(endyear)
         set theYear 0
      } else {
         set theYear [expr $skelplot(endyear) - $size + 1]
         set skelplot(startyear) $theYear
         set SkelPlot(endyear) $skelplot(endyear)
      }

      # Create the Width Array for de-trending and
      # Establish the year number for each of the rings since
      # we now know the size of the ringlist.
#      puts $theWorkList

      # Create the notes list if it does not exist (not loaded or new)
      set createNotes 0
      if {![info exists skelplot(notes)]} {
         set createNotes 1
         set skelplot(notes) {}
      }

	  for {set i 0} {$i < $size} {incr i} {
	     set ringname [lindex $theWorkList $i]
	     upvar #0 $ringname theRing
		 set WidthArray($i,width) $theRing(width)        
         set theRing(year) $theYear
#         if {![info exists theRing(notes)]} {
#            set theRing(notes) ""
#         }
         # Add blank entries to the notes list if it is being created
         if { $createNotes } {
            lappend skelplot(notes) ""
         }

#         puts "$ringname $theRing(year)"
         incr theYear
	  }

#	   puts [get_sorted_array WidthArray]
      Sadie_TREES_ComputeIndices WidthArray
#	   puts [get_sorted_array WidthArray]

	  for {set i 0} {$i < $size} {incr i} {
	     set ringname [lindex $theWorkList $i]
	     upvar #0 $ringname theRing
		 set theRing(indexval) $WidthArray($i,indexval)
	   }
   } else {
	   set size [llength $theMasterList]
       
       # This needs to be removed whenever no longer needed 

	   # This loop needs to be replaced with a call to a C procedure
	   #  that implements spline fitting or similar algorithm
      

      # This loop was revised to create a working list from the master list.
      # A completely sequential working list is created.
      # The name of the list is stored in $SkelPlot(sample,$name,serieslist)
      # The name of the elements of the list are bone-0,bone-1,bone-2,...
      # The relevant indices are (width),(year), and (typelist)
      # The width of a removed ring is added to the next ring boundary.
      # The width of an added ring is subtracted from the next ring boundary. 

      set startyear 0
      set holdwidth 0
      set j 0
	   for {set i 0} {$i < $size} {incr i} {

#	      set elementname [lindex $thelist $i]
         set masterElementName [lindex $theMasterList $i] 
	      upvar #0 $masterElementName theMasterElement

         if {[lsearch -exact $theMasterElement(typelist) removed] >= 0} {

            # The boundary is a removed boundary, do not create a ring,
            # accumulate its width to add to the next ring.
            set holdwidth [expr $holdwidth + $theMasterElement(width)]

         } elseif {[lsearch -exact $theMasterElement(typelist) added] >= 0} {

            # The boundary is an added boundary, create a ring using its width plus
            # whatever is currently in the holdwidth variable.  Put the
            # negative of its width in holdwidth to subtract from the next ring. 

            set bonename "bone-[expr $j]"
            upvar #0 $bonename thebone
            set thebone(width) [expr $theMasterElement(width) + $holdwidth]
            set thebone(indexval) $thebone(width)
            set thebone(year) [expr $j + $startyear]
            set thebone(typelist) [list added]
            set thebone(RBoundary) $masterElementName
            if {$j > 0} {
               set thebone(LBoundary) $prevRBoundary
            } else {
               set thebone(LBoundary) "None"
            }
            set prevRBoundary $masterElementName
            set holdwidth [expr 0 - $theMasterElement(width)]            
            lappend theWorkList $bonename
            incr j

         } else {
         # The boundary is a normal boundary, create a ring using its width plus
         # whatever is in holdwidth.  Reset holdwidth to zero.
            
            set bonename "bone-[expr $j]"
            upvar #0 $bonename thebone
            set thebone(width) [expr $theMasterElement(width) + $holdwidth]
            set thebone(indexval) $thebone(width)            
            set thebone(year) [expr $j + $startyear]
            set thebone(typelist) [list ]
            set thebone(RBoundary) $masterElementName
#            set theBone(notes) "Notes for Ring $j for $theBone(year)"
            if {$j > 0} {
               set thebone(LBoundary) $prevRBoundary
            } else {
               set thebone(LBoundary) "None"
            }
            set prevRBoundary $masterElementName
            set holdwidth 0            
            lappend theWorkList $bonename
            incr j         
                  
         }        
	   }
    }
}


#
# compute_bones --
#
#       Compute skeleton plot bones from a list of normalized index values.
#
# Arguments:
#       var -- Name of global skeleton plotting array
#       ringlist -- The name of the skeleton plot
#
# Results:
#       This procedure expects the tree ring index values associated with
#       the skeleton plot to be registered in the input list.  The input
#       list must contain a list of array names, with the indices at
#       index "indexval".  Each bone length and type will be recorded in the
#       appropriate list array at index "bone" and "typelist".
#
proc compute_bones {var ringlist} {
   upvar #0 $var SkelPlot
   upvar #0 $ringlist thelist

   set size [llength $thelist]
    
    
   # find maximum and minimum of series
   foreach {min max} [find_maxmin_index $ringlist] {}
    
   # Initialize bone lengths
   for {set i 0} {$i < $size} {incr i} {
	   # set default values
	   upvar #0 [lindex $thelist $i] current
	   set current(bone) 0
   }

   # Go through entire list of widths, determining whether there
   # should be a bone for that year.  If so, calculate its relative 
   # length.
   # Assertion: Removed ring widths never make it here.
       
   for {set i 1} {$i < [expr $size - 1]} {incr i} {
	   # references to current year, previous, and next
	   upvar #0 [lindex $thelist [expr $i - 1]] prev
	   upvar #0 [lindex $thelist $i] current
	   upvar #0 [lindex $thelist [expr $i + 1]] next
	
	   if {$current(indexval) < [expr (1 - (1 - $min)/$SkelPlot(abCutoff))] || \
		   [expr $current(indexval) - $next(indexval) ] < [expr $min - $max]/$SkelPlot(diffCutoff)} {
	    
	      if {$SkelPlot(abCutoff) > 1} {
		      set current(bone) [expr 1.0 - ($current(indexval) /(1 - (1 - $min)/$SkelPlot(abCutoff)))]
	      }

	      if {$SkelPlot(diffCutoff) > 1 && $current(indexval) < $next(indexval) } {
		      set currentBone $current(bone)
		      set thisBone [expr 1.0 - (($current(indexval) - $prev(indexval) ) - ($min - $max))/ \
			                (($min - $max)/$SkelPlot(diffCutoff) - ($min - $max))]

		      if {$currentBone > $thisBone} {
		         set current(bone) $currentBone
		      } else {
		         set current(bone) $thisBone
		      }
	      }
	   }
	
	   # if the width of a bone is greater than a certain fraction, denote as a wide bone
	   if {$current(indexval) > [expr $SkelPlot(wideCutoff) * $max ]} {
	       lappend current(typelist) wide
	      set current(bone) 0
	   }
	
	   # correct negative bone lengths
	   if {$current(bone) < 0} {
	       set current(bone) 0 
	   }
   }    
}

# Find the values of the minimum and maximum elements in the series
proc find_maxmin_index { ringlist } {
   upvar #0 $ringlist thelist

   set size [llength $thelist]
   set max 0.0
   set min 2.0
  
   for {set i 0} {$i < $size} {incr i} {
	   set elementname [lindex $thelist $i]
	   upvar #0 $elementname theelement

	   if {$theelement(indexval) < $min} {
	      set min $theelement(indexval)
	   }
	   if {$theelement(indexval) > $max} {
	      set max $theelement(indexval)
	   }
   }
   return [list $min $max]
}


# The following procedures are used to extract information from 
# the list of tree rings produced by the compute_indices and
# compute_bones procedures (above).

# Prints all of the data available for an entire tree ring list.
# Used for testing purposes.
 
proc print_series_info { ringlist } {
   upvar #0 $ringlist thelist
   
#   puts $thelist  
   foreach item $thelist {
      upvar #0 $item theElement
      puts $item
      puts "$theElement(width) $theElement(typelist) \
            $theElement(year) $theElement(bone) \
            $theElement(LBoundary) $theElement(RBoundary) \
            $theElement(indexval) $theElement(notes)"
   }
}


# Returns the width value of a ring
proc getWidth { ringlist ringnum } {

   upvar #0 $ringlist theList
   set item [lindex $theList $ringnum]
   upvar #0 $item theElement
   return $theElement(width)

}

# Returns the indexval value of a ring.
proc getIndexval { ringlist ringnum } {

   upvar #0 $ringlist theList
   set item [lindex $theList $ringnum]
   upvar #0 $item theElement
   return $theElement(indexval)

}

# Returns the tag and array name of the right boundary of a ring 
proc getRBoundary { ringlist ringnum } {

   upvar #0 $ringlist theList
   set item [lindex $theList $ringnum]
   upvar #0 $item theElement
   return $theElement(RBoundary)

}

# Returns the tag and array name of the left boundary of a ring.
proc getLBoundary { ringlist ringnum } {

   upvar #0 $ringlist theList
   set item [lindex $theList $ringnum]
   upvar #0 $item theElement
   return $theElement(LBoundary)

}

# Returns the typelist values of a tree ring {generally a list].
proc getTypelist { ringlist ringnum } {

   upvar #0 $ringlist theList
   set item [lindex $theList $ringnum]
   upvar #0 $item theElement
   return $theElement(typelist)

}

# Returns the year value of a tree ring.
proc getYear { ringlist ringnum } {

   upvar #0 $ringlist theList
   set item [lindex $theList $ringnum]
   upvar #0 $item theElement
   return $theElement(year)

}

# Returns the index value of a tree ring given a year, or 
# -1 if yearnum is not found.
proc getIndex { ringlist yearnum } {
   upvar #0 $ringlist theList
   set i 0
   foreach item $theList {   
      upvar #0 $item theElement
      if { $theElement(year) == $yearnum } {
         return $i      
      } else {
         incr i
      }      
   }
   return -1
}

# Returns the skeleton plot bone length associated with a ring (if any).
proc getBonelength { ringlist ringnum } {

   upvar #0 $ringlist theList
   set item [lindex $theList $ringnum]
   upvar #0 $item theElement
   return $theElement(bone)
}

# Returns the Analyst Notes associated with a ring at a particular index.
proc getRingNotes { ringnum } {
   global skelplot
   
   set skelplot_notes [lindex $skelplot(notes) $ringnum]
   set notes ""
   foreach item $skelplot_notes {
      append notes $item
      append notes "\n"
   }   
   return $notes
}

# Sets the Analyst Notes associated with a ring at a particular index.
proc setRingNotes { ringnum notes } {
   global skelplot

#   puts "Before: [lindex $skelplot(notes) $ringnum]"
   set skelplot(notes) [lreplace $skelplot(notes) $ringnum $ringnum $notes]
#   puts "After: [lindex $skelplot(notes) $ringnum]"

}

# Inserts an Analyst Note for a ring at a particular index.
# Used by the add_ring function to create a new (blank) note for a 
# new ring after adding it.
proc insertRingNote { ringnum notes } {
   global skelplot
   
   set skelplot(notes) [linsert $skelplot(notes) $ringnum $notes]

}

# Deletes an Analyst Note for a ring at a particular index (even if blank).
# Used by the remove_ring function to delete the note at a particular index 
# after removing the ring.
proc deleteRingNote { ringnum } {
   global skelplot
   
   set skelplot(notes) [lreplace $skelplot(notes) $ringnum $ringnum]

}

# Updates the plotdata list variables from the current
# SkelPlot(sample,series,serieslist) array

proc createPlotFileData { ringlist } {
   global plotdata SkelPlot skelplot
   
   upvar #0 $ringlist theList
   
   set plotdata(status) COMPLETE
   set plotdata(size) $SkelPlot(count,rings)
   set plotdata(abCutoff) $SkelPlot(abCutoff)
   set plotdata(diffCutoff) $SkelPlot(diffCutoff)
   set plotdata(year) {}
   set plotdata(LBoundary) {}
   set plotdata(RBoundary) {}
   set plotdata(width) {}
   set plotdata(indexval) {}
   set plotdata(bone) {}
   set plotdata(typelist) {}
#   set plotdata(notes) $skelplot(notes)
   
   for {set i 0} {$i < $plotdata(size)} {incr i} {
      set ringName [lindex $theList $i]
      upvar #0 $ringName theRing
      
      lappend plotdata(year) $theRing(year)
      lappend plotdata(LBoundary) $theRing(LBoundary)
      lappend plotdata(RBoundary) $theRing(RBoundary)
      lappend plotdata(width) $theRing(width)
      lappend plotdata(indexval) $theRing(indexval)
      lappend plotdata(bone) $theRing(bone)
      lappend plotdata(typelist) $theRing(typelist)
   }
}
