# skelplot_plot.tcl - Skeleton Plot plotting functions.

# Procedure Index:
# Removed JE 7-9-99 #proc create_main_skelplot { win top timeseries plotseries bottom scroll } 
# proc create_skelplot_canvas { var win top timeseries seriesname bottom scroll } 
# proc configure_xscroll { win seriesname scroll } 
# proc redraw_skelplot { var win seriesname } 
# proc create_skelplot_grid { var win seriesname } 
# proc draw_skelplot_data { var win seriesname } 
# proc create_skelplot_controls { var win skelplot timeseries seriesname } 
# proc create_timeseries_canvas { var win timeseries seriesname } 
# proc update_timeseries { var win seriesname } 
# proc create_timeseries_grid { var win timeseries seriesname } 
# proc draw_timeseries_data { var win timeseries seriesname } 
# proc create_timeseries_controls_canvas { var win } 
# proc create_bindings {var skelplot timeseries name} 
# proc position_start_set {var x} 
# proc position_update {var skelplot timeseries name x} 
# proc move_skelplot {var skelplot timeseries name steps} 
# proc BindXview { lists args } 
# proc BindYview { lists args } 
# proc create_marker_canvas { win toppath timeseriesname seriesname bottompath } 
# proc bindkey { var win seriesname top bottom timeseries imagecanvas} 
# proc execute_key_bind { var key win seriesname top bottom timeseries imagecanvas} 
# proc bindmouse { var win path top series timeseries bottom imagecanvas} 
# proc execute_mouse_bind { var x y win top timeseries series bottom imagecanvas} 
# proc select_ring { path series toppath bottompath timeseries imagecanvas} 
# proc draw_skelplot { var win top timeseries seriesname bottom scroll imagecanvas} 
# proc redraw_skelplot { var win top timeseries seriesname bottom scroll imagecanvas} 


#--------------------------------------------------------------------


# Creates the main canvas for skeleton plot and timeseries plot window 
# Creates the x scroll bar and binds it to the canvases contained.
# Removed JE 7-9-99 Not needed for trees program 
#proc create_main_skelplot { win top timeseries plotseries bottom scroll } {
#   frame $win -borderwidth 2 -relief ridge -bg blue
#   scrollbar $win.$scroll \
#   	-orient horizontal \
#   	-command [subst {[list BindXview \
#                "$win.$top $win.$timeseries $win.$plotseries \
#                $win.$bottom" ] \
#                }] 
#}    


# Creates the canvas for the skeleton plot section of the display.
# Assumes that win is the name of the main frame - .trees_main.skelplotpanel
# (See also create_timeseries_canvas)
proc create_skelplot_canvas { var win top timeseries seriesname bottom scroll } {
    upvar #0 $var SkelPlot

    set scale $SkelPlot(scale)
    set boxes $SkelPlot(unitboxes,skelplot)
    upvar #0 $SkelPlot(sample,$seriesname,serieslist) serieslist
    set count [llength $serieslist]

    set width [expr (($count+5) * $scale)+1]
    set height [expr ($boxes * $scale) + 51]
    
    canvas $win.$seriesname -width $width -height $height \
	    -highlightthickness 0 \
	    -background white \
	    -confine true \
	    -cursor dotbox \
	    -xscrollincrement $scale \
	    -scrollregion "0 0 $width $height" 
    scrollbar $win.$scroll \
     -background LightGray \
   	 -orient horizontal \
   	 -command [subst {[list BindXview \
                "$win.$top $win.$timeseries $win.$seriesname \
                $win.$bottom" ] \
                }] 

    lappend SkelPlot(plotlist) $seriesname

}

# Configures the x scroll bar for the skeleton plot 
proc configure_xscroll { win seriesname scroll } {

   $win.$seriesname configure -xscrollcommand "$win.$scroll set"
}


# Redraws skeleton plot grid lines and bars.
#proc redraw_skelplot { var win seriesname } {
#    $win delete all
#    create_skelplot_grid $var $win $seriesname
#    draw_skelplot_data $var $win $seriesname
#}

# Creates horizontal and vertical gridlines for skeleton plot section.
# (See also create_timeseries_grid)
proc create_skelplot_grid { var win seriesname } {
    upvar #0 $var SkelPlot
    global skelplot

    set scale $SkelPlot(scale)
    set boxes $SkelPlot(unitboxes,skelplot)
    set invert $SkelPlot(sample,$seriesname,invert)
    
    upvar #0 $SkelPlot(sample,$seriesname,serieslist) serieslist

    set count [llength $serieslist]
    set SkelPlot(size,actual) $count
    set SkelPlot(size,display) $count

    set startindex [lindex $serieslist 0]
    upvar #0 $startindex array
    set startyear $array(year)
    set skelplot(startyear) $startyear
    set offset [expr $startyear % 5]

    set height [$win.$seriesname cget -height]
    set width [$win.$seriesname cget -width]

    # display the current values of absolute value cutoff and 
    # first difference values
    set x [expr 1.5*$scale]
    set y [expr 2*$scale]
    $win.$seriesname create text $x $y -text "AB:"
    set y [expr $y + 1.5*$scale]
    $win.$seriesname create text $x $y -text "$SkelPlot(abCutoff)"
    set y [expr $y + 1.5*$scale]
    $win.$seriesname create text $x $y -text "FD:"
    set y [expr $y + 1.5*$scale]    
    $win.$seriesname create text $x $y -text "$SkelPlot(diffCutoff)"

    # draw vertical lines 
    set y0 0
    set y1 $height
    for {set x 0} { $x < $count } { incr x } {
	    # if at tick mark or end of graph, draw dark green line
	    set sclX [expr ($x+3) * $scale]
	    if {[expr ($x+$offset) % 5] == 0} {
	        $win.$seriesname create line $sclX $y0 $sclX $y1 \
		        -fill darkgreen -tags [list vgridlines i-$x d-$x]
	        if ($invert) {
		        set textid [$win.$seriesname create text $sclX [expr $height - 10] -text [expr $x+$startyear] -tag text]
	        } else {
		        set textid [$win.$seriesname create text $sclX 10 -text [expr $x+$startyear] -tag text]
	        }

	        # Put a white background behind text so easier to read
	        foreach {l t r b} [$win.$seriesname bbox $textid] {}
	        $win.$seriesname create rectangle $l $t $r $b -fill white \
		        -width 1 -outline white -tag textbg

	    } else {
	        $win.$seriesname create line $sclX $y0 $sclX $y1 \
		        -fill green3 -tag [list vgridlines i-$x d-$x]
	    }
    } 
  
    # draw horizontal lines
    set x0 [expr 3*$scale]
    set x1 [expr $width - (3*$scale)]
    set max [expr round(double($height) / $scale)]
    for {set y 0} { $y <= $max } { incr y } {
	    if {$invert} {
	        set sclY [expr $y * $scale]
	    } else {
	        set sclY [expr ($height - 1) - ($y * $scale)]
	    }

	    # if at tick mark, draw dark green line  
	    if {$y % 5 == 0} {
	        $win.$seriesname create line $x0 $sclY $x1 $sclY \
		        -fill darkgreen -tag hgridlines
	    } else {
	        $win.$seriesname create line $x0 $sclY $x1 $sclY \
		        -fill green3 -tag hgridlines
	    }
    }

    $win.$seriesname raise textbg
    $win.$seriesname raise text

}

proc draw_skelplot_data { var win seriesname } {
    global boundarycolor skelplot
    upvar #0 $var SkelPlot

    set scale $SkelPlot(scale)
    set boxes $SkelPlot(unitboxes,skelplot)
    set invert $SkelPlot(sample,$seriesname,invert)
    
    upvar #0 $SkelPlot(sample,$seriesname,serieslist) serieslist
    set count [llength $serieslist]
#    puts $SkelPlot(sample,$seriesname,serieslist)
    set height [$win.$seriesname cget -height]

    for {set i 0} {$i < $count} {incr i} {
	    upvar #0 [lindex $serieslist $i] element

#        puts $element(typelist)
         
	    set sclX [expr ($i+3) * $scale]
	    if {$element(bone) > 0} {
	       if {$invert} {
		       set y1 0
		       set y2 [expr $boxes * $scale * $element(bone)]
	       } else {
		       set y1 [expr ($height - 1)-($boxes * $scale * $element(bone))]
		       set y2 [expr ($height - 1)]
	       }

	    $win.$seriesname create line $sclX $y1 $sclX $y2 \
		    -fill $boundarycolor(plotbones) -tag bones -width 2
	    }

       # invert = 1 means upside down, default is 0
	    if {[lsearch $element(typelist) "wide"] >= 0} {
	       if {$invert} {
#		       set y [expr ($boxes * $scale) + 10]
             set y [expr ($boxes * $scale) + 10]
	       } else {
#		       set y [expr ($height - 11)-($boxes * $scale)]
             set y [expr ($height - 11)-($boxes * $scale)]
	       }
	       $win.$seriesname create text $sclX $y -text "b" -tag bones
	     }
	    if {[lsearch $element(typelist) "added"] >= 0} {
	       if {$invert} {
#		       set y [expr ($boxes * $scale) + 10]
             set y [expr ($boxes * $scale) + 20 ]
	       } else {
#		       set y [expr ($height - 11)-($boxes * $scale)]
             set y [expr ($height - 22)-($boxes * $scale)]
	       }
	       $win.$seriesname create text $sclX $y -text "a" -tag bones
	     }        
        set notes [lindex $skelplot(notes) $i] 
	    if { [string length [string trimright $notes]] } {
	       if {$invert} {
#		       set y [expr ($boxes * $scale) + 10]
             set y [expr ($boxes * $scale) + 30 ]
	       } else {
#		       set y [expr ($height - 11)-($boxes * $scale)]
             set y [expr ($height - 31)-($boxes * $scale)]
	       }
	       $win.$seriesname create text $sclX $y -text "x" -tag bones
	     }        
         
    }

    $win.$seriesname raise bones
}

# Creates a set of control widgets for the skeleton plot canvas when
# plotting multiple skeleton plots.
proc create_skelplot_controls { var win skelplot timeseries seriesname } {
    upvar #0 $var SkelPlot
    
    set scale $SkelPlot(scale)
    set boxes $SkelPlot(unitboxes,skelplot)
    set myfont [font create -family fixed -size 8]
    set bg wheat

    set width 100
    set height [expr ($boxes * $scale) + 41]
    frame $win -width $width -height $height \
	    -bg $bg -bd 2 -relief ridge

    label $win.title -text "Sample: $seriesname" -bg $bg -font myfont
    
    label $win.offsetlbl -text "Offset: " -bg $bg -font myfont
    label $win.offset -textvariable ${var}(sample,$seriesname,position) -bg $bg -font myfont

    checkbutton $win.invert -text "Invert" -onvalue 1 -offvalue 0 \
	    -variable ${var}(sample,$seriesname,invert) -font myfont -bg $bg \
	    -highlightcolor $bg -highlightthickness 0 -activebackground $bg \
	    -command "redraw_skelplot $var $skelplot $seriesname"

    checkbutton $win.timeseries -text "Time Series" -onvalue 1 -offvalue 0 \
	    -variable ${var}(sample,$seriesname,timeseries) -font myfont -bg $bg \
	    -highlightcolor $bg -highlightthickness 0 -activebackground $bg \
	    -command "update_timeseries $var $timeseries $seriesname"

    button $win.color -bg $SkelPlot(sample,$seriesname,color) -bitmap gray12 \
	    -activebackground $SkelPlot(sample,$seriesname,color) \
	    -activeforeground $SkelPlot(sample,$seriesname,color) \
	    -foreground $SkelPlot(sample,$seriesname,color) \
	    -width 5 -height 5 \
	    -command [subst -nocommands {
	         global $var
            set choosecolor [tk_chooseColor -initialcolor \$${var}(sample,$seriesname,color)]
	         if {\$choosecolor != ""} {
			      set ${var}(sample,$seriesname,color) \$choosecolor
			      update_timeseries $var $timeseries $seriesname
			      $win.color configure -activebackground \$choosecolor \
				      -activeforeground \$choosecolor \
				      -foreground \$choosecolor \
				      -bg \$choosecolor
		      }
            }]
    
    
    grid $win.title -row 0 -column 0 -columnspan 3 -sticky w
    grid $win.offsetlbl -row 1 -column 0 -sticky w
    grid $win.offset -row 1 -column 1 -sticky w
    grid $win.invert -row 2 -column 0 -columnspan 2 -sticky w
    grid $win.timeseries -row 3 -column 0 -columnspan 2 -sticky w
    grid $win.color -row 3 -column 2 -sticky w
}

# Creates the canvas for the timeseries section of the skeleton plot.
# (See also create_skelplot_canvas)
# Modified JE 7-9-99 to create a timeseries canvas for only one series
proc create_timeseries_canvas { var win timeseries seriesname } {
    upvar #0 $var SkelPlot

    set scale $SkelPlot(scale)
    set boxes $SkelPlot(unitboxes,timeseries)

#    if {[llength $SkelPlot(plotlist)] > 0} {
#	   set seriesname [lindex $SkelPlot(plotlist) 0]
#	   upvar #0 $SkelPlot(sample,$seriesname,serieslist) serieslist
#	   set count [llength $serieslist]
#    } else {
#	   set count 200
#    }

	 upvar #0 $SkelPlot(sample,$seriesname,serieslist) serieslist
	 set count [llength $serieslist]
    
    set width [$win.$seriesname cget -width]
    set height [expr (2 * $boxes * $scale) + 1]
    canvas $win.$timeseries -width $width -height $height \
	    -highlightthickness 0 \
	    -background white \
	    -confine true \
       -cursor dotbox \
	    -xscrollincrement $scale \
	    -scrollregion "0 0 $width $height"
       
}

# Draws or removes the timeseries section. 
proc update_timeseries { var win seriesname } {
    upvar #0 $var SkelPlot

    $win itemconfigure $seriesname -fill $SkelPlot(sample,$seriesname,color)
    
    if {$SkelPlot(sample,$seriesname,timeseries)} {
	    $win raise $seriesname
    } else {
	    $win lower $seriesname background
    }

}

# Creates horizontal and vertical gridlines for the timeseries section
# of the skeleton plot.
# (See also create_skelplot_grid)
# Modified JE 7-9-99 to create a timeseries grid for only one series
proc create_timeseries_grid { var win timeseries seriesname } {
    upvar #0 $var SkelPlot

    upvar #0 $SkelPlot(sample,$seriesname,serieslist) serieslist
    set startindex [lindex $serieslist 0]
    upvar #0 $startindex array
    set startyear $array(year)
    set offset [expr $startyear % 5]


    set scale $SkelPlot(scale)
    set boxes $SkelPlot(unitboxes,timeseries)
    
    set height [$win.$timeseries cget -height]
    set width [$win.$timeseries cget -width]
  
    $win.$timeseries create rectangle 0 0 $width $height -fill white \
	    -width 1 -outline white -tag background
    $win.$timeseries lower background

#    if {[llength $SkelPlot(plotlist)] > 0} {
#	 set seriesname [lindex $SkelPlot(plotlist) 0]

	 upvar #0 $SkelPlot(sample,$seriesname,serieslist) serieslist
	 set count [llength $serieslist]

#    } else {
#	    set count 200
#    }
    
    # draw vertical lines 
    set y0 0
    set y1 $height

    for {set x 0} { $x < $count } { incr x } {
	    # if at tick mark or end of graph, draw dark green line
	    set sclX [expr  ($x+3) * $scale]
	    if {[expr ($x+$offset) % 5] == 0} {
	       $win.$timeseries create line $sclX $y0 $sclX $y1 \
		       -fill darkgreen -tag [list vgridlines i-$x d-$x]

           # Create a text label at bottom of timeseries    
           set textid [$win.$timeseries create text $sclX 10 -text [expr $x+$startyear] -tag text]    
           # Put a white background behind text so easier to read
	       foreach {l t r b} [$win.$timeseries bbox $textid] {}
	       $win.$timeseries create rectangle $l $t $r $b -fill white \
		        -width 1 -outline white -tag textbg               
               
	    } else {
	       $win.$timeseries create line $sclX $y0 $sclX $y1 \
		       -fill green3 -tag [list vgridlines i-$x d-$x]
	    }
    } 
    
    # draw horizontal lines
    set x0 [expr 3 * $scale]
    set x1 [expr $width - (3*$scale)]
    set unit [expr $boxes * $scale]
    $win.$timeseries create line $x0 $unit $x1 $unit \
	    -fill black -width 2  -tag hgridlines

    for {set y 1} { $y <= $boxes } { incr y } {
	    set sclY1 [expr $unit + ($y * $scale)]
	    set sclY2 [expr $unit - ($y * $scale)]
	    # if at tick mark, draw dark green line  
	    if {$y % 5 == 0} {
	       $win.$timeseries create line $x0 $sclY1 $x1 $sclY1 \
		       -fill darkgreen -tag hgridlines
	       $win.$timeseries create line $x0 $sclY2 $x1 $sclY2 \
		       -fill darkgreen -tag hgridlines
	    } else {
	       $win.$timeseries create line $x0 $sclY1 $x1 $sclY1 \
		       -fill green3 -tag hgridlines
	       $win.$timeseries create line $x0 $sclY2 $x1 $sclY2 \
		       -fill green3 -tag hgridlines
	    }
    }
    $win.$timeseries create text [expr 3*$scale] $unit -anchor e -text "1.0 " -tag axis
    $win.$timeseries create rectangle 0 0 [expr (3*$scale) - 1] $height -fill white \
	    -width 1 -outline white -tag white
    $win.$timeseries raise white
    $win.$timeseries raise text
    $win.$timeseries raise axis
    if { $SkelPlot(sample,$seriesname,timeseries) == 2 } {
       $win.$timeseries raise textbg
       $win.$timeseries raise text    
    } else {
       $win.$timeseries lower textbg
       $win.$timeseries lower text    
    }     
}

# Draws timeseries graph lines onto the timeseries grid.
proc draw_timeseries_data { var win timeseries seriesname } {
    global skelplot

    upvar #0 $var SkelPlot
    upvar #0 $SkelPlot(sample,$seriesname,serieslist) serieslist

    set scale $SkelPlot(scale)
    set boxes $SkelPlot(unitboxes,timeseries)
    set unit [expr $boxes * $scale]
    set color $SkelPlot(sample,$seriesname,color)
    set height [$win.$timeseries cget -height]
    
    set size [llength $serieslist]

    set i 0
    upvar #0 [lindex $serieslist $i] array
    set left $array(indexval)
    set y4 [expr $height - 13]
    set y5 [expr $height - 13]
    set y6 [expr $height -  5]
    for {set i 1} {$i < $size} {incr i} {
	    upvar #0 [lindex $serieslist $i] array
        if { $i == 1 } {
           upvar #0 [lindex $serieslist 0] first
        }
	    set right $array(indexval)
	
	    # get coordinates of points to connect
	    set y1 [expr round($unit - ($boxes * ($left-1) * $scale))]
	    set y2 [expr round($unit - ($boxes *  ($right-1) * $scale))]
	    set x1 [expr ($i+2) * $scale]
	    set x2 [expr ($i+3) * $scale]
	    $win.$timeseries create line $x1 $y1 $x2 $y2 -fill black -width 1 \
		    -fill $color -tag $seriesname

	    set left $right
        
        # Create timeseries annotation labels
#        puts "$i notes length [string length [string trimright $array(notes)]]"
        if {$i == 1} {
           if {[lsearch $first(typelist) "wide"] >= 0} {
	          $win.$timeseries create text $x1 $y4 -text "b" -tag label
	       }
	       if {[lsearch $first(typelist) "added"] >= 0} {
	          $win.$timeseries create text $x1 $y5 -text "a" -tag label
	       }        
           set notes [lindex $skelplot(notes) 0]
	       if { [string length [string trimright $notes]] } {
	          $win.$timeseries create text $x1 $y6 -text "x" -tag label
	       }        
        }
	    if {[lsearch $array(typelist) "wide"] >= 0} {
	       $win.$timeseries create text $x2 $y4 -text "b" -tag label
	     }
	    if {[lsearch $array(typelist) "added"] >= 0} {
	       $win.$timeseries create text $x2 $y5 -text "a" -tag label
	     }        
        set notes [lindex $skelplot(notes) $i] 
	    if { [string length [string trimright $notes]] } {
	       $win.$timeseries create text $x2 $y6 -text "x" -tag label
	     }                
    }

    # Raise timeseries annotation labels only if timeseries is
    # all that is currently displayed
    if { $SkelPlot(sample,$seriesname,timeseries) == 2 } {
       $win.$timeseries raise label
    } else {
       $win.$timeseries lower label
    }       
}

# Creates a set of controls for the timeseries section of the skeleton
# plot.  Used when multiple timeseries are to be plotted. (Cross Dating)
proc create_timeseries_controls_canvas { var win } {
    upvar #0 $var SkelPlot

    set scale $SkelPlot(scale)
    set boxes $SkelPlot(unitboxes,timeseries)

    set width 100
    set height [expr (2 * $boxes * $scale) + 1]
    canvas $win -width $width -height $height \
	    -highlightthickness 0 \
	    -background white
}

# Creates bindings for dragging skeleton plots across the screen.
proc create_bindings {var skelplot timeseries name} {
    bind $skelplot <Button-1> "position_start_set $var %x"
    bind $skelplot <B1-Motion> "position_update $var $skelplot $timeseries $name %x"
}

# Sets the starting position of the skeleton plot.
proc position_start_set {var x} {
    upvar #0 $var SkelPlot
    set SkelPlot(position,start) $x
}

# Updates the position of the skeleton plot on the screen after dragging.
proc position_update {var skelplot timeseries name x} {
    upvar #0 $var SkelPlot

    set scale $SkelPlot(scale)
    set delta [expr $x - $SkelPlot(position,start)]


    if {[expr abs($delta)] > [expr $scale * 0.8]} {
	    if {$delta > 0} {
	       set motion [expr $scale + ((($delta-round($scale/2.0))/$scale)*$scale)]
	    } else {
	       set motion [expr 0 - (((round($scale/2.0)-$delta)/$scale)*$scale)]
	    }
	    move_skelplot $var $skelplot $timeseries $name [expr $motion/$scale]
    }
}

# Moves the skeleton plot and the timeseries after dragging.
proc move_skelplot {var skelplot timeseries name steps} {
    upvar #0 $var SkelPlot

    set scale $SkelPlot(scale)
    set motion [expr $steps * $scale]

    set parent [winfo parent $skelplot]
    set top [lindex $SkelPlot(plotlist) 0]

    if {$name == $top} {
	    set isteps [expr 0-$steps]
	    set imotion [expr 0-$motion]
	    for {set i 1} {$i < [llength $SkelPlot(plotlist)]} {incr i} {
	       set thename [lindex $SkelPlot(plotlist) $i]
	       incr SkelPlot(sample,$thename,position) $isteps
	       $parent move $thename $imotion 0
	       $timeseries move $thename $imotion 0
	    }

	    set initial [$parent xview]
	    $parent xview scroll $isteps units
	    $timeseries xview scroll $isteps units
	    if {[$parent xview] == $initial} {
	       # if cannot scroll, update mouse position
	       incr SkelPlot(position,start) $motion
	    }
       } else {
	    incr SkelPlot(sample,$name,position) $steps
	    $parent move $name $motion 0
	    $timeseries move $name $motion 0
       }

    if {$SkelPlot(sample,$name,timeseries)} {
	    $timeseries raise $name
    } else {
	    $timeseries lower $name background
    }
    
    $timeseries raise white
    $timeseries raise text

    # Make sure that the plot has moved before allowing another mouse
    # motion binding call to this function.
    update idletasks
}

# Binds multiple windows together for scrolling in the x direction.
proc BindXview { lists args } {
    foreach l $lists {
        eval {$l xview} $args
    }
}

# Binds multiple windows together for scrolling in the y direction.
proc BindYview { lists args } {
    foreach l $lists {
        eval {$l yview} $args
    }
}

# Creates the upper and lower canvases in the skeleton plot section
# which contain the red-triangular markers.  Creates the initial markers.
proc create_marker_canvas { win toppath timeseriesname seriesname bottompath } {

   global SkelPlot

   set scale $SkelPlot(scale)
   set width [$win.$seriesname cget -width] 
   set height [expr 2 * $scale]
   set current $SkelPlot(count,current)
   set year [getYear $SkelPlot(sample,$seriesname,serieslist) $SkelPlot(count,current)]
   set x0 [expr ($current + 3) * $scale]
   set y1 [expr 2 * $scale - 3]  

   # Create topmarker canvas
   canvas $win.$toppath \
      -height $height \
      -width $width \
      -background white  \
      -confine true \
      -xscrollincrement $scale \
      -scrollregion [subst {0 0 $width $height}]
  
   # Create bottommarker canvas
   canvas $win.$bottompath \
      -height $height \
      -width $width \
      -background white \
      -confine true \
      -xscrollincrement $scale \
      -scrollregion [subst {0 0 $width $height}]

   # Create initial markers
   $win.$bottompath create poly $x0 3 \
      [expr $x0 - 0.5 * $scale] $y1 [expr $x0 + 0.5 * $scale] $y1 \
      -outline black -fill red -tag marker
   $win.$bottompath create text [expr $x0 + 1.5 * $scale ] $scale \
      -text "\[$year\]" -anchor w -tag text
   $win.$toppath create poly $x0 $y1 \
      [expr $x0 - 0.5 * $scale] 3 [expr $x0 + 0.5 * $scale] 3 \
      -outline black -fill red -tag marker
   $win.$toppath create text [expr $x0 + 1.5 * $scale] $scale \
      -text "\[$year\]" -anchor w -tag text                
} 

# Create keyboard binding to move marker selection

proc bindkey { var win seriesname top bottom timeseries imagecanvas} {

   bind .trees_main <Any-Key>  \
      "execute_key_bind $var %K $win $seriesname $top $bottom $timeseries $imagecanvas"
      
}

# Execute key binding to move the marker selection based on the user
# pressing the up, down, right, or left arrow keys.

proc execute_key_bind { var key win seriesname top bottom timeseries imagecanvas} {
 
   upvar #0 $var SkelPlot
   upvar #0 $SkelPlot(sample,$seriesname,serieslist) serieslist

   set count [llength $serieslist]
   set current $SkelPlot(count,current)

   if {$key == "Right" || $key == "Up"} then {
      if {$current >= 0 && $current < [expr $count - 1]} then {
         unhighlight_image_ring $imagecanvas $SkelPlot(count,current)          
         set SkelPlot(count,current) [expr $current + 1]
         set SkelPlot(request) [getYear $SkelPlot(sample,$seriesname,serieslist) $SkelPlot(count,current)]
         select_ring $win $seriesname $top $bottom $timeseries $imagecanvas    
     }      
   } elseif {$key == "Left" || $key == "Down"} then {
      if {$current > 0} then {
         unhighlight_image_ring $imagecanvas $SkelPlot(count,current) 
         set SkelPlot(count,current) [expr $current - 1]
         set SkelPlot(request) [getYear $SkelPlot(sample,$seriesname,serieslist) $SkelPlot(count,current)]            
         select_ring $win $seriesname $top $bottom $timeseries $imagecanvas           
      }
     }  
}


# Creates the mouse bindings to move the currently selected ring 
# when a user clicks the first mouse button on a vertical grid line
# in either the timeseries or skeleton plot sections.

proc bindmouse { var win path top series timeseries bottom imagecanvas} {

   global SkelPlot
    
   bind "$win.$path" <Button-1> \
      "execute_mouse_bind $var %x %y $win $top $timeseries $series $bottom $imagecanvas"

}

# Execute the binding which causes the mouse to move the currently
# selected ring based on the user clicking a vertical line anywhere
# on the skeleton plot.

proc execute_mouse_bind { var x y win top timeseries series bottom imagecanvas} {

#   upvar #0 $var SkelPlot
   global SkelPlot
  
   set scale $SkelPlot(scale)
   set rect [expr $scale / 2 ]

#   if { $SkelPlot(sample,$series,timeseries) < 2 } {
      set windowoffset [expr [lindex [$win.$series xview] 0]*[$win.$series cget -width]]
      set x0 [expr $x - $rect + 1 + $windowoffset]
      set x1 [expr $x + $rect - 1 + $windowoffset]
      set y0 [expr $y + $rect - 1 ]
      set y1 [expr $y - $rect + 1 ]
      set ROIitems [$win.$series find overlapping $x0 $y0 $x1 $y1]
#   } else {
#      set windowoffset [expr [lindex [$win.$timeseries xview] 0]*[$win.$timeseries cget -width]]
#      set x0 [expr $x - $rect + 1 + $windowoffset]
#      set x1 [expr $x + $rect - 1 + $windowoffset]
#      set y0 [expr $y + $rect - 1 ]
#      set y1 [expr $y - $rect + 1 ]
#      set ROIitems [$win.$timeseries find overlapping $x0 $y0 $x1 $y1]
#   }   

   foreach item $ROIitems {
      set tag [$win.$series gettags $item]
      set index [lsearch $tag vgridlines]

      if { $index >= 0 } {
         set itag [lindex $tag [lsearch -regexp $tag {i-[0-9]+}]]
         set dtag [lindex $tag [lsearch -regexp $tag {d-[0-9]+}]]
         regsub i- $itag "" itag
         regsub d- $dtag "" dtag
         unhighlight_image_ring $imagecanvas $SkelPlot(count,current)           
         set SkelPlot(count,current) $dtag
         set SkelPlot(request) [getYear $SkelPlot(sample,$series,serieslist) $SkelPlot(count,current)] 
         select_ring $win $series $top $bottom $timeseries $imagecanvas
      }
   }
} 


# Moves the red-triangular markers on the top and bottom of the
# skeleton plot based on the value of $SkelPlot(count,current) -
# which is the current index of the ring to DISPLAY.
proc select_ring { path series toppath bottompath timeseries imagecanvas} {
   
   global Segstart Segend SkelPlot guicount measurements Seg_jion_end1 Seg_jion_start2 global_endyear
   set ring_length [expr [llength $measurements(ringwidth)]-1]
   set selectfocus [focus]
  # puts "selectfocus is: $selectfocus"
    if {[string match \.segmentationgui* $selectfocus]} {
      if {$guicount>10} { 
      set inter_number [string range $selectfocus end-1 end]
      } else {set inter_number [string index $selectfocus end] }
    
    if {$global_endyear==$ring_length} {
    set offset 0
    set select_number [expr $inter_number + $offset]
    set start $Segstart($select_number)
    set end $Segend($select_number)
    
    if {[string match *jion_* $selectfocus]} {
       if {[expr $SkelPlot(count,current)+$start]>$Seg_jion_end1($select_number)} {
        set start [expr $Seg_jion_start2($select_number)-$Seg_jion_end1($select_number)+$start-1]
        }
     }
    } else { 
    set offset 0
    set select_number [expr $inter_number + $offset]
    set start [expr $Segstart($select_number)+$global_endyear-$ring_length]
    set end [expr $Segend($select_number)+$global_endyear-$ring_length]
    
    if {[string match *jion_* $selectfocus]} {
       if {[expr $SkelPlot(count,current)+$start-$global_endyear+$ring_length]>$Seg_jion_end1($select_number)} {
        set start [expr $Seg_jion_start2($select_number)-$Seg_jion_end1($select_number)+$start-1]
        #puts "start after into jion branch is : $start"
        }
     }
  
    
   }
   } elseif {[string match \.trees_main* $selectfocus] } {
       if {$global_endyear==$ring_length} {
        set start 0
        set end 0
       } else {
        set end $global_endyear
        set start [expr $global_endyear-$ring_length]
      }
       
} else {puts "error"
        return -1
  }
   
   
   set scale $SkelPlot(scale)
   set current $SkelPlot(count,current)
   set year [expr $current+$start]
   #set SkelPlot(count,year) [getYear $SkelPlot(sample,$series,serieslist) $SkelPlot(count,current)]
   #set year $SkelPlot(count,year)
   set SkelPlot(count,leftboundary) [getLBoundary $SkelPlot(sample,$series,serieslist) [expr $SkelPlot(count,current)\
                                    +$start-$global_endyear+$ring_length]]
   set SkelPlot(count,rightboundary) [getRBoundary $SkelPlot(sample,$series,serieslist) [expr $SkelPlot(count,current)\
                                      +$start-$global_endyear+$ring_length]]
   set SkelPlot(count,width) [format %-108.4f [getWidth $SkelPlot(sample,$series,serieslist) $SkelPlot(count,current)]]
   set SkelPlot(count,indexval) [format %-10.4f [getIndexval $SkelPlot(sample,$series,serieslist) $SkelPlot(count,current)]]
   set SkelPlot(count,bonelength) [format %-10.4f [getBonelength $SkelPlot(sample,$series,serieslist) $SkelPlot(count,current)]]  
   set SkelPlot(count,typelist) [getTypelist $SkelPlot(sample,$series,serieslist) $SkelPlot(count,current)]
   set SkelPlot(count,notes) [getRingNotes $SkelPlot(count,current)]

   # Update the Attributes Window (if it exists) with the current ring's notes
   if { [winfo exists .trees_main.attributes] } {
      .trees_main.attributes.info.notes delete 1.0 end
      .trees_main.attributes.info.notes insert 1.0 $SkelPlot(count,notes)
   }

   
   set x0 [expr ($current + 3) * $scale]
   set y1 [expr 2 * $scale - 3] 

   # Delete old markers on canvas
   $path.$toppath delete marker
   $path.$bottompath delete marker
   $path.$toppath delete text
   $path.$bottompath delete text
   
   # Create new markers at the location specified in current
   $path.$bottompath create poly $x0 3 \
      [expr $x0 - 0.5 * $scale] $y1 [expr $x0 + 0.5 * $scale] $y1 \
      -outline black -fill red -tag marker
   $path.$bottompath create text [expr $x0 + 1.5 * $scale] $scale \
      -text "\[[expr $year-$global_endyear+$ring_length]\]" -anchor w -tag text
   $path.$toppath create poly $x0 $y1 \
      [expr $x0 - 0.5 * $scale] 3 [expr $x0 + 0.5 * $scale] 3 \
      -outline black -fill red -tag marker
   $path.$toppath create text [expr $x0 + 1.5 * $scale] $scale \
      -text "\[$year\]" -anchor w -tag text   
      
   # Scroll the canvases if necessary
   
   if { $SkelPlot(sample,$series,timeseries) < 2 } {
      # Current position relative to entire canvas
      set currpos [expr double($x0) / double([$path.$series cget -width])]  
      # Extents of the viewing window as fractions of total viewable
      set fracviewable [$path.$series xview]
   } else {
      # Current position relative to entire canvas
      set currpos [expr double($x0) / double([$path.$timeseries cget -width])] 
      # Extents of the viewing window as fractions of total viewable
      set fracviewable [$path.$timeseries xview]      
   } 
#   puts "Current position is $currpos"
#   puts "Fraction viewable is $fracviewable"
   
   set canvases [list $path.$toppath  \
                      $path.$timeseries $path.$series $path.$bottompath]
   set loview [lindex $fracviewable 0]
   set upview [lindex $fracviewable 1]
#   puts "loview is $loview upview is $upview"
   
   # Total Fraction viewable based on current window size
   set winfrac [expr $upview - $loview] 
#   puts "winfrac is $winfrac"
   
   if {$currpos < $loview} {
      # Lower
      # Puts markers in middle of the canvas
      BindXview $canvases moveto [expr $currpos - 0.5 * $winfrac]
   } elseif {$currpos > $upview} {
      # Higher
      # Puts markers at the left hand side of the canvas
      BindXview $canvases moveto $currpos
   } else {
      # Inside
      # Does nothing
   }   
   highlight_image_ring $imagecanvas [expr $current+$start-$global_endyear+$ring_length] $SkelPlot(sample,$series,serieslist)
}

# Initial creation of all of the elements on the skeleton plot canvas
proc draw_skelplot { var win top timeseries seriesname bottom scroll imagecanvas} {

   upvar #0 $var SkelPlot
   global skelplot session
   
   set SkelPlot(sample,$seriesname,invert) 0
   set SkelPlot(sample,$seriesname,timeseries) 1
   set SkelPlot(sample,$seriesname,position) 0       

   # Set the skelplot(seriesname) if not set already
   if {![info exists skelplot(seriesname)]} {
      set name [split $session(directory) "/"]
      set skelplot(seriesname) [lindex $name end]
   }   

#   create_main_skelplot $win $top $timeseries $seriesname $bottom $scroll
   create_skelplot_canvas $var $win $top $timeseries $seriesname $bottom $scroll
   configure_xscroll $win $seriesname $scroll
   create_skelplot_grid $var $win $seriesname
   draw_skelplot_data $var $win $seriesname
   create_timeseries_canvas $var $win $timeseries $seriesname
   create_timeseries_grid $var $win $timeseries $seriesname
   draw_timeseries_data $var $win $timeseries $seriesname
   create_marker_canvas $win $top $timeseries $seriesname $bottom    
   bindmouse $var $win $timeseries $top $seriesname $timeseries $bottom $imagecanvas
   bindmouse $var $win $seriesname $top $seriesname $timeseries $bottom $imagecanvas
   bindkey   $var $win $seriesname $top $bottom $timeseries $imagecanvas
   select_ring $win $seriesname $top $bottom $timeseries $imagecanvas
#   highlight_image_ring $imagecanvas $SkelPlot(count,current) $SkelPlot(sample,$seriesname,serieslist)

   grid $win.$top -column 0 -row 0 -sticky new
   if { $SkelPlot(sample,$seriesname,invert) } {
      grid $win.$seriesname -column 0 -row 1 -sticky new
      grid $win.$timeseries -column 0 -row 2 -sticky new
   } else {
      grid $win.$timeseries -column 0 -row 1 -sticky new
      grid $win.$seriesname -column 0 -row 2 -sticky new
   }   
   grid $win.$bottom -column 0 -row 3 -sticky new
   grid $win.$scroll -column 0 -row 4 -sticky new
   grid columnconfigure $win 0 -weight 1
   grid $win -row 3 -column 1 -columnspan 2 -sticky wes
 
}

# Destroy then recreate all the elements on the skeleton plot canvas.
proc redraw_skelplot { var win top timeseries seriesname bottom scroll imagecanvas} {

   upvar #0 $var SkelPlot

   set canvases [list $win.$top $win.$timeseries $win.$seriesname $win.$bottom]
   set SkelPlot(leftfrac) [lindex [$win.$seriesname xview] 0]

   trees_destroy_skelplot $win

#   create_main_skelplot $win $top $timeseries $seriesname $bottom $scroll
   create_skelplot_canvas $var $win $top $timeseries $seriesname $bottom $scroll
   configure_xscroll $win $seriesname $scroll
   create_skelplot_grid $var $win $seriesname 
   draw_skelplot_data SkelPlot $win $seriesname 
   create_timeseries_canvas $var $win $timeseries $seriesname
   create_timeseries_grid $var $win $timeseries $seriesname
   draw_timeseries_data $var $win $timeseries $seriesname
   create_marker_canvas $win $top $timeseries $seriesname $bottom    
   bindmouse $var $win $timeseries $top $seriesname $timeseries $bottom $imagecanvas
   bindmouse $var $win $seriesname $top $seriesname $timeseries $bottom $imagecanvas
   bindkey   $var $win $seriesname $top $bottom $timeseries $imagecanvas
   BindXview $canvases moveto $SkelPlot(leftfrac)

   grid $win.$top -column 0 -row 0 -sticky new
   # Invert and display timeseries and skeleton plot canvases as required
   switch -exact -- $SkelPlot(sample,$seriesname,timeseries) {
      0  {grid $win.$seriesname -column 0 -row 1 -sticky new}
      1  {      
            if { $SkelPlot(sample,$seriesname,invert) } {
               grid $win.$seriesname -column 0 -row 1 -sticky new
               grid $win.$timeseries -column 0 -row 2 -sticky new
            } else {
               grid $win.$timeseries -column 0 -row 1 -sticky new
               grid $win.$seriesname -column 0 -row 2 -sticky new
            }  }
      2  {grid $win.$timeseries -column 0 -row 1 -sticky new}
      3  {grid $win.$timeseries -column 0 -row 1 -sticky new}
      4  {grid $win.$timeseries -column 0 -row 1 -sticky new}
   }



#   if {$SkelPlot(sample,$seriesname,timeseries)} {
#      if { $SkelPlot(sample,$seriesname,invert) } {
#         grid $win.$seriesname -column 0 -row 1 -sticky new
#         grid $win.$timeseries -column 0 -row 2 -sticky new
#      } else {
#         grid $win.$timeseries -column 0 -row 1 -sticky new
#         grid $win.$seriesname -column 0 -row 2 -sticky new
#      }  
#   } else {
#      grid $win.$seriesname -column 0 -row 1 -sticky new
#   } 

   # Display timeseries canvas as required
   if {$SkelPlot(sample,$seriesname,timeseries)} {
	   $win.$timeseries raise $timeseries
   } else {
	   $win.$timeseries lower $timeseries background
   }

      
   grid $win.$bottom -column 0 -row 3 -sticky new
   grid $win.$scroll -column 0 -row 4 -sticky new
   grid columnconfigure $win 0 -weight 1
   grid $win -row 3 -column 1 -columnspan 2 -sticky wes



}

