proc trees_stagelink {} {
    global gui session mosaic stagemotion

    if {[winfo exists .stagelinkgui]} {
	messagebox "Stage Link Error" "Stage link window already exists. Cannot create a second one."
    } elseif {$gui(hardware) == 0} {
        messagebox "Stage Link Error" "Cannot link stage motion because this copy of TREES is not able to access system hardware."
    } elseif {$session(status) == "NONE"} {
        messagebox "Stage Link Error" "Cannot link stage motion because a session has not been started."
    } elseif {$mosaic(status) != "COMPLETE"} {
	messagebox "Stage Link Error" "Cannot link stage motion because the mosaic is incomplete."
    } else {
	if { ![info exists stagemotion(capture,session)] } {
	    set stagemotion(capture,session) "OTHER"
	}
	if { ![info exists stagemotion(registration,done)] } {
	    set stagemotion(registration,done) "FALSE"
	}
        stagelink_creategui
    }

}


# Build the stage motion  GUI and initialize camera and stage variables
proc stagelink_creategui {} {
    global trees_dir
    global video_frame motion position camera stagemotion

    toplevel .stagelinkgui -visual truecolor
    wm resizable .stagelinkgui 0 0

    # Set up procedure to handle the case when the window manager
    # unexpectedly deletes the stage motion GUI window
    wm protocol .stagelinkgui WM_DELETE_WINDOW { 
	stagelink_close
    }

    set video_frame(deci) 2
    set stagemotion(tag) control_marker

    # Check to see if frames were captured in the current session
    if { $stagemotion(capture,session) != "CURRENT" && $stagemotion(registration,done) != "TRUE" } {

	# Initialize the stage drivers
	StageMotion_Open
	
	# Initialize the focus control drivers
	FocusMotion_Open
	
	# Initialize stage motion variables
	# By default move stage in steps of ~32 pixels
	set motion(increment) 32
	# Stage motion calibration values (should periodically be verified)
	set motion(pixperstep,x) 12.9
	set motion(pixperstep,y) 12.9
	
	
	# Initialize the stage position variables
	# Current stage position in pixels
	set position(stage-pixel,x) 0
	set position(stage-pixel,y) 0
	# Start flag stage position in pixels
	set position(start-pixel,x) 0
	set position(start-pixel,y) 0
	# Current stage position in steps
	set position(stage-physical,x) 0
	set position(stage-physical,y) 0
	# Stop flag stage position in steps
	set position(stop-physical,x) 0
	set position(stop-physical,y) 0
	# Stop flag stage position in pixels
	set position(stop-pixel,x) 0
	set position(stop-pixel,y) 0
	
	
	# Initialize the camera variables
	set camera(width) 1280
	set camera(height) 1024
	
	
	# Initialze video frame variables
	# Subsample video frame to display captured frames at 1/4 resolution
	set video_frame(width) [expr round($camera(width) / $video_frame(deci))]
	set video_frame(height) [expr round($camera(height) / $video_frame(deci))]
	set video_frame(center,x) [expr round($video_frame(width) / 2)]
	set video_frame(center,y) [expr round($video_frame(height) / 2)]
	
    }

    # Color specs for control points
    set stagemotion(markcolor,1) orange
    set stagemotion(markcolor,2) orange
    set stagemotion(markcolor,3) orange

    # Create the live video window
    frame .stagelinkgui.live -relief sunken -bd 2 -bg LightGray
    canvas .stagelinkgui.live.canvas -bg lightblue -width $video_frame(width) -height $video_frame(height)

    if {![info exists camera(state)]} {
	set camera(state) live
    }
    set camera(name) .stagelinkgui.live.mycamera
    # Create the tkCamera image.  This is effectively a Tk photo image that is
    # automatically updated by the camera interface routines.  See tkCamera.C
    # for more information.
    if [catch { set camera(photo) [camera $camera(name) -deci $video_frame(deci)] } result] {
    	puts stderr $result
    }
    # Place the tkCamera image in its canvas
    .stagelinkgui.live.canvas create image $video_frame(center,x) $video_frame(center,y) -image $camera(photo) -anchor c

    # Draw the crosshair on the live video canvas
    .stagelinkgui.live.canvas create oval [expr $video_frame(center,x) - 10] \
	    [expr $video_frame(center,y) - 10] [expr $video_frame(center,x) + 10] \
	    [expr $video_frame(center,y) + 10] -width 1 -tag crosshair
    .stagelinkgui.live.canvas create line $video_frame(center,x) [expr $video_frame(center,y) - 15] \
	    $video_frame(center,x) [expr $video_frame(center,y) + 15] -width 1 -tag crosshair
    .stagelinkgui.live.canvas create line [expr $video_frame(center,x) - 15] $video_frame(center,y) \
	    [expr $video_frame(center,x) + 15] $video_frame(center,y) -width 1 -tag crosshair
    grid .stagelinkgui.live.canvas -sticky news
    
    
    
    # Create the control panel
    frame .stagelinkgui.control -relief ridge -bd 3 -bg LightGray
    
    frame .stagelinkgui.control.camera -relief ridge -bd 3 -bg LightGray
    label .stagelinkgui.control.cameratitle -relief ridge -background darkgray -text "Camera Control"
    button .stagelinkgui.control.camera.button -width 5 -bg LightGray -text "Turn Off" \
	    -command {
	global camera
	if {$camera(state) == "live"} {
	    set camera(state) "paused"
	    .stagelinkgui.control.camera.button configure -text "Turn On"
	    .stagelinkgui.control.camera.statevalue configure -background red
	    $camera(name) configure -pause 1
	} else {
	    set camera(state) "live"
	    .stagelinkgui.control.camera.button configure -text "Turn Off"
	    .stagelinkgui.control.camera.statevalue configure -background green
	    $camera(name) configure -pause 0
	}
    }
    if {$camera(state) == "paused"} {
	set camera(state) "live"
	.stagelinkgui.control.camera.button configure -text "Turn Off"
	$camera(name) configure -pause 0
    }
    label .stagelinkgui.control.camera.state -text "Camera State:" -bg LightGray
    label .stagelinkgui.control.camera.statevalue -background green \
	    -relief sunken -width 6 -textvariable camera(state)
    
    grid .stagelinkgui.control.camera.button -row 0 -column 0 -padx 5 -pady 10
    grid .stagelinkgui.control.camera.state -row 0 -column 1 -pady 10
    grid .stagelinkgui.control.camera.statevalue -row 0 -column 2 -padx 2 -pady 10


    frame .stagelinkgui.control.motion -relief ridge -bd 2 -bg LightGray
    label .stagelinkgui.control.motionlabel -relief ridge -background darkgray -text "X-Y Stage Control"
    image create bitmap uparrow -file ${trees_dir}/up_arrow.xbm
    image create bitmap downarrow -file ${trees_dir}/down_arrow.xbm
    image create bitmap rightarrow -file ${trees_dir}/right_arrow.xbm
    image create bitmap leftarrow -file ${trees_dir}/left_arrow.xbm
    button .stagelinkgui.control.motion.right -bg LightGray -image rightarrow \
	    -command { stagelink_move_stage "right" }
    button .stagelinkgui.control.motion.left  -bg LightGray -image leftarrow \
	    -command { stagelink_move_stage "left" }
    button .stagelinkgui.control.motion.up    -bg LightGray -image uparrow \
	    -command { stagelink_move_stage "up" }
    button .stagelinkgui.control.motion.down  -bg LightGray -image downarrow \
	    -command { stagelink_move_stage "down" }
    label .stagelinkgui.control.motion.incrlbl -text "Motion Increment:" -bg LightGray

    # Allow the stage to be moved in increments of either
    #  32 pixels 
    #  512 pixels (~ half a frame),
    #  1024 pixels (~ full frame),
    #  or 4096 pixels (~ four frames)
    set motion(increment) 512
    tk_optionMenu .stagelinkgui.control.motion.incrmenu motion(increment) 32 512 1024 4096
    label .stagelinkgui.control.motion.incrunits -text "pixels" -bg LightGray 


    grid x .stagelinkgui.control.motion.up x x .stagelinkgui.control.motion.incrlbl -sticky news
    grid .stagelinkgui.control.motion.left x .stagelinkgui.control.motion.right x .stagelinkgui.control.motion.incrmenu .stagelinkgui.control.motion.incrunits -sticky news
    grid x .stagelinkgui.control.motion.down x x x x
    grid columnconfigure .stagelinkgui.control.motion 3 -minsize 10
    grid columnconfigure .stagelinkgui.control.motion 4 -minsize 80


    frame .stagelinkgui.control.focus -relief ridge -bd 2 -bg LightGray
    label .stagelinkgui.control.focustitle -relief ridge -background darkgray -text "Focus Control"
    frame .stagelinkgui.control.focus.motion -bg LightGray

    global focus
    set focus(increment) 4
    button .stagelinkgui.control.focus.motion.up -bg LightGray -image uparrow \
	    -command {
	FocusMotion_MoveUp $focus(increment)
    }
    button .stagelinkgui.control.focus.motion.down -bg LightGray -image downarrow \
	    -command {
	FocusMotion_MoveDown $focus(increment)
    }

    button .stagelinkgui.control.focus.motion.coarse -width 3 -bg LightGray -text "Coarse" -justify center \
	    -command {
	if { $focus(increment) < 8 } {
	    set focus(increment) [expr $focus(increment) * 2]
	}
    }
    button .stagelinkgui.control.focus.motion.fine -width 3 -bg LightGray -text "Fine" -justify center \
	    -command {
	if { $focus(increment) > 1 } {
	    set focus(increment) [expr $focus(increment) / 2]
	}
    }

    label .stagelinkgui.control.focus.step -text "Focus Step Size:" -bg LightGray
    label .stagelinkgui.control.focus.stepvalue -relief sunken -bg LightGray -width 2 -textvariable focus(increment)
    
    grid x .stagelinkgui.control.focus.motion.up x -sticky news
    grid .stagelinkgui.control.focus.motion.coarse x .stagelinkgui.control.focus.motion.fine -sticky news
    grid x .stagelinkgui.control.focus.motion.down x -sticky news
    grid .stagelinkgui.control.focus.motion -row 0 -column 0 -columnspan 2 -pady 3
    grid .stagelinkgui.control.focus.step -row 1 -column 0 -pady 3
    grid .stagelinkgui.control.focus.stepvalue -row 1 -column 1 -pady 3


    frame .stagelinkgui.control.coord -relief ridge -bd 2 -bg LightGray
    label .stagelinkgui.control.coordtitle -relief ridge -background darkgray -text "X-Y Stage Position"
    label .stagelinkgui.control.coord.pixlbl -text "Pixels: " -justify left -bg LightGray
    label .stagelinkgui.control.coord.physlbl -text "Steps: " -justify left -bg LightGray
    label .stagelinkgui.control.coord.pixcoord -text "(0,0)" -bg LightGray
    label .stagelinkgui.control.coord.physcoord -text "(0,0)" -bg LightGray
    grid .stagelinkgui.control.coord.pixlbl .stagelinkgui.control.coord.pixcoord -sticky news
    grid .stagelinkgui.control.coord.physlbl .stagelinkgui.control.coord.physcoord -sticky news
    grid columnconfigure .stagelinkgui.control.coord 1 -minsize 100

    
    frame .stagelinkgui.control.rc -relief ridge -bd 2 -bg LightGray
    button .stagelinkgui.control.rc.register -bg LightGray -text "Register" \
	    -command {
	global stagemotion

	# Check to see if registration is complete
	if { $stagemotion(registration,done) == "TRUE" } {
	    global labels

	    set answer [tk_dialog .dlg "Stage Motion Registration" \
		    "Registration was done earlier. Do you wish to redo?" \
		    question \
		    0 $labels(button,no) $labels(button,yes) ]
	    
	    if { $answer == 1 } {
		stagelink_register
	    }
	    
	} else { stagelink_register }

    }
    button .stagelinkgui.control.rc.cancel -bg LightGray -text "Cancel" -command {

	set result [tk_dialog .dlg "Stage Link Close" \
		"Stage motion linking will not work if this window is closed. Do you wish to close anyway?" \
		question \
		0 "No" "Yes" ]

	if { $result } {
	    stagelink_close
	}
    }
    grid .stagelinkgui.control.rc.register -row 0 -column 0 -sticky news -padx 5
    grid  .stagelinkgui.control.rc.cancel -row 0 -column 1 -sticky news -padx 5

    
    grid .stagelinkgui.control.cameratitle -sticky news
    grid .stagelinkgui.control.camera -sticky news
    grid .stagelinkgui.control.focustitle -sticky news
    grid .stagelinkgui.control.focus -sticky news
    grid .stagelinkgui.control.motionlabel -sticky news
    grid .stagelinkgui.control.motion -sticky news
    grid .stagelinkgui.control.coordtitle -sticky news
    grid .stagelinkgui.control.coord -sticky news
    grid .stagelinkgui.control.rc -sticky news
    grid rowconfigure .stagelinkgui.control 1 -minsize 10


    # Lay out the overall gui configuration
    grid .stagelinkgui.live -row 0 -column 0 -sticky news
    grid .stagelinkgui.control -row 0 -column 1 -sticky news
    grid rowconfigure .stagelinkgui 0 -weight 1
    grid columnconfigure .stagelinkgui 0 -weight 1
    grid columnconfigure .stagelinkgui 1 -weight 1

    # Disable registration if frames were captured in the
    # current session.
    if { $stagemotion(capture,session) == "CURRENT" } {
	.stagelinkgui.control.rc.register configure -state disabled

	bind .trees_main.imagepanel.canvas <1> { stagelink_from_capture %W %x %y }
    }
}


# Move the stage either up, down, right, or left by the current increment
proc stagelink_move_stage {dir} {

    global motion position

    if {$dir == "left"} {
        stagelink_move_stage_to [expr $position(stage-pixel,x) - $motion(increment)] $position(stage-pixel,y)
    } elseif {$dir == "right"} {
        stagelink_move_stage_to [expr $position(stage-pixel,x) + $motion(increment)] $position(stage-pixel,y)
    } elseif {$dir == "up"} {
        stagelink_move_stage_to $position(stage-pixel,x) [expr $position(stage-pixel,y) + $motion(increment)]
    } else {
        stagelink_move_stage_to $position(stage-pixel,x) [expr $position(stage-pixel,y) - $motion(increment)]
    }
}


# Procedure to move the stage to a particular location and update the
# maps on the gui.  The input parameters are specified in pixels.
proc stagelink_move_stage_to {xpos_request ypos_request} {

    global motion position video_frame
    
      # Move is limited by the step size of the positioning table motors
    set position(stage-pixel,x) [request_x_pos $xpos_request]
    set position(stage-pixel,y) [request_y_pos $ypos_request]
    set position(stage-physical,x) [get_physical_x $position(stage-pixel,x)]
    set position(stage-physical,y) [get_physical_y $position(stage-pixel,y)]

    puts "moving stage to:            (after request ($xpos_request,$ypos_request))"
    puts "\tpixel coordinate: ($position(stage-pixel,x),$position(stage-pixel,y))   step coordinate: ($position(stage-physical,x),$position(stage-physical,y))"
    
    # Command the stage to move to a particular step location
    StageMotion_Move $position(stage-physical,x) $position(stage-physical,y)

    .stagelinkgui.control.coord.physcoord configure -text "($position(stage-physical,x),$position(stage-physical,y))"
    .stagelinkgui.control.coord.pixcoord configure -text "($position(stage-pixel,x),$position(stage-pixel,y))"
}

# Procedure to handle close
proc stagelink_close {} {
    global stagemotion motion video_frame focus
    global capture camera

    # Remove binding
    bind .trees_main.imagepanel.canvas <1> {}
    
    # Remove control point markers on the browse image
    set vismarks [.trees_main.imagepanel.canvas find withtag $stagemotion(tag)]
    foreach mark $vismarks {
	.trees_main.imagepanel.canvas delete $mark
    }
    
    update idletasks
    
    # Pause camera
    $camera(name) configure -pause 1
    set camera(state) "paused"

    # We want to comment this out for now
    # StageMotion_Close
    
    # Restore previous defaults
    set motion(increment) 512
    set video_frame(deci) 2
    set focus(increment) 4

    # We don't want to carry over any stagelink info
    # from the current session
    # unset stagemotion

    destroy .stagelinkgui
}


# Procedure to handle registration
proc stagelink_register {} {
    global stagemotion

    messagebox "Stage Motion Registration" "To complete registration specify three control points with the mouse. Click on the browse image for control point 1."

    # Initialize counter for control points
    set stagemotion(numpoints) 0

    # Remove control point markers on the browse image
    set vismarks [.trees_main.imagepanel.canvas find withtag $stagemotion(tag)]
    foreach mark $vismarks {
	.trees_main.imagepanel.canvas delete $mark
    }
    
    # Create binding on browse image
    bind .trees_main.imagepanel.canvas <1> { register_mosaic_point %W %x %y }
}


# Procedure to handle control points on mosaic
proc register_mosaic_point { w windowx windowy } {
    global stagemotion zoom

    # Convert window to browse image co-ordinates
    set canv_x [$w canvasx $windowx]
    set canv_y [$w canvasy $windowy]

    # Update counter for control points
    set stagemotion(numpoints) [expr $stagemotion(numpoints) + 1]
    set index $stagemotion(numpoints)

    # Create visual mark on browse image
    stagelink_mark_point $w $canv_x $canv_y  $stagemotion(tag) $stagemotion(markcolor,$index)

    set canv_y [expr $zoom(canvasHeight) - $canv_y]
    
    # Convert browse image co-ordinates to mosaic co-ordinates
    set canv_x [expr $canv_x * $zoom(default)]
    set canv_y [expr $canv_y * $zoom(default)]

    # Store the co-ordinates of control point
    set stagemotion(mosaic,point,$index) [list $canv_x $canv_y]

    bind .trees_main.imagepanel.canvas <1> {}
    # Create binding on video image
    bind .stagelinkgui.live.canvas <1> { register_stage_point %W %x %y }

    messagebox "Stage Motion Registration" "Click at the corresponding point on the live video image. Use X-Y stage control if needed."
}


# Procedure to handle control points on stage
proc register_stage_point { w windowx windowy } {
    global stagemotion video_frame position

    # Convert window to canvas co-ordinates
    set canv_x [$w canvasx $windowx]
    set canv_y [$w canvasy $windowy]
    set canv_y [expr $video_frame(height) - $canv_y]
    
    # Convert canvas co-ordinates to stage co-ordinates
    set canv_x [expr $position(stage-pixel,x) + ($canv_x - $video_frame(center,x))*$video_frame(deci)]
    set canv_y [expr $position(stage-pixel,y) + ($canv_y - $video_frame(center,y))*$video_frame(deci)]

    # Store the co-ordinates of control point
    set index $stagemotion(numpoints)
    set stagemotion(stage,point,$index) [list $canv_x $canv_y]


    if { $stagemotion(numpoints) < 3 } {
	bind .trees_main.imagepanel.canvas <1> { register_mosaic_point %W %x %y }
	bind .stagelinkgui.live.canvas <1> {}

	messagebox "Stage Motion Registration" "Click on the browse image for control point [expr $stagemotion(numpoints) + 1]"
    } else {

	set stagemotion(registration,done) "TRUE"
	bind .stagelinkgui.live.canvas <1> {}
	
	stagelink_create_mapping
    }
}    
    
# Procedure to map co-ordinates
proc stagelink_create_mapping {} {
    global stagemotion

    # Computer parameters of co-ordinate transformation
    stagelink_compute_param 1 2
    stagelink_compute_param 2 3
    stagelink_compute_param 3 1

    bind .trees_main.imagepanel.canvas <1> { stagelink_command_stage %W %x %y }
}


# Procedure to compute co-ordinate transformation
proc stagelink_compute_param { pointA pointB } {
    global stagemotion
    
    # Mosaic co-ordinates of control points
    set mx1 [lindex $stagemotion(mosaic,point,$pointA) 0]
    set my1 [lindex $stagemotion(mosaic,point,$pointA) 1]
    set mx2 [lindex $stagemotion(mosaic,point,$pointB) 0]
    set my2 [lindex $stagemotion(mosaic,point,$pointB) 1]

    # Stage co-ordinates of control points
    set sx1 [lindex $stagemotion(stage,point,$pointA) 0]
    set sy1 [lindex $stagemotion(stage,point,$pointA) 1]
    set sx2 [lindex $stagemotion(stage,point,$pointB) 0]
    set sy2 [lindex $stagemotion(stage,point,$pointB) 1]
    
    # Compute the angle between mosaic and stage co-ordinate systems
    set thetaMosaic [expr atan2($my2-$my1,$mx2-$mx1)]
    set thetaStage [expr atan2($sy2-$sy1,$sx2-$sx1)]
    set theta [expr $thetaStage - $thetaMosaic]
    set stagemotion(theta,$pointA,$pointB) $theta

    # ROTATIONAL TRANSFORMATION
    # Project mosaic co-ordinates to an intermediate co-ordinate system
    # with common origin but parallel to the stage co-ordinate system.
    set x [expr $mx1*cos($theta) - $my1*sin($theta)]
    set y [expr $mx1*sin($theta) + $my1*cos($theta)]

    # LINEAR TRANSLATION
    # This determines the shift between the origins of mosaic and stage
    # co-ordinate systems.
    set a [expr $sx1 - $x]
    set b [expr $sy1 - $y]
    set stagemotion(a,$pointA,$pointB) $a
    set stagemotion(b,$pointA,$pointB) $b 

    # Let us test the mapping now
    for { set i 1 } { $i <= $stagemotion(numpoints) } { incr i } {
	set mx1 [lindex $stagemotion(mosaic,point,$i) 0]
	set my1 [lindex $stagemotion(mosaic,point,$i) 1]
	
#	puts "[expr $mx1*cos($theta) - $my1*sin($theta) + $a], [lindex $stagemotion(stage,point,$i) 0]"
#	puts "[expr $mx1*sin($theta) + $my1*cos($theta) + $b], [lindex $stagemotion(stage,point,$i) 1]"
    }
}

# Procedure to mark control point on the browse image
proc stagelink_mark_point { can x y ctag markcolor } {
    
    $can create line $x [expr $y - 7] \
	    $x [expr $y + 8] -width 1 -tag $ctag -fill $markcolor
    $can create line [expr $x - 7] $y \
	    [expr $x + 8] $y -width 1 -tag $ctag -fill $markcolor
}

# Procedure to convert browse image co-ordinates to stage co-ordinates
proc stagelink_browse_stagexy { browsex browsey } {
    global stagemotion zoom

    # Co-ordinate transformation parameters
    set theta $stagemotion(theta,1,2)
    set a $stagemotion(a,1,2)
    set b $stagemotion(b,1,2)

    # Browse Image to Mosaic co-ordinate conversion
    set mosaicx [expr $browsex*$zoom(default)]
    set mosaicy [expr $browsey*$zoom(default)]

    # Mosaic to stage co-ordinate conversion
    set stagexy [list [expr $mosaicx*cos($theta) - $mosaicy*sin($theta) + $a] \
	    [expr $mosaicx*sin($theta) + $mosaicy*cos($theta) + $b]]

    return $stagexy
}


proc stagelink_command_stage { w windowx windowy } {
	global stagemotion zoom video_frame

	# Convert window to canvas co-ordinates
	set canv_x [$w canvasx $windowx]
	set canv_y [$w canvasy $windowy]
	set canv_y [expr $zoom(canvasHeight) - $canv_y]
	
	# Move stage
	set stagexy [stagelink_browse_stagexy $canv_x $canv_y]
	set stagex [expr round([lindex $stagexy 0])]
	set stagey [expr round([lindex $stagexy 1])]

	# Command stage to new location
	stagelink_move_stage_to $stagex $stagey
}


proc stagelink_from_capture { w windowx windowy } {
    global zoom

    # Convert canvas to browse image co-ordinates
    set canv_x [$w canvasx $windowx]
    set canv_y [$w canvasy $windowy]
    set canv_y [expr $zoom(canvasHeight) - $canv_y]

    # Browse Image to Mosaic co-ordinate conversion
    set mosaicx [expr $canv_x*$zoom(default)]
    set mosaicy [expr $canv_y*$zoom(default)]

    # Compute stage co-ordinates from offset
    set stagex [expr $mosaic(stage_offset,x) + $mosaicx]
    set stagey [expr $mosaic(stage_offset,y) + $mosaicy]

    # Command stage to new location
    stagelink_move_stage_to $stagex $stagey
}








