# This procedure is called whenever the Zoom On button is clicked
# Creates binding on mouse events over the image canvas
proc trees_zoomin {} {
    global zoom findrings

    .trees_main.zoompanel.zoombutton configure -state disabled
    .trees_main.zoompanel.resetbutton configure -state normal
    .trees_main.zoompanel.scalevaluelabel config -text "1:$findrings(ring,resolution)" 

    bind .trees_main.imagepanel.canvas <Motion> { zoom_region %W %x %y }
    bind .trees_main.imagepanel.canvas <2> { zoom_toggle_state %W %x %y }
    
#  set defualt cursor to crosshair in trees_main.tcl
#    .trees_main.imagepanel.canvas config -cursor crosshair

    set centerx [expr $zoom(framewidth)/2 + 1]
    set centery [expr $zoom(frameheight)/2 + 1]
    set x1 [expr $centerx - 15]
    set x2 [expr $centerx + 15]
    set y1 [expr $centery - 15]
    set y2 [expr $centery + 15]
    
    .trees_main.zoompanel.zoomframe.canvas create line \
	    $x1 $centery $x2 $centery -fill yellow -width 1 -tag crosshair
    .trees_main.zoompanel.zoomframe.canvas create line \
	    $centerx $y1 $centerx $y2 -fill yellow -width 1 -tag crosshair

    update idletasks
}

proc zoomBind {zoomwin imagewin x y} {
   global findrings zoom Image

   # Convert zoom window coords back to screen resolution
   set imageX [expr round(($x * $findrings(ring,resolution)/$zoom(default)) + $zoom(topleft_x))]
   set imageY [expr round(($y * $findrings(ring,resolution)/$zoom(default)) + $zoom(topleft_y))]
   # Note: Screen coordinates are already adjusted for scroll offset!!
#   puts "Image Coords: $imageX $imageY"

   # Set the Image(location) variable same as execute_mouse_bind (trees_skelplot_plot.tcl)
   
   # Set the search area to 1 pixel right and left, 0 pixel up and down, canvas coords
   set x0 [expr $imageX - 1 ]
   set x1 [expr $imageX + 1 ]
   set y0 [expr $imageY + 0 ]
   set y1 [expr $imageY + 0 ]   
   
   # add and remove ring expect location coordinates in terms of viewport (vs. canvas)
   set xwindowoffset [expr round([lindex [$imagewin xview] 0]*[$imagewin cget -width])]
   set ywindowoffset [expr round([lindex [$imagewin yview] 0]*[$imagewin cget -height])]
   set imageX [expr $imageX - $xwindowoffset]
   set imageY [expr $imageY - $ywindowoffset]   

   # Find tags at current image location
   set ROIitems [$imagewin find overlapping $x0 $y0 $x1 $y1]
#   puts $ROIitems
   # Build the Image(location) variable [ x y tag1 tag2 ... ]
   set Image(location) [list $x $y]
   if {[llength $ROIitems] > 0 } {
      foreach item $ROIitems {
         set tags [$imagewin gettags $item]
         foreach tag $tags {
            lappend Image(location) $tag
         }
      }
   }
   # puts "ZoomBind $Image(location)"
}

# This procedure is called whenever the Zoom Off button is clicked
# Deletes binding on mouse events over the image canvas
proc trees_zoomoff {} {
    global zoom
    
    .trees_main.zoompanel.zoombutton configure -state normal
    .trees_main.zoompanel.resetbutton configure -state disabled

    # Remove the mouse motion binding
    bind .trees_main.imagepanel.canvas <Motion> {}
    bind .trees_main.imagepanel.canvas <2> {}
    bind .trees_main.zoompanel.zoomframe.canvas <1> {}
    
    # Delete the zoom region indicator on the browse image 
    catch {.trees_main.imagepanel.canvas delete "zoomrectangle"}
    
    # Destroy the image canvas
    .trees_main.zoompanel.zoomframe.canvas delete all
    
    # Remove the Resolution Indicator
    .trees_main.zoompanel.scalevaluelabel config -text ""
    
    # Set zoomFrozen to FALSE
    set zoom(frozen) 0
    
    update idletasks
}

# This procedure is called when the user initiates rectangle region for zoom-in
proc zoom_region { w windowx windowy } {
    global zoom session mosaic findrings gui SkelPlot boundarycolor
    
    # Convert from screen to canvas co-ordinates
    set canv_x [$w canvasx $windowx]
    set canv_y [$w canvasy $windowy]

    # Convert co-ordinates to ring resolution
    set canv_x [expr $canv_x * $zoom(default)/$findrings(ring,resolution)]
    set canv_y [expr $canv_y * $zoom(default)/$findrings(ring,resolution)]


    # Compute zoom region at ring resolutoin
    set zoom(X1) [expr round($canv_x - $zoom(framewidth)/2)]
    set zoom(X2) [expr $zoom(X1) + $zoom(framewidth)]

    set zoom(Y1) [expr round($canv_y - $zoom(frameheight)/2)]
    set zoom(Y2) [expr $zoom(Y1) + $zoom(frameheight)]

    # Handle case when mouse is close to canvas boundary
    if { $zoom(X1) < 0 } {
	set zoom(X1) 0
	set zoom(X2) [expr $zoom(X1) + $zoom(framewidth)]
    }
    if { $zoom(Y1) < 0 } {
	set zoom(Y1) 0
	set zoom(Y2) [expr $zoom(Y1) + $zoom(frameheight)]
    }
    if { $zoom(X2) >= [expr $zoom(fullresolwidth)/$findrings(ring,resolution)] } {
	set zoom(X2) [expr $zoom(fullresolwidth)/$findrings(ring,resolution) - 1]
	set zoom(X1) [expr $zoom(X2) - $zoom(framewidth)] 
    }
    if { $zoom(Y2) >= [expr $zoom(fullresolheight)/$findrings(ring,resolution)] } {
	set zoom(Y2) [expr $zoom(fullresolheight)/$findrings(ring,resolution) - 1]
	set zoom(Y1) [expr $zoom(Y2) - $zoom(frameheight)] 
    }

    # Load, extract and display image of zoom region in window
    # Load 
    if {![info exists mosaic(loaded,index)]} {
	set mosaic(loaded,index) [Sadie_TREES_MosaicIndexOpen \
		$session(directory)/images/mosaic/$mosaic(file,index)]
    }
    
    set zoom(partial,mosaic) [Sadie_TREES_MosaicImport \
	    $mosaic(loaded,index) \
	    $session(directory)/images/mosaic/$mosaic(file,mosaic_columns) \
	    $zoom(Y1) $zoom(Y2) 1 $zoom(X1) $zoom(X2) 1]

    # Delete existing rings and image in zoom panel
    catch {.trees_main.zoompanel.zoomframe.canvas delete "ring"}
    catch {.trees_main.zoompanel.zoomframe.canvas delete $zoom(partial,mosaic)}

    # Create image of new zoom region
    image create photo $zoom(partial,mosaic) -format SADIE -data $zoom(partial,mosaic)

    .trees_main.zoompanel.zoomframe.canvas create image 0 0 \
	    -image $zoom(partial,mosaic) \
	    -tag $zoom(partial,mosaic) \
	    -anchor nw
    .trees_main.zoompanel.zoomframe.canvas raise crosshair

    update idletasks

    # Map co-ordinates of zoom region at ring resolution back to canvas co-ordinates
    # Need this for drawing zoom region rectangle on canvas
    set zoom(topleft_x) [expr $zoom(X1) * $findrings(ring,resolution)/$zoom(default)]
    set zoom(topleft_y) [expr $zoom(Y1) * $findrings(ring,resolution)/$zoom(default)]
    set zoom(bottomright_x) [expr $zoom(X2) * $findrings(ring,resolution)/$zoom(default)]
    set zoom(bottomright_y) [expr $zoom(Y2) * $findrings(ring,resolution)/$zoom(default)]

}

# This procedure is called upon button-2 click.
# Toggles zoom-hold on/off.
proc zoom_toggle_state { w windowx windowy } {
    global findrings zoom mosaic session gui SkelPlot boundarycolor
    
    # Check for zoom hold state
    if { [bind .trees_main.imagepanel.canvas <Motion>] == "" } {
	# Hold is ON, so turn it OFF...bind mouse motion to proc zoom_region
	catch {.trees_main.imagepanel.canvas delete "ring"}
	catch {.trees_main.imagepanel.canvas delete "zoomrectangle"}
	bind .trees_main.imagepanel.canvas <Motion> { zoom_region %W %x %y }
	bind .trees_main.zoompanel.zoomframe.canvas <1> {}
	set zoom(frozen) 0
	zoom_region $w $windowx $windowy
    } else {
	# Hold is OFF, so turn it ON...remove binding on mouse motion
	# And freeze zoom display
	# And initiate mouse <1> binding on zoom window
	
	# Store current values of windowx and windowy for re-use
	# if window needs to be re-drawn after adding or removing a ring
	set zoom(X) $windowx
	set zoom(Y) $windowy
	set zoom(frozen) 1
      
	bind .trees_main.imagepanel.canvas <Motion> {}
	zoom_region $w $windowx $windowy
	bind .trees_main.zoompanel.zoomframe.canvas <1> { zoomBind %W .trees_main.imagepanel.canvas %x %y }
	
	# Draw zoom region on browse image
	.trees_main.imagepanel.canvas create rectangle \
		$zoom(topleft_x) $zoom(topleft_y) $zoom(bottomright_x) $zoom(bottomright_y) \
		-outline yellow -width 1 -tag zoomrectangle
	
	if { $findrings(status) == "COMPLETE" } {
	    
	    if {![info exists mosaic(loaded,chainmap)]} {
		set findrings(loaded,chainmap) [Sadie_TREES_Disk2Chain \
			$session(directory)/images/mosaic/$findrings(file,chainmap)]
	    }
	    
	    # Display rings in zoom panel
	    Sadie_TREES_DisplayPartRings $findrings(loaded,chainmap) \
		    $zoom(X1) $zoom(Y1) $zoom(X2) $zoom(Y2) .trees_main.zoompanel.zoomframe.canvas
	    
	    # Highlight current ring in zoom panel
	    if { $gui(skelplot) } {
		if { [llength $SkelPlot(tags,highlight)] > 0 } {
		    set LBoundary [lindex $SkelPlot(tags,highlight) 0]
		    set RBoundary [lindex $SkelPlot(tags,highlight) 1]
		    .trees_main.zoompanel.zoomframe.canvas itemconfigure $LBoundary -fill red
		    .trees_main.zoompanel.zoomframe.canvas itemconfigure $RBoundary -fill red     
		}    
	    }
	}
    }
    update idletasks
}
