#if HAVE_CONFIG_H
#include <config.h>
#endif /* HAVE_CONFIG_H */
#include <tcl.h>
#include <tk.h>
#include "motion.h"
#include "project_utilities.h"

static stagecontrolp_t z = 0;
static const char * const devname = "/dev/parport1";
static const char * const stopped = "stopped";
static const char * const moving  = "moving";

int 
FocusMotion_Open_Cmd (ClientData clientData, Tcl_Interp *interp,
                      int objc, Tcl_Obj * const objv[])
{
  int err = 0;
  axismappingp_t map = 0;

  if (z == 0)
    {
      map = newaxismapping (1);
      if (map)
        {
          setaxismapping (map, 0, 1);
          z = newstage (map);
        }
      if (z)
        err = openstage (z, devname, STAGEPROP_HEAVY);
      if (map)
        disposeaxismapping (map);
    }
  else
    return TCL_OK;
  return (err || (z == 0)) ? TCL_ERROR : TCL_OK;
}

int
FocusMotion_Close_Cmd (ClientData clientData, Tcl_Interp *interp,
                       int objc, Tcl_Obj * const objv[])
{  
  if (closestage (z))
    return TCL_ERROR;
  disposestage (z);
  z = 0;
  return TCL_OK;
}  

int
FocusMotion_MoveUp_Cmd (ClientData clientData, Tcl_Interp *interp,
                        int objc, Tcl_Obj * const objv[])
{  
  int deltaz;
  int err = 0;
  stagelocationp_t nextpoint = 0;

  if ((z == 0) || (objc <= 1)
      || (Tcl_GetIntFromObj (interp, objv[1], &deltaz) != TCL_OK))
    return TCL_ERROR;
  nextpoint = newstagelocation (1);
  err = findstage (z, nextpoint);
  if (!err)
    {
      setstagelocation (nextpoint, 0,
                        (getstagelocation (nextpoint, 0) + (coord_t) deltaz));
      err = movestage (z, nextpoint);
    }
  if (nextpoint != 0)
    disposestagelocation (nextpoint);
  return (err) ? TCL_ERROR : TCL_OK;
}  

int
FocusMotion_MoveDown_Cmd (ClientData clientData, Tcl_Interp *interp,
                          int objc, Tcl_Obj * const objv[])
{  
  int deltaz;
  int err = 0;
  stagelocationp_t nextpoint = 0;

  if ((z == 0) || (objc <= 1)
      || (Tcl_GetIntFromObj (interp, objv[1], &deltaz) != TCL_OK))
    return TCL_ERROR;
  nextpoint = newstagelocation (1);
  err = findstage (z, nextpoint);
  if (!err)
    {
      setstagelocation (nextpoint, 0,
                        (getstagelocation (nextpoint, 0) - (coord_t) deltaz));
      err = movestage (z, nextpoint);
    }
  if (nextpoint != 0)
    disposestagelocation (nextpoint);
  return (err) ? TCL_ERROR : TCL_OK;
}

int
FocusMotion_GetState_Cmd (ClientData clientData, Tcl_Interp *interp,
                          int objc, Tcl_Obj * const objv[])
{
  if (z == 0)
    return TCL_ERROR;
  if (stageactive (z))
    Tcl_SetStringObj(Tcl_GetObjResult(interp), moving, -1);
  else
    Tcl_SetStringObj(Tcl_GetObjResult(interp), stopped, -1);
  return TCL_OK;
}  

BEGIN_C_DECLARATIONS

/*************************************************************************
 *
 * This procedure initializes the the stagemotion procedures when
 * this package is loaded.
 *
 *************************************************************************/
int 
FocusMotion_Init (Tcl_Interp *interp)
{
  Tcl_CreateObjCommand(interp, "FocusMotion_Open",
                       FocusMotion_Open_Cmd, (ClientData) NULL, NULL);
  Tcl_CreateObjCommand(interp, "FocusMotion_Close",
                       FocusMotion_Close_Cmd, (ClientData) NULL, NULL);
  Tcl_CreateObjCommand(interp, "FocusMotion_MoveUp",
                       FocusMotion_MoveUp_Cmd, (ClientData) NULL, NULL);
  Tcl_CreateObjCommand(interp, "FocusMotion_MoveDown",
                       FocusMotion_MoveDown_Cmd, (ClientData) NULL, NULL);
  Tcl_CreateObjCommand(interp, "FocusMotion_GetState",
                       FocusMotion_GetState_Cmd, (ClientData) NULL, NULL);
  return TCL_OK;
}

END_C_DECLARATIONS
