#if HAVE_CONFIG_H
#include <config.h>
#endif /* HAVE_CONFIG_H */
#include <tcl.h>
#include <tk.h>
#include <errno.h>
#include <time.h>
#include "motion.h"
#include "StageMotion.h"
#include "project_utilities.h"

static stagecontrolp_t xy = 0;
static const char * const devname = "/dev/parport0";
static const char * const stopped = "stopped";
static const char * const moving  = "moving";

int 
StageMotion_Open_Cmd (ClientData clientData, Tcl_Interp *interp,
                      int objc, Tcl_Obj * const objv[])
{
  int err = 0;
  axismappingp_t map = 0;

  if (xy == 0)
    {
      map = newaxismapping (2);
      if (map)
        xy = newstage (map);
      if (xy)
        err = openstage (xy, devname, STAGEPROP_NONE);
      if (map)
        disposeaxismapping (map);
    }
  else
    return TCL_OK;
  return (err || (xy == 0)) ? TCL_ERROR : TCL_OK;
}

int
StageMotion_Close_Cmd (ClientData clientData, Tcl_Interp *interp,
                       int objc, Tcl_Obj * const objv[])
{  
  int err = 0;

  if (closestage (xy))
    return TCL_ERROR;
  disposestage (xy);
  xy = 0;
  return TCL_OK;
}



int
StageMotion_Move_Cmd (ClientData clientData, Tcl_Interp *interp,
                      int objc, Tcl_Obj * const objv[])
{  
  int xcoord, ycoord;
  int err = 0;
  stagelocationp_t nextpoint = 0;

  if ((xy == 0) || (objc <= 2)
      || (Tcl_GetIntFromObj (interp, objv[1], &xcoord) != TCL_OK)
      || (Tcl_GetIntFromObj (interp, objv[2], &ycoord) != TCL_OK))
    return TCL_ERROR;
  nextpoint = newstagelocation (2);
  setstagelocation (nextpoint, 0, (coord_t) xcoord);
  setstagelocation (nextpoint, 1, (coord_t) ycoord);
  err = movestage (xy, nextpoint);
  if (nextpoint != 0)
    disposestagelocation (nextpoint);
  return (err) ? TCL_ERROR : TCL_OK;
}  

int
StageMotion_GetState_Cmd (ClientData clientData, Tcl_Interp *interp,
                          int objc, Tcl_Obj * const objv[])
{
  if (xy == 0)
    return TCL_ERROR;
  if (stageactive (xy))
    Tcl_SetStringObj(Tcl_GetObjResult(interp), moving, -1);
  else
    Tcl_SetStringObj(Tcl_GetObjResult(interp), stopped, -1);
  return TCL_OK;
}  

/*************************************************************************
 *
 * This procedure initializes the the stagemotion procedures when
 * this package is loaded.
 *
 *************************************************************************/
int
StageMotion_Init (Tcl_Interp *interp)
{
  Tcl_CreateObjCommand (interp, "StageMotion_Open",
                        StageMotion_Open_Cmd, (ClientData) NULL, NULL);
  Tcl_CreateObjCommand (interp, "StageMotion_Close",
                        StageMotion_Close_Cmd, (ClientData) NULL, NULL);
  Tcl_CreateObjCommand (interp, "StageMotion_Move",
                        StageMotion_Move_Cmd, (ClientData) NULL, NULL);
  Tcl_CreateObjCommand (interp, "StageMotion_GetState", 
                        StageMotion_GetState_Cmd, (ClientData) NULL, NULL);
  return TCL_OK;
}

int
Stagemotion_Find (stagelocationp_t loc)
{
  return findstage (xy, loc);
}

int
Stagemotion_Position (stagelocationp_t loc)
{
  return movestage (xy, loc);
}

int
Stagemotion_Active (void)
{
  return stageactive (xy);
}
