/*******************************************************************************
 RTN DIVSER: Divide one series by another; subtract if one goes negative

  Programmed copyright (C) AUG 1982 by
  Richard L Holmes, Lab. de Dendrocronologia, CRICYT - Mendoza - Argentina
  Modified copyright (C) 18 JUN 1998

  Divides array RM by array CV, both of length N, to produce array RZ.
  If RM has negative values or CV is not entirely positive,
  CV is subtracted from RM and a constant of 1.0 is added.

********************************************************************************/

void DIVSER(int N, float *RM, float *CV, float *RZ)
{
    int i;
    float BRM, BCV, AV;

    /* Find minimum values in RM and CV arrays */
    BRM = 1.0;
    BCV = 1.0;
    for(i = 0; i < N; i++)
    {
	BRM = (RM[i] < BRM? RM[i]: BRM);
	BCV = (CV[i] < BCV? CV[i]: BCV);
    }

    /* Subtract if minimum in RM is negative or minimum in CV is zero or negative */
    if (BRM < 0.0 || BCV <= 0.0)
    {
	BRM = 0.0;
	AV = 0.0;
	for(i = 0; i < N; i++)
	{
	    RZ[i] = RM[i] - CV[i] + 1.0;
	    AV = AV + RZ[i];
	    BRM = (RZ[i] < BRM? RZ[i]: BRM);
	}
	AV = AV/(float)N;
	
	if (BRM < 0.0)
	{
	    AV = 0.0;
	    for(i = 0; i < N; i++)
	    {
		RZ[i] = RZ[i] - BRM;
		AV = AV + RZ[i];
	    }
	    AV = AV/(float)N;
	}
	
	for(i = 0; i < N; i++)
	    RZ[i] = RZ[i]/AV;
    }
    else /* Divide series RM by CV to yield RZ */
    {
	for(i = 0; i < N; i++)
	    RZ[i] = RM[i]/CV[i];
    }

    return;
}
