#include "trees.h"

#define HIST_BLOCK_DIM           64    /* Block size                          */
#define HIST_RESOL               256   /* Histogram resolution                */
#define MAG_PERCENTILE  (double)(0.25) /* Rth percentile */
#define LOCAL_DEVN               25    /* Tolerable deviation of local 
					  orientation from global orientation */
#define DIR_TOL                  25    /* Direction tolerance for pixel
					  elimination                         */
#define SEARCH_TOL               10    /* Search tolerance in Edge tracking   */

typedef struct {
    int edge;                    /* Reserved for later use                    */
    int bin;                     /* Local gradient direction - resolved       */
    int cbin;                    /* Local gradient direction - computed       */
    int numpixels;               /* Number of pixels from ROI in the block    */
    int mag[HIST_RESOL];         /* Magnitude histogram                       */
    int dir[HIST_RESOL];         /* Direction histogram                       */
    double r;                    /* r                                         */
    int per;                     /* rth percentile                            */
} HISTOGRAM;

/*----------------------------------------------------------------------------*/

extern void HIST_INIT (HISTOGRAM *);
extern void HIST_MAG (IMAGE_BYTE *, MOSAIC_INDEX *, HISTOGRAM *, HISTOGRAM **, int, int);
extern void HIST_RTH_PERCENTILE (HISTOGRAM  *, HISTOGRAM  **, double, int, int);
extern void HIST_DIR (IMAGE_BYTE *, IMAGE_BYTE *, MOSAIC_INDEX *, HISTOGRAM *, HISTOGRAM **, int, int); 
extern void HIST_ORIENTATION (HISTOGRAM *, HISTOGRAM **, int, int);
extern int  HIST_DIR_OF_EDGE_NEIGHBOUR (HISTOGRAM *, HISTOGRAM **, int, int, int, int);
extern void HIST_RESOLVE (HISTOGRAM *, HISTOGRAM **, int, int);
extern int  HIST_NUM_RING_NEIGH(IMAGE_BYTE *, MOSAIC_INDEX *, int, int, int);
extern void HIST_SUPPRESS (IMAGE_BYTE *, IMAGE_BYTE *, MOSAIC_INDEX *, HISTOGRAM **);
extern int  HIST_FIND_MAX_NEIGH (IMAGE_BYTE *, IMAGE_BYTE *, MOSAIC_INDEX *, int, int, int *, int *);
extern int  HIST_NUM_NEIGH (IMAGE_BYTE *, MOSAIC_INDEX *, int, int);
extern void MOSAIC_BYTE_SUPPRESS (IMAGE_BYTE *, MOSAIC_INDEX *, IMAGE_BYTE *, IMAGE_BYTE *, HISTOGRAM *, HISTOGRAM ***);
