#if HAVE_CONFIG_H
#include <config.h>
#endif /* HAVE_CONFIG_H */
#include        <sadie.h>
#include        "sadie_byte.h"
#include        "mosaic.h"

/*----------------------------------------------------------------------------*/
/*-General Information--------------------------------------------------------*/
/*                                                                            */
/*   This function inserts edge data into a byte mosaic image.                */
/*                                                                            */
/*----------------------------------------------------------------------------*/
/*-Interface Information------------------------------------------------------*/
void MOSAIC_BYTE_INSERT (
IMAGE *in,           /*  I   Pointer to the input image.                      */
MOSAIC_INDEX *index, /*  I   Pointer to the input mosaic index.               */
int joff,            /*  I   Vertical offset of input image edge map data.    */
int koff,            /*  I   Horizontal offset of image data.                 */
IMAGE_BYTE **out     /*  O   Address of a pointer to the mosaic image.        */
/*----------------------------------------------------------------------------*/
) { register int j, k, m, n;
    char   msg[SLEN];

    /* check input */
    if (!CHECKIMG(in)) {
        MESSAGE('E'," Can't identify input image.");
        goto the_end;
    } else if (!CHECKIMG_MOSAIC_INDEX(index)) {
        MESSAGE('E'," Can't identify input mosaic index.");
        goto the_end;
    } else if (in->nbnd > 1) {
        MESSAGE('E'," Input image must be single-band.");
        goto the_end;
    } else if (joff < 0 || koff < 0) {
        MESSAGE('E'," Offsets must be positive.");
        goto the_end;
    } else if (!CHECKIMG_BYTE(*out)) {
        MESSAGE('E'," Can't identify mosaic image.");
        goto the_end;
    } else if (index->npix != (*out)->npix) {
        printf("Mosaic index size (%d) does not match mosaic size (%d).",index->npix,(*out)->npix);
        MESSAGE('E'," Mosaic index size does not match mosaic size.");
        goto the_end;
    }

    /* if (TIMES) TIMING(T_SCONVL); */
    if (NAMES) {
        MESSAGE('I',"");
        MESSAGE('I',"MOSAIC_BYTE_INSERT");
        MESSAGE('I',"");
        sprintf(msg," Input image:           %s",in->text);
        MESSAGE('I',msg);
        sprintf(msg,"  Horizontal Offset:    %d",koff);
        MESSAGE('I',msg);
        sprintf(msg,"    Vertical Offset:    %d",joff);
        MESSAGE('I',msg);
        sprintf(msg," Mosaic image:           %s",(*out)->text);
        MESSAGE('I',msg);
        MESSAGE('I'," ...............");
    }


    for (j=0; j<in->nlin; j++) {
        for (k=0; k<in->npix; k++) {
	    n = k+koff;
	    m = (j+joff)-index->voffset[n];
	    
	    if (n>0 && n<(*out)->npix && m>0 && m<(*out)->nlin) {
		(*out)->data[0][m][n] = (PIXEL_BYTE)rnd(in->data[0][j][k]);
	    }
	}
    }
    
    the_end:
    return;
    /* if (TIMES) TIMING(T_EXIT); */
}
