#if HAVE_CONFIG_H
#include <config.h>
#endif /* HAVE_CONFIG_H */
#include        <sadie.h>
#include        "mosaic.h"

/*-Copyright Information------------------------------------------------------*/
/* Copyright (c) 1988 by the University of Arizona Digital Image Analysis Lab */
/*-General Information--------------------------------------------------------*/
/*                                                                            */
/*   This function reads a mosaic index from disk into main memory.            */
/*                                                                            */
/*----------------------------------------------------------------------------*/
/*-Interface Information------------------------------------------------------*/
void MOSAIC_DISK2INDEX(
unsigned char  *name,    /*  I   String, containing the name of the disk file.         */
MOSAIC_INDEX **out    /*  O   Address of a pointer to the mosaic index.                    */
)
/*----------------------------------------------------------------------------*/
{   
    int i;
    static char array[2] = { 0x00, 0x01 };
    char msg[SLEN];
    int  nlin, npix, height_max, hstart, hend;
    uint32_t nframe;
    int value;
    FILE *fp=NULL;


    /* if (TIMES) TIMING(T_DISK2IMG); */


    /* open image file */
    if (!(fp=fopen(name,"r"))) {
        sprintf(msg," Can't open file %s.",name);
        MESSAGE('E',msg);
        goto the_end;
    }



    /* read index size parameters from file */
    if (fread(&height_max,sizeof(height_max),1,fp) != 1) {
        sprintf(msg," Can't read number of uncompressed lines from file %s.",name);
        MESSAGE('E',msg);
        goto the_end;
    }

    if (fread(&nlin,sizeof(nlin),1,fp) != 1) {
        sprintf(msg," Can't read number of compressed lines from file %s.",name);
        MESSAGE('E',msg);
        goto the_end;
    }

    if (fread(&npix,sizeof(npix),1,fp) != 1) {
        sprintf(msg," Can't read number of pixels from file %s.",name);
        MESSAGE('E',msg);
        goto the_end;
    }

    if (fread(&nframe,sizeof(nframe),1,fp) != 1) {
        sprintf(msg," Can't read number of frames from file %s.",name);
        MESSAGE('E',msg);
        goto the_end;
    }

    if (fread(&hstart,sizeof(hstart),1,fp) != 1) {
        sprintf(msg," Can't read X-coord of left extreme point in ROI from file %s.",name);
        MESSAGE('E',msg);
        goto the_end;
    }

    if (fread(&hend,sizeof(hend),1,fp) != 1) {
        sprintf(msg," Can't read X-coord of right extreme point in ROI from file %s.",name);
        MESSAGE('E',msg);
        goto the_end;
    }

    /* swap bytes, if necessary */
    if (*(short *)array != 1) {
        /*printf("DISK2IMG: Swapping bytes...\n");*/
    
        SWAP4((BYTE *)&height_max);
        SWAP4((BYTE *)&npix);
        SWAP4((BYTE *)&nlin);
        SWAP4((BYTE *)&nframe);
        SWAP4((BYTE *)&hstart);
        SWAP4((BYTE *)&hend);
    } 
    /*
    else {
        printf("DISK2IMG: No need to swap bytes...\n");
    }
    */
    
    printf("nlin = %d, npix = %d, nframe = %d\n",nlin,npix,nframe);


    /* create index of appropriate size */
    if (!CHECKIMG_MOSAIC_INDEX(*out)) GETMEM_MOSAIC_INDEX(nlin,npix,nframe,out);
    if (!CHECKIMG_MOSAIC_INDEX(*out)) {
        sprintf(msg," Can't allocate memory for index.");
        MESSAGE('E',msg);
        goto the_end;
    }


    (*out)->height_max = height_max;
    (*out)->npix = npix;
    (*out)->nlin = nlin;
    (*out)->nframe = nframe;
    (*out)->hstart = hstart;
    (*out)->hend = hend;
    
    if (*(short *)array != 1) {
        for (i=0; i<npix; i++) {
            if (fread(&value,sizeof(value),1,fp) != 1) {
                sprintf(msg," Can't read data from file %s.",name);
                MESSAGE('E',msg);
                goto the_end;
            }
            SWAP4((BYTE *)&value);
            (*out)->voffset[i] = value;
        }
        for (i=0; i<npix; i++) {
            if (fread(&value,sizeof(value),1,fp) != 1) {
                sprintf(msg," Can't read data from file %s.",name);
                MESSAGE('E',msg);
                goto the_end;
            }
            SWAP4((BYTE *)&value);
            (*out)->vheight[i] = value;
        }
        for (i=0; i<npix; i++) {
            if (fread(&value,sizeof(value),1,fp) != 1) {
                sprintf(msg," Can't read data from file %s.",name);
                MESSAGE('E',msg);
                goto the_end;
            }
            SWAP4((BYTE *)&value);
            (*out)->vstart[i] = value;
        }
        for (i=0; i<npix; i++) {
            if (fread(&value,sizeof(value),1,fp) != 1) {
                sprintf(msg," Can't read data from file %s.",name);
                MESSAGE('E',msg);
                goto the_end;
            }
            SWAP4((BYTE *)&value);
            (*out)->vend[i] = value;
        }
        for (i=0; i<nframe; i++) {
            if (fread(&value,sizeof(value),1,fp) != 1) {
                sprintf(msg," Can't read data from file %s.",name);
                MESSAGE('E',msg);
                goto the_end;
            }
            SWAP4((BYTE *)&value);
            (*out)->frame[i].startx = value;

            if (fread(&value,sizeof(value),1,fp) != 1) {
                sprintf(msg," Can't read data from file %s.",name);
                MESSAGE('E',msg);
                goto the_end;
            }
            SWAP4((BYTE *)&value);
            (*out)->frame[i].endx = value;

            if (fread(&value,sizeof(value),1,fp) != 1) {
                sprintf(msg," Can't read data from file %s.",name);
                MESSAGE('E',msg);
                goto the_end;
            }
            SWAP4((BYTE *)&value);
            (*out)->frame[i].starty = value;

            if (fread(&value,sizeof(value),1,fp) != 1) {
                sprintf(msg," Can't read data from file %s.",name);
                MESSAGE('E',msg);
                goto the_end;
            }
            SWAP4((BYTE *)&value);
            (*out)->frame[i].endy = value;
        }

	for(i = 0; i < 2; i++) {
	    if (fread(&value,sizeof(value),1,fp) != 1) {
                sprintf(msg," Can't read data from file %s.",name);
                MESSAGE('E',msg);
                goto the_end;
	    }
            SWAP4((BYTE *)&value);
            (*out)->roi->top[i] = value;

	    if (fread(&value,sizeof(value),1,fp) != 1) {
                sprintf(msg," Can't read data from file %s.",name);
                MESSAGE('E',msg);
                goto the_end;
	    }
            SWAP4((BYTE *)&value);
            (*out)->roi->bottom[i] = value;

	    if (fread(&value,sizeof(value),1,fp) != 1) {
                sprintf(msg," Can't read data from file %s.",name);
                MESSAGE('E',msg);
                goto the_end;
	    }
            SWAP4((BYTE *)&value);
            (*out)->roi->left[i] = value;

	    if (fread(&value,sizeof(value),1,fp) != 1) {
                sprintf(msg," Can't read data from file %s.",name);
                MESSAGE('E',msg);
                goto the_end;
	    }
            SWAP4((BYTE *)&value);
            (*out)->roi->right[i] = value;
	}

    } else {
        for (i=0; i<npix; i++) {
            if (fread(&value,sizeof(value),1,fp) != 1) {
                sprintf(msg," Can't read data from file %s.",name);
                MESSAGE('E',msg);
                goto the_end;
            }
            
            (*out)->voffset[i] = value;
        }
        for (i=0; i<npix; i++) {
            if (fread(&value,sizeof(value),1,fp) != 1) {
                sprintf(msg," Can't read data from file %s.",name);
                MESSAGE('E',msg);
                goto the_end;
            }
            
            (*out)->vheight[i] = value;
        }
        for (i=0; i<npix; i++) {
            if (fread(&value,sizeof(value),1,fp) != 1) {
                sprintf(msg," Can't read data from file %s.",name);
                MESSAGE('E',msg);
                goto the_end;
            }
            
            (*out)->vstart[i] = value;
        }
        for (i=0; i<npix; i++) {
            if (fread(&value,sizeof(value),1,fp) != 1) {
                sprintf(msg," Can't read data from file %s.",name);
                MESSAGE('E',msg);
                goto the_end;
            }
            
            (*out)->vend[i] = value;
        }
        for (i=0; i<nframe; i++) {
            if (fread(&value,sizeof(value),1,fp) != 1) {
                sprintf(msg," Can't read data from file %s.",name);
                MESSAGE('E',msg);
                goto the_end;
            }
            (*out)->frame[i].startx = value;

            if (fread(&value,sizeof(value),1,fp) != 1) {
                sprintf(msg," Can't read data from file %s.",name);
                MESSAGE('E',msg);
                goto the_end;
            }
            (*out)->frame[i].endx = value;

            if (fread(&value,sizeof(value),1,fp) != 1) {
                sprintf(msg," Can't read data from file %s.",name);
                MESSAGE('E',msg);
                goto the_end;
            }
            (*out)->frame[i].starty = value;

            if (fread(&value,sizeof(value),1,fp) != 1) {
                sprintf(msg," Can't read data from file %s.",name);
                MESSAGE('E',msg);
                goto the_end;
            }
            (*out)->frame[i].endy = value;
        }
	for(i = 0; i < 2; i++) {
	    if (fread(&value,sizeof(value),1,fp) != 1) {
                sprintf(msg," Can't read data from file %s.",name);
                MESSAGE('E',msg);
                goto the_end;
	    }
            (*out)->roi->top[i] = value;

	    if (fread(&value,sizeof(value),1,fp) != 1) {
                sprintf(msg," Can't read data from file %s.",name);
                MESSAGE('E',msg);
                goto the_end;
	    }
            (*out)->roi->bottom[i] = value;

	    if (fread(&value,sizeof(value),1,fp) != 1) {
                sprintf(msg," Can't read data from file %s.",name);
                MESSAGE('E',msg);
                goto the_end;
	    }
            (*out)->roi->left[i] = value;

	    if (fread(&value,sizeof(value),1,fp) != 1) {
                sprintf(msg," Can't read data from file %s.",name);
                MESSAGE('E',msg);
                goto the_end;
	    }
            (*out)->roi->right[i] = value;
	}
    }


    if (NAMES) {
        MESSAGE('I',"");
        MESSAGE('I',"MOSAIC_DISK2INDEX");
        MESSAGE('I',"");
        sprintf(msg," Input file:                                 %s",name);
        MESSAGE('I',msg); 
        sprintf(msg," Number of pixels:             %d",(*out)->npix);
        MESSAGE('I',msg);
        sprintf(msg," Number of frames:             %d",(*out)->nframe);
        MESSAGE('I',msg);
        MESSAGE('I'," ...............");
    }

    /* close image file */

    the_end:
    if (fp) {
        if (fclose(fp)) {
            sprintf(msg," Can't close file %s.",name);
            MESSAGE('W',msg);
        }
    }

    /* if (TIMES) TIMING(T_EXIT); */

}
