#if HAVE_CONFIG_H
#include <config.h>
#endif /* HAVE_CONFIG_H */
#include        <stdio.h>
#include        <sadie.h>
#include        "mosaic.h"


/*-Copyright Information------------------------------------------------------*/
/* Copyright (c) 1988 by the University of Arizona Digital Image Analysis Lab */
/*----------------------------------------------------------------------------*/
/*-General Information--------------------------------------------------------*/
/*                                                                            */
/*   This function writes a mosaic index from main memory to disk.            */
/*                                                                            */
/*----------------------------------------------------------------------------*/
/*-Interface Information------------------------------------------------------*/
void MOSAIC_INDEX2DISK(
MOSAIC_INDEX *in,   /*  I   Pointer to the mosaic index.                                 */
unsigned char *name /*  I   String, containing the name of the disk file.         */
)
/*----------------------------------------------------------------------------*/
{   
    int i;
    static char array[2] = { 0x00, 0x01 };
    char msg[SLEN];
    int nlin, npix, height_max, hstart, hend;
    uint32_t nframe;
    int value;
    FILE *fp=NULL;

    /* if (TIMES) TIMING(T_IMG2DISK); */

    /* check input */

    if (!CHECKIMG_MOSAIC_INDEX(in)) {
        MESSAGE('E'," Can't identify image.");
        goto the_end;
    }

    if (NAMES) {
        MESSAGE('I',"");
        MESSAGE('I',"MOSAIC_INDEX2DISK");
        MESSAGE('I',"");
        sprintf(msg," Output disk file:          %s",name);
        MESSAGE('I',msg);
        MESSAGE('I'," ...............");
    }




    height_max = in->height_max;
    npix = in->npix;
    nlin = in->nlin;
    nframe = in->nframe;
    hstart = in->hstart;
    hend = in->hend;
    if (*(short *)array != 1) {

        /*printf("IMG2DISK: Swapping bytes...\n"); */

        SWAP4((BYTE *)&height_max);
        SWAP4((BYTE *)&npix);
        SWAP4((BYTE *)&nlin);
        SWAP4((BYTE *)&nframe);
        SWAP4((BYTE *)&hstart);
        SWAP4((BYTE *)&hend);
    } 
    /*
    else {
        //printf("IMG2DISK: No need to swap bytes...\n");
    }
    */

    /* open image file */
    if (!(fp=fopen(name,"w"))) {
        sprintf(msg," Can't open file %s.",name);
        MESSAGE('E',msg);
        goto the_end;
    }



    /*
     * write data to file 
     */

    /* number of lines in uncompressed raster format mosaic */
    if (fwrite(&height_max,sizeof(height_max),1,fp) != 1) {
        sprintf(msg," Can't write number of lines in the uncompressed mosaic to file %s.",name);
        MESSAGE('E',msg);
        goto the_end;
    }

    /* number of lines in compressed mosaic */
    if (fwrite(&nlin,sizeof(nlin),1,fp) != 1) {
        sprintf(msg," Can't write number of lines in the compressed mosaic to file %s.",name);
        MESSAGE('E',msg);
        goto the_end;
    }

    /* number of pixels/line */
    if (fwrite(&npix,sizeof(npix),1,fp) != 1) {
        sprintf(msg," Can't write number of pixels/line to file %s.",name);
        MESSAGE('E',msg);
        goto the_end;
    }

    /* number of frames */
    if (fwrite(&nframe,sizeof(nframe),1,fp) != 1) {
        sprintf(msg," Can't write number of frames to file %s.",name);
        MESSAGE('E',msg);
        goto the_end;
    }

    /* X-coordinate of left extreme point in ROI */
    if (fwrite(&hstart,sizeof(hstart),1,fp) != 1) {
        sprintf(msg," Can't write X-coord of left extreme point to file %s.",name);
        MESSAGE('E',msg);
        goto the_end;
    }

    /* X-coordinate of right extreme point in ROI */
    if (fwrite(&hend,sizeof(hend),1,fp) != 1) {
        sprintf(msg," Can't write X-coord of right extreme point to file %s.",name);
        MESSAGE('E',msg);
        goto the_end;
    }


    if (*(short *)array != 1) {
        for (i=0; i<in->npix; i++) {
            value = in->voffset[i];
            SWAP4((BYTE *)&value);
            if (fwrite(&value,sizeof(value),1,fp) != 1) {
                sprintf(msg," Can't write data to file %s.",name);
                MESSAGE('E',msg);
                goto the_end;
            }
        }
        for (i=0; i<in->npix; i++) {
            value = in->vheight[i];
            SWAP4((BYTE *)&value);
            if (fwrite(&value,sizeof(value),1,fp) != 1) {
                sprintf(msg," Can't write data to file %s.",name);
                MESSAGE('E',msg);
                goto the_end;
            }
        }
        for (i=0; i<in->npix; i++) {
            value = in->vstart[i];
            SWAP4((BYTE *)&value);
            if (fwrite(&value,sizeof(value),1,fp) != 1) {
                sprintf(msg," Can't write data to file %s.",name);
                MESSAGE('E',msg);
                goto the_end;
            }
        }
        for (i=0; i<in->npix; i++) {
            value = in->vend[i];
            SWAP4((BYTE *)&value);
            if (fwrite(&value,sizeof(value),1,fp) != 1) {
                sprintf(msg," Can't write data to file %s.",name);
                MESSAGE('E',msg);
                goto the_end;
            }
        }
        for (i=0; i<in->nframe; i++) {
            value = in->frame[i].startx;
            SWAP4((BYTE *)&value);
            if (fwrite(&value,sizeof(value),1,fp) != 1) {
                sprintf(msg," Can't write data to file %s.",name);
                MESSAGE('E',msg);
                goto the_end;
            }

            value = in->frame[i].endx;
            SWAP4((BYTE *)&value);
            if (fwrite(&value,sizeof(value),1,fp) != 1) {
                sprintf(msg," Can't write data to file %s.",name);
                MESSAGE('E',msg);
                goto the_end;
            }

            value = in->frame[i].starty;
            SWAP4((BYTE *)&value);
            if (fwrite(&value,sizeof(value),1,fp) != 1) {
                sprintf(msg," Can't write data to file %s.",name);
                MESSAGE('E',msg);
                goto the_end;
            }

            value = in->frame[i].endy;
            SWAP4((BYTE *)&value);
            if (fwrite(&value,sizeof(value),1,fp) != 1) {
                sprintf(msg," Can't write data to file %s.",name);
                MESSAGE('E',msg);
                goto the_end;
            }
        }
	for(i = 0; i < 2; i++) {
	    value = in->roi->top[i];
	    SWAP4((BYTE *)&value);
	    if (fwrite(&value,sizeof(value),1,fp) != 1) {
		sprintf(msg," Can't write data to file %s.",name);
		MESSAGE('E',msg);
		goto the_end;
	    }
	    value = in->roi->bottom[i];
	    SWAP4((BYTE *)&value);
	    if (fwrite(&value,sizeof(value),1,fp) != 1) {
		sprintf(msg," Can't write data to file %s.",name);
		MESSAGE('E',msg);
		goto the_end;
	    }
	    value = in->roi->left[i];
	    SWAP4((BYTE *)&value);
	    if (fwrite(&value,sizeof(value),1,fp) != 1) {
		sprintf(msg," Can't write data to file %s.",name);
		MESSAGE('E',msg);
		goto the_end;
	    }
	    value = in->roi->right[i];
	    SWAP4((BYTE *)&value);
	    if (fwrite(&value,sizeof(value),1,fp) != 1) {
		sprintf(msg," Can't write data to file %s.",name);
		MESSAGE('E',msg);
		goto the_end;
	    }
	}
    } else {
        for (i=0; i<in->npix; i++) {
            value = in->voffset[i];
            if (fwrite(&value,sizeof(value),1,fp) != 1) {
                sprintf(msg," Can't write data to file %s.",name);
                MESSAGE('E',msg);
                goto the_end;
            }
        }
        for (i=0; i<in->npix; i++) {
            value = in->vheight[i];
            if (fwrite(&value,sizeof(value),1,fp) != 1) {
                sprintf(msg," Can't write data to file %s.",name);
                MESSAGE('E',msg);
                goto the_end;
            }
        }
        for (i=0; i<in->npix; i++) {
            value = in->vstart[i];
            if (fwrite(&value,sizeof(value),1,fp) != 1) {
                sprintf(msg," Can't write data to file %s.",name);
                MESSAGE('E',msg);
                goto the_end;
            }
        }
        for (i=0; i<in->npix; i++) {
            value = in->vend[i];
            if (fwrite(&value,sizeof(value),1,fp) != 1) {
                sprintf(msg," Can't write data to file %s.",name);
                MESSAGE('E',msg);
                goto the_end;
            }
        }
        for (i=0; i<in->nframe; i++) {
            value = in->frame[i].startx;
            if (fwrite(&value,sizeof(value),1,fp) != 1) {
                sprintf(msg," Can't write data to file %s.",name);
                MESSAGE('E',msg);
                goto the_end;
            }

            value = in->frame[i].endx;
            if (fwrite(&value,sizeof(value),1,fp) != 1) {
                sprintf(msg," Can't write data to file %s.",name);
                MESSAGE('E',msg);
                goto the_end;
            }

            value = in->frame[i].starty;
            if (fwrite(&value,sizeof(value),1,fp) != 1) {
                sprintf(msg," Can't write data to file %s.",name);
                MESSAGE('E',msg);
                goto the_end;
            }

            value = in->frame[i].endy;
            if (fwrite(&value,sizeof(value),1,fp) != 1) {
                sprintf(msg," Can't write data to file %s.",name);
                MESSAGE('E',msg);
                goto the_end;
            }
        }
	
	for(i = 0; i < 2; i++) {

	    value = in->roi->top[i];
	    if (fwrite(&value,sizeof(value),1,fp) != 1) {
		sprintf(msg," Can't write data to file %s.",name);
		MESSAGE('E',msg);
		goto the_end;
	    }

	    value = in->roi->bottom[i];
	    if (fwrite(&value,sizeof(value),1,fp) != 1) {
		sprintf(msg," Can't write data to file %s.",name);
		MESSAGE('E',msg);
		goto the_end;
	    }

	    value = in->roi->left[i];
	    if (fwrite(&value,sizeof(value),1,fp) != 1) {
		sprintf(msg," Can't write data to file %s.",name);
		MESSAGE('E',msg);
		goto the_end;
	    }

	    value = in->roi->right[i];
	    if (fwrite(&value,sizeof(value),1,fp) != 1) {
		sprintf(msg," Can't write data to file %s.",name);
		MESSAGE('E',msg);
		goto the_end;
	    }
	}
    }


    /* close image file */
    the_end:

    if (fp) {
        if (fclose(fp)) {
            sprintf(msg," Can't close file %s.",name);
            MESSAGE('W',msg);
        }
    }

    if (TIMES) TIMING(T_EXIT);

}
