#if HAVE_CONFIG_H
#include <config.h>
#endif /* HAVE_CONFIG_H */
#include  <stdio.h>
#include  <sadie.h>
#include  "proto.h"
#include  "mosaic.h"
#if WITH_DMALLOC
#include <dmalloc.h>
#endif /* WITH_DMALLOC */


/* variables necessary for computing vstart and vend with bresh algorithm */
extern int **x_array_buffer;        /* The data where info is stored      */
extern int point;                   /* Counter for points used            */
extern int count_point;             /* Flag to decide whether to count or fill array */



/*----------------------------------------------------------------------------*/
/*-General Information--------------------------------------------------------*/
/*                                                                            */
/*   This function computes vstart for each column in the mosaic image        */
/*   from starting point to ending point based on bresh algorithm             */
/*                                                                            */
/*----------------------------------------------------------------------------*/
/*-Interface Information------------------------------------------------------*/
void COMPUTEVSTART(
MOSAIC_INDEX *index,      /* Pointer to mosaic index */
int X1,                   /* Coordinates of starting point */
int Y1,                   
int X2,                   /* Coordinates of ending point   */
int Y2
)
/*----------------------------------------------------------------------------*/
{
    int l;
    int prev;

    /* Count number of points in line */
    count_point = TRUE;
    point = 0;
    brshnm(X1, Y1, X2, Y2);
    
    /* Allocate memory */
    x_array_buffer = (int **) malloc((long)point*sizeof(int *));
    if (x_array_buffer == (int **) NULL){
	MESSAGE('E'," Memory request failed.");
        goto the_end;
    } else {
	
	for  (l = 0; l < point; l++){
	    x_array_buffer[l] = (int  *) calloc(2, sizeof(int));
	    if (x_array_buffer[l] == (int) NULL){
		MESSAGE('E'," Memory request failed.");
		goto the_end;
	    }
	}
	
	/* Fill array */ 
	count_point = FALSE;
	point = 0;
	brshnm(X1, Y1, X2, Y2);

	prev = -1;
	for (l = 0; l < point; l++) {
	    if (x_array_buffer[l][0] != prev) {
		if (x_array_buffer[l][1] >= index->voffset[x_array_buffer[l][0]])
		    index->vstart[x_array_buffer[l][0]] = x_array_buffer[l][1];
		else
		    index->vstart[x_array_buffer[l][0]] = index->voffset[x_array_buffer[l][0]];
	    }
	    
	    prev = x_array_buffer[l][0];
	}
    	
	if(x_array_buffer) {
	    /* Free Bresh Array */
	    for (l = 0; l < point; l++){
		if(x_array_buffer[l]) {
		    free(x_array_buffer[l]);
		    x_array_buffer[l] = NULL;
		}
	    }
	    
	    free(x_array_buffer);
	    x_array_buffer = NULL;
	}
	
    }
    
    
    the_end:
    return;
    
}


/*----------------------------------------------------------------------------*/
/*-General Information--------------------------------------------------------*/
/*                                                                            */
/*   This function computes vend for each column in the mosaic image          */
/*   from starting point to ending point based on bresh algorithm             */
/*                                                                            */
/*----------------------------------------------------------------------------*/
/*-Interface Information------------------------------------------------------*/
void COMPUTEVEND(
MOSAIC_INDEX *index,      /* Pointer to mosaic index */
int X1,                   /* Coordinates of starting point */
int Y1,                   
int X2,                   /* Coordinates of ending point   */
int Y2		   
)
/*----------------------------------------------------------------------------*/
{
    int l;
    int prev;
    
    /* Count number of points in line */
    count_point = TRUE;
    point = 0;
    brshnm(X1, Y1, X2, Y2);
    
    /* Allocate memory */
    x_array_buffer = (int **) malloc((long)point*sizeof(int *));
    if (x_array_buffer == (int **) NULL){
	MESSAGE('E'," Memory request failed.");
        goto the_end;
    } else {
	
	for  (l = 0; l < point; l++){
	    x_array_buffer[l] = (int  *) calloc(2, sizeof(int));
	    if (x_array_buffer[l] == (int) NULL){
		MESSAGE('E'," Memory request failed.");
		goto the_end;
	    }
	}
	
	/* Fill array */ 
	count_point = FALSE;
	point = 0;
	brshnm(X1, Y1, X2, Y2);

	prev = -1;
	for (l = 0; l < point; l++) {
	    if (x_array_buffer[l][0] != prev) {
		if (x_array_buffer[l][1] < index->voffset[x_array_buffer[l][0]] + index->vheight[x_array_buffer[l][0]])
		    index->vend[x_array_buffer[l][0]] = x_array_buffer[l][1];
		else
		    index->vend[x_array_buffer[l][0]] = index->voffset[x_array_buffer[l][0]] + index->vheight[x_array_buffer[l][0]] - 1;
	    }
	    prev = x_array_buffer[l][0];
	}
	
	if(x_array_buffer) {
	    /* Free Bresh Array */
	    for (l = 0; l < point; l++){
		if(x_array_buffer[l]) {
		    free(x_array_buffer[l]);
		    x_array_buffer[l] = NULL;
		}
	    }
	    
	    free(x_array_buffer);
	    x_array_buffer = NULL;
	}
	
    }


    the_end:
    return;
    
}


/*----------------------------------------------------------------------------*/
/*-General Information--------------------------------------------------------*/
/*                                                                            */
/*   This function computes the local index for a given frame from mosaic     */
/*   index.                                                                   */
/*                                                                            */
/*----------------------------------------------------------------------------*/
/*-Interface Information------------------------------------------------------*/
void COMPUTE_LOCAL_INDEX(
MOSAIC_INDEX *index,      /*  I   Pointer to mosaic index                     */
int i,                  /*  I   Frame number                                */
int jbgn,               /*  I   Starting line.                              */
int jend,               /*  I   Ending line.                                */
int kbgn,               /*  I   Starting pixel.                             */
int kend,               /*  I   Ending pixel.                               */
LOCAL_INDEX **localindex   /*  O   Address of a pointer to the local index     */			 
)
/*----------------------------------------------------------------------------*/
{

    int k, npix, nlin;
    int startx, endx, starty, endy, shift;
    int vstart, vend;

    nlin = jend - jbgn + 1;
    npix = kend - kbgn + 1;
    startx = index->frame[i].startx;
    endx = index->frame[i].endx;
    starty = index->frame[i].starty;
    endy = index->frame[i].endy;
    
    (*localindex) = (LOCAL_INDEX *)malloc(sizeof(LOCAL_INDEX));
    (*localindex)->vstart = malloc(npix*sizeof(uint32_t));
    (*localindex)->vend = malloc(npix*sizeof(uint32_t));

    (*localindex)->hstart = (index->hstart > startx? index->hstart - startx + 5: 5);
    (*localindex)->hend = (index->hend < endx? npix-1-5 - (endx - index->hend): npix-1-5 );

    for (k = (*localindex)->hstart; k <= (*localindex)->hend; k++) {
	shift = k-1-5 + startx;
	if (shift < index->hstart)
	    shift = index->hstart;
	else if (shift > index->hend)
	    shift = index->hend;
	
	vstart = index->vstart[shift] - starty + 5;
	vstart = vstart < 5? 5: vstart;
	(*localindex)->vstart[k] = vstart;
	
	vend = nlin-1-5 - (endy - index->vend[shift]);
	vend = vend > nlin-1-5? nlin-1-5: vend;
	(*localindex)->vend[k] = vend;
    }

    
    
    return;
}
