#if HAVE_CONFIG_H
#include <config.h>
#endif /* HAVE_CONFIG_H */
#include        <stdio.h>
#include	<tcl.h>
#include        <sadie.h>
#include        "sadie_byte.h"
#if WITH_DMALLOC
#include <dmalloc.h>
#endif /* WITH_DMALLOC */






/*----------------------------------------------------------------------------*/
/*-General Information--------------------------------------------------------*/
/*                                                                            */
/*   This low-level function allocates memory for a new image, byte and      */
/*   initializes its data structure.  It is based on the SADIE function       */
/*   GETMEM.                                                                  */
/*                                                                            */
/*----------------------------------------------------------------------------*/
/*-Interface Information------------------------------------------------------*/
void GETMEM_BYTE(
uint32_t   nbnd,   /*  I   Number of bands.                                   */
uint32_t   nlin,   /*  I   Number of lines.                                   */
uint32_t   npix,   /*  I   Number of pixels/line.                             */
IMAGE_BYTE **out   /*  O   Address of a pointer to the image being allocated. */
)
/*----------------------------------------------------------------------------*/
{
  uint64_t total;
  BYTE *base_addr;
  size_t bands, lines, pixperline, i, j;

  /* allocate space for image data structure, image data and list entry */
  total = (sizeof(IMAGE)
           + (uint64_t)nbnd * (sizeof(BYTE **)
                               + (uint64_t)nlin * (sizeof(BYTE *)
                                                   + (uint64_t)npix * sizeof(BYTE))));
  if (total > (uint64_t)SIZE_MAX
      ||  (*out = malloc((size_t)total)) == NULL)
    {
      MESSAGE('E'," Memory request failed.");
      *out = NULL;
      return;
    }
  
  /* assign default values */
  (*out)->fileformat  = FILE_FORMAT;
  (*out)->fileversion = FILE_VERSION;
  (*out)->application = Application;
  (*out)->appversion  = AppVersion;
  (*out)->dataformat  = EXTDATA_UINT8;
  (*out)->datapacking = BE_BYTEPACKED;
  (*out)->datarotation[0] = PIXEL_DIMENSION    | VALUES_NORMAL;
  (*out)->datarotation[1] = LINE_DIMENSION     | VALUES_NORMAL;
  (*out)->datarotation[2] = BAND_DIMENSION     | VALUES_NORMAL;
  (*out)->datarotation[3] = INTERVAL_DIMENSION | VALUES_NORMAL;
  (*out)->npix = npix;
  (*out)->nlin = nlin;
  (*out)->nbnd = nbnd;
  (*out)->ntvl = 1;
  (*out)->nbit = SBPB;
  memset((*out)->text, 0, TLEN);

  /* initialize pointers */
  bands = nbnd;
  lines = nlin;
  pixperline = npix;
  base_addr = ((BYTE *)(*out) + sizeof (IMAGE_BYTE));
  (*out)->data = (BYTE ***)base_addr;
  for (i = 0; i < bands; i++)
    {
      (*out)->data[i]
        = (BYTE **)(base_addr
                    + bands * sizeof (BYTE **)
                    + i * lines * sizeof (BYTE *));
      for (j = 0; j < lines; j++)
        {
          (*out)->data[i][j]
            = (BYTE *)(base_addr
                       + bands * (sizeof (BYTE **) + (lines * sizeof (BYTE *)))
                       + (j * bands + i) * pixperline * sizeof (BYTE));
        }
    }
}

/*----------------------------------------------------------------------------*/
/*-General Information--------------------------------------------------------*/
/*                                                                            */
/*   This function releases a byte image to free memory space.                */
/*   It is based on the SADIE function RELIMG.                                */
/*                                                                            */
/*----------------------------------------------------------------------------*/
/*-Interface Information------------------------------------------------------*/
void RELIMG_BYTE(
IMAGE_BYTE **out    /* I/O  Address of the pointer to the image to be released.   */
)
/*----------------------------------------------------------------------------*/
{
  /* check input */
  if (!CHECKIMG_BYTE (*out))
    {
      MESSAGE ('E'," Can't identify image.");
      return;
    }
  /* release image */
  RELMEM_BYTE (*out);
  *out = NULL;
}
 
/*-General Information--------------------------------------------------------*/
/*                                                                            */
/*   This low-level function releases a byte image and frees its data         */
/*   structure.  It is based on the SADIE function RELMEM.                    */
/*                                                                            */
/*----------------------------------------------------------------------------*/
/*-Interface Information------------------------------------------------------*/
void RELMEM_BYTE(
IMAGE_BYTE *in      /*  I   Pointer to the image to be released.                  */
)
/*----------------------------------------------------------------------------*/
{   
  free (in);
}
 
/*----------------------------------------------------------------------------*/
/*-General Information--------------------------------------------------------*/
/*                                                                            */
/*   This low-level function determines whether or not an IMAGE_BYTE pointer */
/*   points to an image that was properly created by SADIE.                   */
/*   It is based on the SADIE routine CHECKIMG.                               */
/*                                                                            */
/*----------------------------------------------------------------------------*/
/*-Interface Information------------------------------------------------------*/
short CHECKIMG_BYTE(
IMAGE_BYTE *in      /*  I   Pointer to the image to be checked.                   */
)
/*----------------------------------------------------------------------------*/
{ 
  return (in != NULL);
}
