#ifndef SADIE_BYTE_H
#define SADIE_BYTE_H 1

#include "project_utilities.h"
#include <sadie.h>

BEGIN_C_DECLARATIONS

typedef BYTE PIXEL_BYTE;

/**
 * Almost identical to an IMAGE structre, but subclassed to single-byte pixels.
 */
typedef struct 
{                    
  uint16_t   fileformat;      /**< File format (magic number). */
  uint16_t   fileversion;     /**< File format version. */
  uint16_t   application;     /**< Application identifier. */
  uint16_t   appversion;      /**< Application version. */
  uint16_t   dataformat;      /**< Data format. */
  uint16_t   datapacking;     /**< Data packing. */
  uint8_t    datarotation[4]; /**< Data rotation. */
  uint32_t   npix;            /**< Number of pixels/line. */
  uint32_t   nlin;            /**< Number of lines. */
  uint32_t   nbnd;            /**< Number of bands. */
  uint32_t   ntvl;            /**< Number of intervals (reserved). */
  uint32_t   nbit;            /**< Number of bits/pixel (must be SBPB). */
  PIXEL      gmin;            /**< Minimum gray level in image. */
  PIXEL      gmax;            /**< Maximum gray level in image. */
  BYTE       text[TLEN];      /**< Application-specific data. */
  BYTE      ***data;          /**< Pointer to the data themselves. */
}                       IMAGE_BYTE;

short CHECKIMG_BYTE (IMAGE_BYTE *);
void  RELIMG_BYTE (IMAGE_BYTE **);
void  RELMEM_BYTE (IMAGE_BYTE *);
void  GETMEM_BYTE (uint32_t, uint32_t, uint32_t, IMAGE_BYTE **);
int   sad_byte_header_write (IMAGE_BYTE *buf, FILE *fp);
void  BYTEIMG2DISK (IMAGE_BYTE *, const char *);
void  BYTEDISK2IMG (const char *, IMAGE_BYTE **);

END_C_DECLARATIONS
 
#endif /* !SADIE_BYTE_H */


