#if HAVE_CONFIG_H
#include <config.h>
#endif /* HAVE_CONFIG_H */
#include <stdlib.h>
#include <sadie.h>
#include "chain.h"
#if WITH_DMALLOC
#include <dmalloc.h>
#endif /* WITH_DMALLOC */


void SCALE_CHAIN (
CHAIN *chainmap,                     /* Pointer to the chain code structure */
int   scale,                         /* Scaling factor to scale chain code */
CHAIN **scaledChain                  /* Address of pointer to the scaled chain code structure */
) {
    unsigned char code;
    unsigned int j, k, prevj, prevk;
    unsigned int currj, currk;
    int skip;
    LIST_NODE *newLnode, *currLnode, **prevLnode;
    CHAIN_NODE *newCnode, *currCnode, **prevCnode;
    
    /* Allocate memory for the new chain code structure */
    (*scaledChain) = (CHAIN *)malloc(sizeof(CHAIN));

    /* Write header info into the new chain code structure */
    memcpy((char *)&((*scaledChain)->nlin), (char *)&(chainmap->nlin), CHAIN_HEADER_SIZE);
    (*scaledChain)->nlin = chainmap->nlin/scale;
    (*scaledChain)->npix = chainmap->npix/scale;
    (*scaledChain)->list = NULL;
    prevLnode = (LIST_NODE **)&((*scaledChain)->list);

    /* Scale chain code */
    currLnode = chainmap->list;
    while (currLnode != NULL) {
	j = currLnode->startj;
	k = currLnode->startk;
	prevj = j/scale;
	prevk = k/scale;
	
	newLnode = (LIST_NODE *)malloc(sizeof(LIST_NODE));
	newLnode->startj = j/scale;
	newLnode->startk = k/scale;
	newLnode->ring_type = currLnode->ring_type;
	newLnode->ring_id = currLnode->ring_id;
	newLnode->chain = NULL;
	newLnode->next = NULL;
	(*prevLnode) = newLnode;
	prevLnode = (LIST_NODE **)&(newLnode->next);
	prevCnode = (CHAIN_NODE **)&(newLnode->chain);
	
	currCnode = currLnode->chain;
	skip = 1;
	while (currCnode != NULL) {
	    CHAIN_COMPUTE_COORDINATES(k, j, currCnode->code, &k, &j);
	    if (!(skip%scale)) {
		currj = j/scale;
		currk = k/scale;
		CHAIN_COMPUTE_CODE(prevj, prevk, currj, currk, &code);
		newCnode = (CHAIN_NODE *)malloc(sizeof(CHAIN_NODE));
		newCnode->code = code;
		newCnode->valid = currCnode->valid;
		newCnode->next = NULL;
		(*prevCnode) = newCnode;
		prevCnode = (CHAIN_NODE **)&(newCnode->next);
		
		prevj = currj;
		prevk = currk;
	    }

	    skip++;
	    currCnode = (CHAIN_NODE *)currCnode->next;
	}

	currLnode = currLnode->next;
    }
    
}

    
    
		  
