#if HAVE_CONFIG_H
#include <config.h>
#endif /* HAVE_CONFIG_H */
#include <stdio.h>
#include <tcl.h>
#include <tk.h>
#include <sadie.h>
#include "logerr.h"
#include "Sadie_Index.h"

static int FLAG=0;
static int DIRECTORY=1;

static int libhomeset=0;
static char libhome[200];
static char *ProgName;


/*-------------------------------------------------------------*/
/* Global Sadie variables that must be set from within Tcl/Tk  */
/*-------------------------------------------------------------*/
uint32_t 	nlev;
uint32_t	csize;
double	weight;
double  power;
double	*count;
PIXEL	gain;
PIXEL 	bias;
PIXEL	gmin;
PIXEL	gmax;
PIXEL	thresh;
PIXEL	gbrk[2][4];
PIXEL	*table;
extern short Application;
/* -------------------------------------------------------------------------- */

/* Global database... */
sad_doclistp_t global_doclist;

char* msg_ptr;
Tcl_Interp* MainTclInterp;


#ifdef TREES_HARDWARE
/* Defined in StageMotion.c */
int StageMotion_Init(Tcl_Interp*);
/* Defined in Sadie_Camera.c */
int Sadie_Camera_Init(Tcl_Interp*);
/* Defined in Sadie_Camera.c */
int tkCamera_Init(Tcl_Interp*);
/* Defined in FocusMotion.c */
int FocusMotion_Init(Tcl_Interp*);
#endif

int Sadie_General_Init(Tcl_Interp *interp);
int Sadie_FileIO_Init(Tcl_Interp *interp);
int Sadie_Image_Init(Tcl_Interp *interp);
int Sadie_Multi_Init(Tcl_Interp *interp);
int Sadie_Proto_Init(Tcl_Interp *interp);
int TREES_addremoverings_Init(Tcl_Interp *interp);

int Sadie_TREES_Init(Tcl_Interp *interp);


/*-Copyright Information------------------------------------------------------*/
/* Copyright (c) 1999 by the University of Arizona Digital Image Analysis Lab */
/*----------------------------------------------------------------------------*/
/*-General Information--------------------------------------------------------*/
/*                                                                            */
/*   This function displays the command line options for use when invoking    */
/*   trees.                                                                */
/*                                                                            */
/*----------------------------------------------------------------------------*/
/*-Interface Information------------------------------------------------------*/
void usage(void) {
	fprintf(stderr, "\nUsage: %s [options]\n",ProgName);
	fprintf(stderr, "Options:\n");
	fprintf(stderr, "\t-d <trees library directory>\n");
	fprintf(stderr, "\t-h\tprint this message\n");
	fprintf(stderr, "\n");
}

/*-Copyright Information------------------------------------------------------*/
/* Copyright (c) 1999 by the University of Arizona Digital Image Analysis Lab */
/*----------------------------------------------------------------------------*/
/*-General Information--------------------------------------------------------*/
/*                                                                            */
/*   This function checks to see if a file exists.                            */
/*                                                                            */
/*----------------------------------------------------------------------------*/
/*-Interface Information------------------------------------------------------*/
int verifyfile (
Tcl_Interp* interp, /*  I  The Tcl/Tk command interpreter.                    */
char* name          /*  I  The name of the file.                              */
) {
    Tcl_Obj *cmds=NULL;
    char tempstr[250];
    int result;
    
    sprintf(tempstr,"file exists %s",name);
    cmds = Tcl_NewStringObj(tempstr,-1);
    Tcl_EvalObj(interp,cmds);
    Tcl_GetIntFromObj(interp, Tcl_GetObjResult(interp), &result);

    if (result) {
      sprintf(tempstr,"auto_mkindex [file dirname %s] *.tcl",name);
      cmds = Tcl_NewStringObj(tempstr,-1);
      Tcl_EvalObj(interp,cmds);
    }

    return result;
}

/*-Copyright Information------------------------------------------------------*/
/* Copyright (c) 1999 by the University of Arizona Digital Image Analysis Lab */
/*----------------------------------------------------------------------------*/
/*-General Information--------------------------------------------------------*/
/*                                                                            */
/*   This function checks to see if the environment variable TREES_HOME    */
/*   contains the location of the .tcl files needed by trees.              */
/*                                                                            */
/*----------------------------------------------------------------------------*/
/*-Interface Information------------------------------------------------------*/
int setlibdir_env (Tcl_Interp* interp) {
    Tcl_Obj *cmds=NULL;
    char *libdir=NULL;
    int result;

    /* Verify that the environment variable TREES_HOME exists */
    cmds = Tcl_NewStringObj("global env\ninfo exists env(TREES_HOME)",-1);
    Tcl_EvalObj(interp,cmds);
    Tcl_GetIntFromObj(interp, Tcl_GetObjResult(interp), &result);

    /* Read the environment variable TREES_HOME */
    if (result) {
	cmds = Tcl_NewStringObj("global env\nset env(TREES_HOME)",-1);
	Tcl_EvalObj(interp,cmds);
	libdir = Tcl_GetStringFromObj(Tcl_GetObjResult(interp), NULL);
	sprintf(libhome,"%s",libdir);
    }
    
    return result;
}

/*-Copyright Information------------------------------------------------------*/
/* Copyright (c) 1999 by the University of Arizona Digital Image Analysis Lab */
/*----------------------------------------------------------------------------*/
/*-General Information--------------------------------------------------------*/
/*                                                                            */
/*   This function computes the default location of the .tcl files needed by  */
/*   trees, based on the location of the executable.                       */
/*                                                                            */
/*----------------------------------------------------------------------------*/
/*-Interface Information------------------------------------------------------*/
int setlibdir_default (Tcl_Interp* interp) {
    Tcl_Obj *cmds=NULL;
    char *libdirroot=NULL;
    
    cmds = Tcl_NewStringObj("file dirname [info nameofexecutable]",-1);
    Tcl_EvalObj(interp,cmds);
    libdirroot = Tcl_GetStringFromObj(Tcl_GetObjResult(interp), NULL);
    sprintf(libhome,"%s/../library",libdirroot);
    
    return 1;
}


/*-Copyright Information------------------------------------------------------*/
/* Copyright (c) 1999 by the University of Arizona Digital Image Analysis Lab */
/*----------------------------------------------------------------------------*/
/*-General Information--------------------------------------------------------*/
/*                                                                            */
/*   This function determines the location of the .tcl library files needed   */
/*   by trees.                                                             */
/*                                                                            */
/*----------------------------------------------------------------------------*/
/*-Interface Information------------------------------------------------------*/
void setlibdir (Tcl_Interp* interp) {
    char testfile[250];
    Tcl_Obj *cmds=NULL;

    /*
     * First check if the library directory was supplied
     * on the command line.
     */
    if (!libhomeset) {
	/* Check if the environment variable TREES_HOME exists */
	if (!setlibdir_env(interp)) {
	    /* Finally, as a last resort, try a default location */
	    setlibdir_default(interp);
	}
    }

    /* Verify that trees_main.tcl can be found in the library directory */
    sprintf(testfile,"%s/trees_main.tcl",libhome);
    if (verifyfile(interp,testfile)) {
	Tcl_SetVar(interp, "trees_dir", libhome, TCL_GLOBAL_ONLY);
	cmds = Tcl_NewStringObj("global trees_dir auto_path\nlappend auto_path $trees_dir",-1);
	Tcl_EvalObj(interp,cmds);
    } else {
	fprintf(stderr, "\nERROR: trees library directory: %s is not valid!\n",libhome);
	fprintf(stderr, "       \tYou can indicate the location of the library by using the\n");
	fprintf(stderr, "       \tenvironment variable TREES_HOME or the command line option -d.\n");
	usage();
	exit(0);
    }
}


int Tcl_AppInit(Tcl_Interp* interp)
{
  int status;
  msg_ptr = (char *) Tcl_Alloc(1024);
  
  
  MainTclInterp = interp;

  status = Tcl_Init(interp);
  
  if (status != TCL_OK) {
    return TCL_ERROR;
  }
  
  /* Initialize Tk values. */
  status = Tk_Init(interp);
  
  if (status != TCL_OK) {
    return TCL_ERROR;
  }
  

  /*
   * Call the init procedures for included packages.  Each call should
   * look like this:
   *
   * if (Mod_Init(interp) == TCL_ERROR) {
   *     return TCL_ERROR;
   * }
   *
   * where "Mod" is the name of the module.
   */


#ifdef TREES_HARDWARE
      Tcl_SetVar2(interp, "gui", "hardware", "1", TCL_GLOBAL_ONLY);

      if (StageMotion_Init(interp) == TCL_ERROR) {
          return TCL_ERROR;
      }
      if (Sadie_Camera_Init(interp) == TCL_ERROR) {
          return TCL_ERROR;
      }
      if (FocusMotion_Init(interp) == TCL_ERROR) {
          return TCL_ERROR;
      }
      if (tkCamera_Init(interp) == TCL_ERROR) {
          return TCL_ERROR;
      }
#else
      Tcl_SetVar2(interp, "gui", "hardware", "0", TCL_GLOBAL_ONLY);
#endif

  if (Sadie_FileIO_Init(interp) == TCL_ERROR) {
      return TCL_ERROR;
  }
  if (Sadie_General_Init(interp) == TCL_ERROR) {
      return TCL_ERROR;
  }
  if (Sadie_Image_Init(interp) == TCL_ERROR) {
    return TCL_ERROR;
  }
  if (Sadie_Multi_Init(interp) == TCL_ERROR) {
    return TCL_ERROR;
  }
  if (TREES_addremoverings_Init(interp) == TCL_ERROR) {
    return TCL_ERROR;
  }

  if (Sadie_Proto_Init(interp) == TCL_ERROR) {
    return TCL_ERROR;
  }
  
  if (Sadie_TREES_Init(interp) == TCL_ERROR) {
    return TCL_ERROR;
  }

  /* Find the location onf the library files */
  setlibdir(interp);
  printf("Initializing tclSadie with TREES_HOME=%s\n",libhome);

  

  /* Initialize the sadie program */
  if (Tcl_Eval(interp, "trees_main_init") == TCL_ERROR) {
    return TCL_ERROR;
  }
  Tk_MainLoop();

  return TCL_OK;
}


/*-Copyright Information------------------------------------------------------*/
/* Copyright (c) 1999 by the University of Arizona Digital Image Analysis Lab */
/*----------------------------------------------------------------------------*/
/*-General Information--------------------------------------------------------*/
/*                                                                            */
/*   This function parses command line arguements and spawns the Tcl/Tk       */
/*   command interpreter used by trees.                                    */
/*                                                                            */
/*----------------------------------------------------------------------------*/
/*-Interface Information------------------------------------------------------*/
int main(int argc, char** argv)
{
    int	i;
    int state=FLAG;
    
    /* Parse Command Line */
    
    ProgName = argv[0];
    
    for (i=1; i<argc; i++) {
	char *arg = argv[i];

	if (state==FLAG  &&  *arg=='-') {
	    switch (arg[1]) {
	    case 'd' :
		/* The next arg should be a directory */
		state = DIRECTORY;
		break;
	    default:
		usage();
		exit(0);
		break;
	    }
	} else if (state == DIRECTORY  && *arg!='-') {
	    sprintf(libhome,"%s",arg);
	    libhomeset = 1;
	    state = FLAG;
	} else {
	    usage();
	    exit(0);
	}
    }
    
    
    if (state != FLAG) {
	fprintf(stderr, "\nERROR: Expecting parameter after: %s\n",argv[i-1]);
	usage();
	exit(0);	
    }
    Application = APP_TREES;
    global_doclist = make_doclist (6);
    if (global_doclist == NULL)
      logerr_crit (LOGERR_FUNC, "allocating memory for object database");
    
    /* Start TK interpreter without passing in command line options */
    Tk_Main(1, argv, Tcl_AppInit);
    dispose_doclist (global_doclist);
    return 0;
}
