#if HAVE_CONFIG_H
#include <config.h>
#endif /* HAVE_CONFIG_H */
#include "trees_profile.h"
#include <stdlib.h>
#include "logerr.h"
#include "trees.h"
#if WITH_DMALLOC
#include <dmalloc.h>
#endif /* WITH_DMALLOC */

PROFILE_LIST
new_profile (int n)
{
  PROFILE_LIST newprofile;
  BRIGHTNESS * newval;
  int i;

  if (n < 0)
    return NULL;
  else if (n == 0)
    newval = NULL;
  else if ((newval = (BRIGHTNESS *) malloc (n * sizeof (BRIGHTNESS))) == NULL)
    return NULL;
  if ((newprofile = (PROFILE_LIST) malloc (sizeof (PROFILE_NODE))) == NULL)
    {
      free(newval);
      return NULL;
    }
  newprofile->next = NULL;
  newprofile->len  = n;
  newprofile->val  = newval;
  for (i = 0; i < n; i++)
    *(newval++) = DARKNESS;
  return newprofile;
}

void
demolish_profiles (PROFILE_LIST victim)
{
  PROFILE_LIST successor;

  while (victim != NULL)
    {
      successor = victim->next;
      if (victim->val != NULL)
        free (victim->val);
      free (victim);
      victim = successor;
    }
}
void demolish_boundbox (BOUNDBOX_LIST victim)
{
  BOUNDBOX_LIST successor;
  while (victim !=NULL)
    {
      successor = victim->next;
      free (victim);
      victim = successor; 
    }
}

PROFILE_LIST
reduce_profiles (PROFILE_LIST thelist)
{
  PROFILE_LIST newhead;

  if (thelist == NULL)
    return NULL;
  else
    {
      newhead = thelist->next;
      if (thelist->val != NULL)
        free (thelist->val);
      free(thelist);
      return newhead;
    }
}

#define INITIALIZE_PROFILE_SUM(S,N) *(S++) = 0.0L; *(N++) = 0
#define UPDATE_PROFILE_SUM(S,N,Z) *(S++) += (Z); (*(N++))++

static void
sum_to_mean_values (int n, long double *sum, int *count, double *mean)
{
  while (n > 0)
    {
      if (*count > 0)
        *(mean++) = (BRIGHTNESS) (*(sum++) / *(count++));
      else
        {
          *(mean++) = DARKNESS;
          count++;
          sum++;
        }
      n--;
    }
}
          
int
find_mean_profile (PROFILE_LIST mean, PROFILE_LIST prof)
{
  int n;
  int nb;
  int h, i, j, m;
  double s;
  BRIGHTNESS *b=NULL;
  long double *sum=NULL, *t=NULL;
  int *count=NULL, *k=NULL;

  if ((mean == NULL) || (prof == NULL) || (mean->len <= 0))
    return EXIT_FAILURE;
  n = mean->len;
  logerr_assert (LOGERR_DEBUG, LOGERR_FUNC, (n < MAXPROFILE) );
  if ((sum = (long double *) malloc (n * sizeof (long double))) == NULL)
    return EXIT_FAILURE;
  if ((count = (int *) malloc (n * sizeof (int))) == NULL)
    {
      free(sum);
      return EXIT_FAILURE;
    }
  k = count;
  t = sum;
  for (h = 0; h < n; h++)
    {
      INITIALIZE_PROFILE_SUM(t, k);
    }
  while (prof != NULL)
    {
      nb = prof->len;
      b = prof->val;
      k = count;
      t = sum;
      i = j = 0;
      while (i < n)
        {
          if ((n * j) <= (nb * i))
            j++;
          else if ((n * j) >= (nb * (i + 1)))
            {
              UPDATE_PROFILE_SUM(t, k, (double) b[j-1]);
              i++;
            }
          else
            {
              m = nb * (i + 1);
              s = ((double) b[j-1]) * ((n * j) - (nb * i));
              while (m > (n * (j + 1)))
                {
                  s += (n * ((double) b[j]));
                  j++;
                }
              UPDATE_PROFILE_SUM(t, k, (s + (((double) b[j]) * (m - (n * j))))/nb);
              i++;
              j++;
            }
        }
      prof = prof->next;
    }
  sum_to_mean_values (n, sum, count, mean->val);
  if(sum) free(sum);
  if(count) free(count);

  return EXIT_SUCCESS;
}

void 
First_diff_profiles(PROFILE_LIST *profiles, PROFILE_LIST *diffprofiles,
                    widthvecp_t ringwidths,
                    widthvecp_t earlyringwidth, widthvecp_t lateringwidth)
{
  int i, j, npix; 
  int min_count = 0;
  double last_value, min_value=0;
  PROFILE_LIST ringprofile;
  PROFILE_LIST current_profile;

  logerr_assert (LOGERR_CRIT, LOGERR_FUNC,
                 ((profiles != NULL) && (*profiles != NULL)
                  && (ringwidths != NULL)
                  && (ringwidths->len > 0) && (ringwidths->val != NULL)
                  && ((*profiles)->len > 0) && ((*profiles)->val != NULL)));
  ringprofile = *profiles;
  *diffprofiles = new_profile(ringprofile->len);
  current_profile = *diffprofiles;
  earlyringwidth->val = calloc (ringwidths->len, sizeof (double));
  if (earlyringwidth->val == NULL)
    logerr_crit (LOGERR_FUNC, "allocating memory for earlywood width values");
  lateringwidth->val = calloc (ringwidths->len, sizeof (double));
  if (lateringwidth->val == NULL)
    logerr_crit (LOGERR_FUNC, "allocating memory for latewood width values");
  for (i = 0; i < ringwidths->len; i++)
    {
      min_value = 0;
      min_count = 0;
      if ((ringwidths->val[i] < MAXPROFILE) && (ringprofile != NULL)) 
   	{
          npix = ringprofile->len;
          for (j = 0; j < npix; j++)
            {
              if ((i == 0) && (j == 0))
                current_profile->val[j] = 0;
              else if ((j == 0) && (i != 0))
                current_profile->val[j] = (ringprofile->val[j]
                                           - last_value);
              else
                current_profile->val[j] = (ringprofile->val[j]
                                           - ringprofile->val[j-1]);
              min_value = min (min_value, current_profile->val[j]);
              if (min_value == current_profile->val[j])
                min_count = j;
            }
          last_value = ringprofile->val[j-1];
        }
      /* Calculate late & earlywood width of the current first diff profile. */
      earlyringwidth->val[i] = min_count + 1;
      lateringwidth->val[i] = ringwidths->val[i] - earlyringwidth->val[i];
      if (ringprofile->next) 
        {
          ringprofile = ringprofile->next; 
          current_profile->next = new_profile (ringprofile->len);
          current_profile = current_profile->next;
        }
    }
  lateringwidth->len = earlyringwidth->len = ringwidths->len;
  logerr_assert (LOGERR_DEBUG, LOGERR_FUNC, (ringprofile->next == NULL));
}
