/*
##########################################
# Sadie_Multi.c -
#   Set of routines for linking SADIE multi routines to tcl/tk.
#
# RCS: $Id: Sadie_Multi.c,v 1.5 2004/09/07 19:05:14 vishwask Exp $ 
# 
# Digital Image Analysis Lab
# Dept. of Electrical and Computer Engineering
# University of Arizona
##########################################
*/

#if HAVE_CONFIG_H
#include <config.h>
#endif /* HAVE_CONFIG_H */
#include <tcl.h>
#include <tk.h>
#include <stdio.h>
#include <string.h>
#include <sadie.h>


/* RCS Indentification information */
static char rcsid[] = "$Id: Sadie_Multi.c,v 1.5 2004/09/07 19:05:14 vishwask Exp $";



/*-------------------------------------------------------------*/
/* Global Sadie variables that must be set from within Tcl/Tk  */
/*-------------------------------------------------------------*/
extern 	short   nlev;
extern	short   csize;
extern	double  weight;
extern	double  *count;
extern	PIXEL   gain;
extern	PIXEL   bias;
extern	PIXEL   gmin;
extern	PIXEL   gmax;
extern	PIXEL   thresh;
extern	PIXEL   gbrk[2][4];
extern	PIXEL   *table;
/* -------------------------------------------------------------------------- */


/*-Copyright Information------------------------------------------------------*/
/* Copyright (c) 1999 by the University of Arizona Digital Image Analysis Lab */
/*----------------------------------------------------------------------------*/
/*-General Information--------------------------------------------------------*/
/*                                                                            */
/*   This procedure provides an interface to the SADIE function               */
/*   COMBINE from Tcl/Tk.  It expects a tcl global array                      */
/*   with these indices to exist:                                             */
/*      array(inimg,addr1)        --  SADIE image address                     */
/*      array(inimg,addr2)        --  SADIE image address                     */
/*      array(option)             --  int                                     */
/*      array(weight1)            --  double                                  */
/*      array(weight2)            --  double                                  */
/*      array(outname)            --  char*                                   */
/*                                                                            */
/*----------------------------------------------------------------------------*/
/*-Interface Information------------------------------------------------------*/
int Sadie_Multi_CombineCmd(ClientData client_data, Tcl_Interp* interp, int argc, char *argv[])
{
    Tcl_Obj* tclobj=NULL;
    Tcl_Obj* tclarrayname = NULL;
    Tcl_Obj* tclindexname = NULL;
    char msg[SLEN];
    char* array=NULL;
    char* tempstr=NULL;
    int strlen;
    int inimgaddr1, inimgaddr2;
    IMAGE *inimg1=NULL, *inimg2=NULL;
    int outimgaddr;
    IMAGE* outimg=NULL;
    char* outname=NULL;
    int option;
    double weight1, weight2;

    if( argc != 2 ) {
	Tcl_AppendResult(interp,"wrong # args: should be \"",
	argv[0], " arrayname\"", (char *) NULL);
	return TCL_ERROR;
    }
    array=argv[1];
            

    /* Read input image array(inimg,addr1) */
    tclarrayname = Tcl_NewStringObj(array,-1);
    tclindexname = Tcl_NewStringObj("inimg,addr1",-1);
    if (tclobj = Tcl_ObjGetVar2(interp,  tclarrayname , tclindexname , TCL_GLOBAL_ONLY | TCL_LEAVE_ERR_MSG)) {
	tempstr = Tcl_GetStringFromObj(tclobj,&strlen);
	if (strlen <= 0) return TCL_ERROR;
	sscanf(tempstr,"%x",&inimgaddr1);
	inimg1 = (IMAGE *) inimgaddr1;
    } else {
	return TCL_ERROR;
    }
    Tcl_DecrRefCount(tclarrayname);
    Tcl_DecrRefCount(tclindexname);
		

    /* Read input image array(inimg,addr2) */
    tclarrayname = Tcl_NewStringObj(array,-1);
    tclindexname = Tcl_NewStringObj("inimg,addr2",-1);
    if (tclobj = Tcl_ObjGetVar2(interp,  tclarrayname , tclindexname , TCL_GLOBAL_ONLY | TCL_LEAVE_ERR_MSG)) {
	tempstr = Tcl_GetStringFromObj(tclobj,&strlen);
	if (strlen <= 0) return TCL_ERROR;
	sscanf(tempstr,"%x",&inimgaddr2);
	inimg2 = (IMAGE *) inimgaddr2;
    } else {
	return TCL_ERROR;
    }
    Tcl_DecrRefCount(tclarrayname);
    Tcl_DecrRefCount(tclindexname);
		

    /* Read the input integer array(option) */
    tclarrayname = Tcl_NewStringObj(array,-1);
    tclindexname = Tcl_NewStringObj("option",-1);
    if (tclobj = Tcl_ObjGetVar2(interp,  tclarrayname , tclindexname , TCL_GLOBAL_ONLY | TCL_LEAVE_ERR_MSG)) {
	if (Tcl_GetIntFromObj(interp,tclobj,&option) == TCL_ERROR) return TCL_ERROR;
    } else {
	return TCL_ERROR;
    }
    Tcl_DecrRefCount(tclarrayname);
    Tcl_DecrRefCount(tclindexname);
		

    /* Read the input double array(weight1) */
    tclarrayname = Tcl_NewStringObj(array,-1);
    tclindexname = Tcl_NewStringObj("weight1",-1);
    if (tclobj = Tcl_ObjGetVar2(interp,  tclarrayname , tclindexname , TCL_GLOBAL_ONLY | TCL_LEAVE_ERR_MSG)) {
	if (Tcl_GetDoubleFromObj(interp,tclobj,&weight1) == TCL_ERROR) return TCL_ERROR;
    } else {
	return TCL_ERROR;
    }
    Tcl_DecrRefCount(tclarrayname);
    Tcl_DecrRefCount(tclindexname);
		

    /* Read the input double array(weight2) */
    tclarrayname = Tcl_NewStringObj(array,-1);
    tclindexname = Tcl_NewStringObj("weight2",-1);
    if (tclobj = Tcl_ObjGetVar2(interp,  tclarrayname , tclindexname , TCL_GLOBAL_ONLY | TCL_LEAVE_ERR_MSG)) {
	if (Tcl_GetDoubleFromObj(interp,tclobj,&weight2) == TCL_ERROR) return TCL_ERROR;
    } else {
	return TCL_ERROR;
    }
    Tcl_DecrRefCount(tclarrayname);
    Tcl_DecrRefCount(tclindexname);
		

    /* Read the output image 1 name */
    tclarrayname = Tcl_NewStringObj(array,-1);
    tclindexname = Tcl_NewStringObj("outname",-1);
    if (tclobj = Tcl_ObjGetVar2(interp,  tclarrayname , tclindexname , TCL_GLOBAL_ONLY | TCL_LEAVE_ERR_MSG)) {
	outname = Tcl_GetStringFromObj(tclobj,&strlen);
	if (strlen <= 0) return TCL_ERROR;
    } else {
	return TCL_ERROR;
    }
    Tcl_DecrRefCount(tclarrayname);
    Tcl_DecrRefCount(tclindexname);
		

    COMBINE(inimg1,inimg2,option,weight1,weight2,&outimg);
	    

    if (CHECKIMG(outimg))  sprintf(outimg->text, "%s", outname);
    outimgaddr = (int) outimg;
	        

    sprintf(msg, "%x", outimgaddr);
    Tcl_SetResult(interp, msg, TCL_VOLATILE);
	    
    return TCL_OK;
}







/*-Copyright Information------------------------------------------------------*/
/* Copyright (c) 1999 by the University of Arizona Digital Image Analysis Lab */
/*----------------------------------------------------------------------------*/
/*-General Information--------------------------------------------------------*/
/*                                                                            */
/*   This procedure initializes the all of the procedures                     */
/*   in this file by registering them with Tcl.                               */
/*                                                                            */
/*----------------------------------------------------------------------------*/
/*-Interface Information------------------------------------------------------*/
int Sadie_Multi_Init(Tcl_Interp *interp)
{
    Tcl_CreateCommand(interp, "Sadie_Multi_Combine", Sadie_Multi_CombineCmd,(ClientData) NULL, NULL);
    return TCL_OK;
}

