/*
##########################################
# bresh.c -
#   Algorithm for drawing a line in an image.
#
# RCS: $Id: bresh.c,v 1.2 2004/09/08 21:22:27 mmunro Exp $ 
# 
# Digital Image Analysis Lab
# Dept. of Electrical and Computer Engineering
# University of Arizona
##########################################
*/



/*
    Breshenham line drawing algorithm

Modified so that the points to be drawn 
      are in one function.  
      plot_point(x,y)  is only function that uses the 
      external variables.

Also:  All of the lines are draw
       from ymax values to ymin values.
       X ordering doesn't matter.

*/

#if HAVE_CONFIG_H
#include <config.h>
#endif /* HAVE_CONFIG_H */
#include <stdio.h>
/*#include <sys/param.h>*/
#include <sys/types.h>


/* RCS Indentification information */
static char rcsid[] = "$Id: bresh.c,v 1.2 2004/09/08 21:22:27 mmunro Exp $";


int **x_array_buffer;	/* Window Size */
int count_point;
int point;




brshnm(x1,y1,x2,y2)
    int x1,y1,x2,y2;
{  

    int x,y;
    int tempx, tempy;
    int dely,delx;
    int ebar;
    int i;

    #ifdef  DEBUG
    printf ("x1 = %d , y1 = %d\n",x1,y1);
    printf ("x2 = %d , y2 = %d\n",x2,y2);
    #endif
    
    
    if (( x1 > x2 ) && ( y1 > y2)) {
    
        /* ------------------------------------------------------
         *  Case 1
         *  Neg. Slope
         *  X1, Y1 is greater than X2, Y2
         *  ie. Line starts at low end of screen and goes up
         */

        #ifdef  DEBUG
        printf ("First point is greater than second point\n");
        printf ("x1 = %d , y1 = %d\n",x1,y1);
        printf ("x2 = %d , y2 = %d\n",x2,y2);
        #endif
        
        pos_hor_slope(x1,y1,x2,y2);
    


    } else if (( x1 > x2) && ( y1 == y2)) {
    
        /* ------------------------------------------------------
         *  Case 2.
         *  Horizontal Line where 
         *  X1 > X2
         *  ie. Line starts at right and goes left
         */

        pos_hor_slope(x1,y1,x2,y2);
    
    } else if (( x1 < x2) && (y1 > y2)) {

        #ifdef  DEBUG
        printf (" Neg slope points \n");
        #endif

        neg_slope(x1,y1,x2,y2);
    
    } else if (( x1 > x2) && ( y1 < y2)) {

        #ifdef  DEBUG
        printf ("Neg slope points w/ \n");
        printf ("Start at Upper right  to Lower Left \n");
        #endif

        neg_slope(x1,y1,x2,y2);


    } else if (( x1 == x2) && ( y1 != y2)) {

        /* ------------------------------------------------------
         *  Case 3. 
         *  VERTICAL line 
         */

        #ifdef  DEBUG
        printf ("Vertical Slope\n");
        #endif

        vertical_slope(x1,y1,x2,y2);

    } else {

        pos_hor_slope(x1,y1,x2,y2);

    }

}




/* ------------------------------------------------------ */
/* This routine plots point by point vertical lines.
 * 
 */
vertical_slope(x1,y1,x2,y2)
int x1,y1,x2,y2;
{
    int x, y, tempy;
    x = x1;

    if ( y1 < y2){
        for (y = y1 ; y <= y2; y++){
                plot_points (x,y);
        }
    } else {
        for (y = y1 ; y >= y2; y--){
                plot_points (x,y);
        }
    }
}




/* ------------------------------------------------------ */
/* This routine plots point by point  lines with
 * Neg slope .
 */
neg_slope(x1,y1,x2,y2)
int x1,y1,x2,y2;
{
    int x, y;
    int delx, dely;
    int ebar;
    int i;

    delx = abs(x2-x1);
    dely = abs(y2-y1);


    /* Initialization
       of loop
     */
    x = x1;
    y = y1;


    if (  dely < delx ) {
        /*
         * Slope is less than 1
         */ 
        ebar =  2*dely - delx;
        for (i = 0; i < delx; i++){
	    plot_points (x,y);

            if (( x1 < x2) && (y1 > y2)) {

                while (ebar > 0){
		        y = y - 1;
    		        ebar = ebar - 2*delx;
	        }

                x = x + 1;
    	        ebar = ebar + 2*dely;

            } else {

                while (ebar > 0){
		        y = y + 1;
    		        ebar = ebar - 2*delx;
	        }

                x = x - 1;
    	        ebar = ebar + 2*dely;

            } 
        }
    }

    else{
        /*
         * Slope Greater than 1
         */
        #ifdef  DEBUG
        printf ("Slope greater than 1\n");
        #endif

        ebar = 2*delx - dely;
        for ( i = 0; i < dely; i++){
	    plot_points (x,y);

            if (( x1 < x2) && (y1 > y2)) {
            
                while (ebar > 0){
		    x = x + 1;
	            ebar = ebar - 2*dely;
	        }

                y = y - 1;
	        ebar = ebar + 2*delx;
            
            } else {
            
                while (ebar > 0){
		    x = x - 1;
	            ebar = ebar - 2*dely;
	        }

                y = y + 1;
	        ebar = ebar + 2*delx;
            
            }
        }
    }
}




/* ------------------------------------------------------ */
/* This routine plots point by point  lines with
 * Pos slope and horizontal lines.
 */
pos_hor_slope(x1,y1,x2,y2)
int x1,y1,x2,y2;
{
    int x, y;
    int delx, dely;
    int ebar;
    int i;


    /* Initialization
       of loop
     */
    x = x1;
    y = y1;

    delx = abs(x2-x1);
    dely = abs(y2-y1);


    if (  dely < delx ) {
        /*
         * Slope is less than 1
         */ 
	    ebar = 2*dely - delx;
	    for (i = 0; i < delx; i++){
		plot_points (x,y);

                while (ebar > 0){
		    if (y1 < y2) {
                        y = y + 1;
                    } else {
                        y = y - 1;
                    }
		    ebar = ebar - 2*delx;
		}

		if (x1 < x2) {
                    x = x + 1;
                } else {
                    x = x - 1;
                }
		ebar = ebar + 2*dely;
                
            }
    
    } else {
        /*
         * Slope Greater than 1
         */

        #ifdef  DEBUG
	printf ("Slope greater than 1\n");
	printf ("than 1\n");
        #endif

	ebar = 2*delx - dely;
        for ( i = 0; i < dely; i++){
	    plot_points (x,y);

            while (ebar > 0){
		if (x1 < x2) {
		    x = x + 1;
                } else {
		    x = x - 1;
                }
		ebar = ebar - 2*dely;
	    }

	    if (y1 < y2) {
	        y = y + 1;
            } else {
	        y = y - 1;
            }
	    ebar = ebar + 2*delx;
	}
    }

}







/* 
   Uses global variables to either count or fill the 
   passed array.
*/
plot_points (x,y)
int x,y;
{

    #ifdef  DEBUG
    printf ("x = %d, y= %d \n",x,y);
    #endif

    if (count_point){

        point++;
        #ifdef  DEBUG
        printf("Points allocated %d\n",point);
        #endif

    } else {

        if (x < 0) {
            printf ("Bad X value %d\n",x);
        } else {
        
            x_array_buffer[point  ][0] = x;
            x_array_buffer[point++][1] = y;

        }
    
    } 

}

